@@ -24,6 +24,7 @@
 #include <string>
 
 #include "MoveResult.h"
+#include "stackreflist.h"
 
 class AI_Analysis;
 class Player;
@@ -31,8 +32,10 @@ class Ruin;
 class Citylist;
 class Stack;
 class Threat;
-class Stacklist;
+class StackReflist;
+class Threatlist;
 class City;
+class Quest;
 
 using namespace std;
 
@@ -42,14 +45,19 @@ using namespace std;
 class AI_Allocation
 {
     public:
-        AI_Allocation(AI_Analysis *analysis, Player *owner);
+        AI_Allocation(AI_Analysis *analysis, const Threatlist *threats, Player *owner);
         ~AI_Allocation();
 
         // make the player's moves - return the number of stacks which moved.
-        int move();
+        int move(City *first_city, bool build_capacity);
 
+        //! remove the stack from our consideration.
         static void deleteStack(Stack* s);
         static void deleteStack(guint32 id);
+        StackReflist::iterator eraseStack(StackReflist::iterator it);
+
+	//! Emitted whenever anything happens.
+	sigc::signal<void> sbusy;
 
     private:
         /** Assign stacks to defend cities
@@ -63,6 +71,8 @@ class AI_Allocation
           * @return number of stacks moved
           */
         int allocateDefensiveStacks(Citylist *allCities);
+
+        int allocateDefensiveStacksToCity(City *city);
         
         /** Allocate stacks to threats
           * 
@@ -71,7 +81,15 @@ class AI_Allocation
           * @return the number of stacks which moved
           */
         int allocateStacksToThreats();
+
+        int allocateStacksToThreat(Threat *threat);
         
+
+        //! Target neutral cities and empty foreign cities.
+        int allocateStacksToCapacityBuilding(City *first_city, bool take_neutrals);
+
+        int allocateStackToCapacityBuilding(Threat *threat, City *first_city, bool take_neutrals);
+
         // stack return to a safe city
         bool stackReinforce(Stack *s);
         
@@ -83,26 +101,59 @@ class AI_Allocation
 				     Vector<int> diff);
         
         // tell the stack to move to the point
-        MoveResult *moveStack(Stack *stack, Vector<int> pos);
+        //MoveResult *moveStack(Stack *stack, Vector<int> pos);
+
+
+        bool moveStack(Stack *stack, bool &stack_died);
+	bool moveStack(Stack *stack, Vector<int> dest, bool &stack_died);
+
+        bool shuffleStack(Stack *stack, Vector<int> dest, bool split_if_necessary);
+
+        bool groupStacks(Stack *stack);
         
+        void setParked(Stack *stack, bool force_park = false);
+
         // find the best attacker for the given threat
-        Stack *findBestAttackerFor(Threat *threat);
+        Stack *findBestAttackerFor(Threat *threat, guint32 &num_city_defenders);
         
         // find the closest stack to the given position, but 0 if none within
-        // limitInMoves (estimate of a maximum number of turns for getting there) 
-        Stack *findClosestStackToCity(City *city, int limitInMoves);
+        Stack *findClosestStackToCity(City *city);
+
+        Stack *findClosestStackToEnemyCity(City *city, bool try_harder);
         
         // find a position in the city that a stack can move to
         Vector<int> getFreeSpotInCity(City *city, int stackSize);
+
+        // find a ANOTHER position in the city that the stack can move to
+        Vector<int> getFreeOtherSpotInCity(City *city, Stack *stack);
         
         // move stacks that we have no particular use for
         int defaultStackMovements();
+
+        int continueAttacks();
+
+        int continueQuests();
+        bool continueQuest(Quest *quest, Stack *stack);
+
+        int attackNearbyEnemies();
         
+        bool checkAmbiguities();
+
+        bool emptyOutCities();
+
+        int visitTemples(bool get_quests);
+
+        int pickupItems();
+
+        int visitRuins();
+
         static AI_Allocation* s_instance;
         
         Player *d_owner;
         AI_Analysis *d_analysis;
-        Stacklist *d_stacks;
+        StackReflist *d_stacks;
+        const Threatlist *d_threats;
+        bool *abort_turn;
 };
 
 #endif // AI_ALLOCATION_H
