@@ -28,7 +28,6 @@
 
 #include "Configuration.h"
 
-#include "string_tokenizer.h"
 #include "xmlhelper.h"
 #include "defs.h"
 #include "File.h"
@@ -51,20 +50,20 @@ string Configuration::s_dataPath = "./data/";
 string Configuration::s_savePath = "./saves/";
 #endif
 string Configuration::s_lang = "";
-int Configuration::s_displaySpeedDelay = 0;
+int Configuration::s_displaySpeedDelay = 300000;
 int Configuration::s_displayFightRoundDelayFast = 250;
 int Configuration::s_displayFightRoundDelaySlow = 500;
+bool Configuration::s_displayCommentator = true;
 guint32 Configuration::s_cacheSize = 1000000;
 bool Configuration::s_zipfiles = false;
 int Configuration::s_autosave_policy = 1;
-bool Configuration::s_ggz = false;
 bool Configuration::s_musicenable = true;
 guint32 Configuration::s_musicvolume = 64;
 guint32 Configuration::s_musiccache = 10000000;
 string Configuration::s_filename = "";
 bool Configuration::s_see_opponents_stacks = false;
 bool Configuration::s_see_opponents_production = false;
-bool Configuration::s_play_with_quests = true;
+GameParameters::QuestPolicy Configuration::s_play_with_quests = GameParameters::ONE_QUEST_PER_PLAYER;
 bool Configuration::s_hidden_map = false;
 bool Configuration::s_diplomacy = false;
 GameParameters::NeutralCities Configuration::s_neutral_cities = GameParameters::AVERAGE;
@@ -72,7 +71,7 @@ GameParameters::RazingCities Configuration::s_razing_cities = GameParameters::AL
 bool Configuration::s_intense_combat = false;
 bool Configuration::s_military_advisor = false;
 bool Configuration::s_random_turns = false;
-bool Configuration::s_quick_start = false;
+GameParameters::QuickStartPolicy Configuration::s_quick_start = GameParameters::NO_QUICK_START;
 bool Configuration::s_cusp_of_war = false;
 bool Configuration::s_decorated = true;
 bool Configuration::s_remember_recent_games = true;
@@ -133,13 +132,15 @@ bool Configuration::saveConfigurationFile(string filename)
     retval &= helper.saveData("speeddelay", s_displaySpeedDelay);
     retval &= helper.saveData("fightrounddelayfast", s_displayFightRoundDelayFast);
     retval &= helper.saveData("fightrounddelayslow", s_displayFightRoundDelaySlow);
+    retval &= helper.saveData("commentator", s_displayCommentator);
     retval &= helper.saveData("shownextplayer", s_showNextPlayer);
     retval &= helper.saveData("musicenable", s_musicenable);
     retval &= helper.saveData("musicvolume", s_musicvolume);
     retval &= helper.saveData("musiccache", s_musiccache);
     retval &= helper.saveData("view_enemies", s_see_opponents_stacks);
     retval &= helper.saveData("view_production", s_see_opponents_production);
-    retval &= helper.saveData("quests", s_play_with_quests);
+    std::string quest_policy_str = questPolicyToString(GameParameters::QuestPolicy(s_play_with_quests));
+    retval &= helper.saveData("quests", quest_policy_str);
     retval &= helper.saveData("hidden_map", s_hidden_map);
     retval &= helper.saveData("diplomacy", s_diplomacy);
     std::string neutral_cities_str = neutralCitiesToString(GameParameters::NeutralCities(s_neutral_cities));
@@ -149,7 +150,8 @@ bool Configuration::saveConfigurationFile(string filename)
     retval &= helper.saveData("intense_combat", s_intense_combat);
     retval &= helper.saveData("military_advisor", s_military_advisor);
     retval &= helper.saveData("random_turns", s_random_turns);
-    retval &= helper.saveData("quick_start", s_quick_start);
+    std::string quick_start_str = quickStartPolicyToString(GameParameters::QuickStartPolicy(s_quick_start));
+    retval &= helper.saveData("quick_start", quick_start_str);
     retval &= helper.saveData("cusp_of_war", s_cusp_of_war);
     retval &= helper.saveData("decorated", s_decorated);
     retval &= helper.saveData("remember_recent_games", s_remember_recent_games);
@@ -246,6 +248,9 @@ bool Configuration::parseConfiguration(string tag, XML_Helper* helper)
     helper->getData(s_displayFightRoundDelayFast, "fightrounddelayfast");
     helper->getData(s_displayFightRoundDelaySlow, "fightrounddelayslow");
 
+    //parse whether or not the commentator should be shown
+    helper->getData(s_displayCommentator, "commentator");
+
     //parse if nextplayer dialog should be enabled
     helper->getData(s_showNextPlayer, "shownextplayer");
 
@@ -256,7 +261,9 @@ bool Configuration::parseConfiguration(string tag, XML_Helper* helper)
     
     helper->getData(s_see_opponents_stacks, "view_enemies");
     helper->getData(s_see_opponents_production, "view_production");
-    helper->getData(s_play_with_quests, "quests");
+    std::string quest_policy_str;
+    helper->getData(quest_policy_str, "quests");
+    s_play_with_quests = questPolicyFromString(quest_policy_str);
     helper->getData(s_hidden_map, "hidden_map");
     helper->getData(s_diplomacy, "diplomacy");
     std::string neutral_cities_str;
@@ -268,7 +275,9 @@ bool Configuration::parseConfiguration(string tag, XML_Helper* helper)
     helper->getData(s_intense_combat, "intense_combat");
     helper->getData(s_military_advisor, "military_advisor");
     helper->getData(s_random_turns, "random_turns");
-    helper->getData(s_quick_start, "quick_start");
+    std::string quick_start_str;
+    helper->getData(quick_start_str, "quick_start");
+    s_quick_start = quickStartPolicyFromString(quick_start_str);
     helper->getData(s_cusp_of_war, "cusp_of_war");
     helper->getData(s_decorated, "decorated");
     helper->getData(s_remember_recent_games, "remember_recent_games");
@@ -445,3 +454,66 @@ Configuration::SavingPolicy Configuration::savingPolicyFromString(std::string st
     
   return Configuration::WRITE_NUMBERED_AUTOSAVE_FILE;
 }
+
+std::string Configuration::quickStartPolicyToString(const GameParameters::QuickStartPolicy policy)
+{
+  switch (policy)
+    {
+    case GameParameters::NO_QUICK_START:
+      return "GameParameters::NO_QUICK_START";
+      break;
+    case GameParameters::EVENLY_DIVIDED:
+      return "GameParameters::EVENLY_DIVIDED";
+      break;
+    case GameParameters::AI_HEAD_START:
+      return "GameParameters::AI_HEAD_START";
+      break;
+    }
+  return "GameParameters::NO_QUICK_START";
+}
+
+GameParameters::QuickStartPolicy Configuration::quickStartPolicyFromString(std::string str)
+{
+  if (str.size() > 0 && isdigit(str.c_str()[0]))
+    return GameParameters::QuickStartPolicy(atoi(str.c_str()));
+  if (str == "GameParameters::NO_QUICK_START")
+    return GameParameters::NO_QUICK_START;
+  else if (str == "GameParameters::EVENLY_DIVIDED")
+    return GameParameters::EVENLY_DIVIDED;
+  else if (str == "GameParameters::AI_HEAD_START")
+    return GameParameters::AI_HEAD_START;
+    
+  return GameParameters::NO_QUICK_START;
+}
+
+std::string Configuration::questPolicyToString(const GameParameters::QuestPolicy quest)
+{
+  switch (quest)
+    {
+      case GameParameters::NO_QUESTING:
+	return "GameParameters::NO_QUESTING";
+	break;
+      case GameParameters::ONE_QUEST_PER_PLAYER:
+	return "GameParameters::ONE_QUEST_PER_PLAYER";
+	break;
+      case GameParameters::ONE_QUEST_PER_HERO:
+	return "GameParameters::ONE_QUEST_PER_HERO";
+	break;
+    }
+  return "GameParameters::NO_QUESTING";
+}
+
+GameParameters::QuestPolicy Configuration::questPolicyFromString(std::string str)
+{
+  if (str.size() > 0 && isdigit(str.c_str()[0]))
+    return GameParameters::QuestPolicy(atoi(str.c_str()));
+  if (str == "GameParameters::NO_QUESTING")
+    return GameParameters::NO_QUESTING;
+  else if (str == "GameParameters::ONE_QUEST_PER_PLAYER")
+    return GameParameters::ONE_QUEST_PER_PLAYER;
+  else if (str == "GameParameters::ONE_QUEST_PER_HERO")
+    return GameParameters::ONE_QUEST_PER_HERO;
+    
+  return GameParameters::NO_QUESTING;
+}
+
