@@ -48,7 +48,7 @@
 #include "roadlist.h"
 #include "road.h"
 #include "armysetlist.h"
-
+#include "citysetlist.h"
 #include "real_player.h"
 #include "ai_fast.h"
 #include "ai_smart.h"
@@ -87,6 +87,9 @@ CreateScenario::CreateScenario(int width, int height)
 
     setWidth(width);
     setHeight(height);
+
+    d_generator = new MapGenerator();
+    d_generator->progress.connect (sigc::mem_fun(*this, &CreateScenario::on_progress));
 }
 
 CreateScenario::~CreateScenario()
@@ -100,21 +103,6 @@ CreateScenario::~CreateScenario()
         delete d_scenario;
 }
 
-void CreateScenario::setMaptype(MapType type)
-{
-    debug("CreateScenario::setMaptype")
-
-    //this function currently does almost nothing. It's purpose is to initialize
-    //the right MapGenerator (e.g. islands, mountain chains, normal etc.).
-    //Currently we have only a normal d_generator, so it is initialized here
-
-    if (d_generator)
-        delete d_generator;
-
-    d_generator = new MapGenerator();
-    d_generator->progress.connect (sigc::mem_fun(*this, &CreateScenario::on_progress));
-}
-    
 void CreateScenario::on_progress(double percent, std::string description)
 {
   progress.emit();
@@ -125,9 +113,6 @@ void CreateScenario::setPercentages(int pgrass, int pwater, int pforest,
 {
     debug("CreateScenario::setPercentages")
 
-    if (!d_generator)
-        setMaptype(NORMAL);
-
     //handle input with !=100% sum
     int sum = pgrass + pwater + pforest + pswamp +phills + pmountains;
 
@@ -153,25 +138,23 @@ void CreateScenario::setMapTiles(std::string tilesname)
     d_tilesname = tilesname;
 }
 
-void CreateScenario::setShieldset(std::string shieldsname)
+void CreateScenario::setShieldset(std::string shieldset)
 {
     debug("CreateScenario::setShieldset")
-    d_shieldsname = shieldsname;
+    d_shieldsname = shieldset;
 }
 
 void CreateScenario::setCityset(std::string citysetname)
 {
     debug("CreateScenario::setCityset")
     d_citysetname = citysetname;
+    d_generator->setCityset(Citysetlist::getInstance()->getCityset(citysetname));
 }
 
 void CreateScenario::setNoCities(int nocities)
 {
     debug("CreateScenario::setNoCities")
 
-    if (!d_generator)
-        setMaptype(NORMAL);
-
     d_generator->setNoCities(nocities);
 }
 
@@ -179,9 +162,6 @@ void CreateScenario::setNoRuins(int noruins)
 {
     debug("CreateScenario::setNoRuins")
 
-    if (!d_generator)
-        setMaptype(NORMAL);
-
     d_generator->setNoRuins(noruins);
 }
 
@@ -189,9 +169,6 @@ void CreateScenario::setNoSignposts (int nosignposts)
 {
     debug("CreateScenario::setNoSignposts")
 
-    if (!d_generator)
-        setMaptype(NORMAL);
-
     d_generator->setNoSignposts(nosignposts);
 }
 
@@ -199,9 +176,6 @@ void CreateScenario::setNoTemples(int notemples)
 {
     debug("CreateScenario::setNoTemples")
 
-    if (!d_generator)
-        setMaptype(NORMAL);
-
     d_generator->setNoTemples(notemples);
 }
 
@@ -243,7 +217,7 @@ Player* CreateScenario::addPlayer(std::string name, guint32 armyset,
 
     Player* p = Player::create(name, armyset, color, d_width, d_height,
 			       Player::Type(type));
-    Playerlist::getInstance()->push_back(p);
+    Playerlist::getInstance()->add(p);
 
     return p;
 }
@@ -303,12 +277,6 @@ int CreateScenario::getNoTemples() const
     return d_generator->getNoTemples();
 }
 
-std::string CreateScenario::getMapTiles() const
-{
-    debug("CreateScenario::getMapTiles")
-    return d_tilesname;
-}
-
 bool CreateScenario::create(const GameParameters &g)
 {
     debug("CreateScenario::create")
@@ -347,7 +315,7 @@ bool CreateScenario::create(const GameParameters &g)
 
     getRuinDifficulty (g.difficulty, &sage_factor, &no_guardian_factor,
 		       &stronghold_factor);
-    if (!setupRuins(GameScenario::s_play_with_quests, 20, 10, 6))
+    if (!setupRuins(GameScenario::s_play_with_quests != GameParameters::NO_QUESTING, 20, 10, 6))
         return false;
 
     int base_gold;
@@ -407,6 +375,7 @@ bool CreateScenario::createMap()
 
     //...and create cities, temples, ruins ,signposts
     map = d_generator->getBuildings(d_width, d_height);
+    Cityset *cityset = Citysetlist::getInstance()->getCityset(d_citysetname);
     
     for (int y = 0; y < d_height; y++)
         for (int x = 0; x < d_width; x++)
@@ -418,14 +387,19 @@ bool CreateScenario::createMap()
                     break;
                 case Maptile::TEMPLE:
                     Templelist::getInstance()->add
-		      (new Temple(Vector<int>(x,y), popRandomTempleName()));
+		      (new Temple(Vector<int>(x,y), 
+				  cityset->getTempleTileWidth(),
+				  popRandomTempleName()));
                     break;
                 case Maptile::RUIN:
 		    Ruinlist::getInstance()->add
-		      (new Ruin(Vector<int>(x,y), popRandomRuinName()));
+		      (new Ruin(Vector<int>(x,y), 
+				cityset->getRuinTileWidth(),
+				popRandomRuinName()));
 		    break;
                 case Maptile::CITY:
-                    Citylist::getInstance()->add(new City(Vector<int>(x,y)));
+                    Citylist::getInstance()->add
+		      (new City(Vector<int>(x,y), cityset->getCityTileWidth()));
                     break;
                 case Maptile::ROAD:
                     Roadlist::getInstance()->add(new Road(Vector<int>(x,y)));
@@ -535,8 +509,6 @@ bool CreateScenario::setupCities(bool cities_can_produce_allies,
         c->setRandomArmytypes(cities_can_produce_allies, 
 			      number_of_armies_factor);
 
-	if (rand() % 2 == 0)
-	  c->raiseDefense();
 	c->setGold(getRandomCityIncome(c->isCapital()));
     }
 
@@ -667,7 +639,7 @@ bool CreateScenario::setupPlayers(bool random_turns,
 
     // Give players some gold to start with
     for (Playerlist::iterator pit = pl->begin(); pit != pl->end(); pit++)
-      (*pit)->setGold(base_gold + ((rand() % 8) * 50));
+      (*pit)->setGold(adjustBaseGold(base_gold));
 
 
     if (random_turns)
@@ -759,22 +731,6 @@ void CreateScenario::getCityDifficulty(int difficulty,
     *number_of_armies_factor = 0;
 }
     
-void CreateScenario::getBaseGold (int difficulty, int *base_gold)
-{
-  if (difficulty < 50)
-    *base_gold = 1600;
-  else if (difficulty < 60)
-    *base_gold = 1200;
-  else if (difficulty < 70)
-    *base_gold = 1000;
-  else if (difficulty < 80)
-    *base_gold = 900;
-  else if (difficulty < 90)
-    *base_gold = 800;
-  else
-    *base_gold = 700;
-}
-
 int CreateScenario::calculateRoadType (Vector<int> t)
 {
     Roadlist *rl = Roadlist::getInstance();
@@ -842,3 +798,9 @@ int CreateScenario::calculateRoadType (Vector<int> t)
 	type = Road::CONNECTS_EAST;
     return type;
 }
+
+int CreateScenario::calculateNumberOfSignposts(int width, int height, int grass)
+{
+  int area = width * height;
+  return int(area * (grass / 100.0) * SIGNPOST_FREQUENCY);
+}
