@@ -45,6 +45,7 @@
 #include "File.h"
 #include "armysetlist.h"
 #include "tilesetlist.h"
+#include "citysetlist.h"
 #include "shieldsetlist.h"
 #include "stacklist.h"
 #include "stack.h"
@@ -58,11 +59,13 @@
 #include "army.h"
 #include "QuestsManager.h"
 #include "Itemlist.h"
-#include "string_tokenizer.h"
 #include "player.h"
 #include "vectoredunitlist.h"
 #include "history.h"
 #include "xmlhelper.h"
+#include "tarhelper.h"
+#include "stacktile.h"
+#include "GraphicsCache.h"
 
 std::string GameScenario::d_tag = "scenario";
 using namespace std;
@@ -72,8 +75,8 @@ using namespace std;
 
 GameScenario::GameScenario(std::string name,std::string comment, bool turnmode,
 			   GameScenario::PlayMode playmode)
-    :d_name(name),d_comment(comment), d_turnmode(turnmode), 
-    d_playmode(playmode), recording_file(""), inhibit_autosave_removal(false)
+    :d_name(name),d_comment(comment), d_copyright(""), d_license(""),
+    d_turnmode(turnmode), d_playmode(playmode), inhibit_autosave_removal(false)
 {
     Armysetlist::getInstance();
     Tilesetlist::getInstance();
@@ -85,25 +88,144 @@ GameScenario::GameScenario(std::string name,std::string comment, bool turnmode,
     setNewRandomId();
 }
 
-// savegame is a filename with absolute path!
-
+//savegame has an absolute path
 GameScenario::GameScenario(string savegame, bool& broken)
-  :d_turnmode(true), d_playmode(GameScenario::HOTSEAT), recording_file(""), 
+  :d_turnmode(true), d_playmode(GameScenario::HOTSEAT), 
     inhibit_autosave_removal(false)
 {
-  XML_Helper helper(savegame, ios::in, Configuration::s_zipfiles);
+  Tar_Helper t(savegame, std::ios::in);
+  loadArmysets(&t);
+  loadTilesets(&t);
+  loadCitysets(&t);
+  loadShieldsets(&t);
+  std::string filename = t.getFirstFile(broken);
+  XML_Helper helper(filename, std::ios::in, Configuration::s_zipfiles);
   broken = loadWithHelper(helper);
+  File::erase(filename);
   helper.close();
+  t.Close();
+}
+
+bool GameScenario::loadArmysets(Tar_Helper *t)
+{
+  bool broken = false;
+  std::list<std::string> armysets;
+  armysets = t->getFilenamesWithExtension(Armyset::file_extension);
+  for (std::list<std::string>::iterator it = armysets.begin(); 
+       it != armysets.end(); it++)
+    {
+      Armyset *armyset = Armysetlist::getInstance()->import(t, *it, broken);
+      if (armyset)
+        Armysetlist::getInstance()->getArmyset(armyset->getId())->instantiateImages();
+    }
+  return !broken;
+}
+
+bool GameScenario::loadTilesets(Tar_Helper *t)
+{
+  bool broken = false;
+  std::list<std::string> tilesets;
+  tilesets = t->getFilenamesWithExtension(Tileset::file_extension);
+  for (std::list<std::string>::iterator it = tilesets.begin(); 
+       it != tilesets.end(); it++)
+    {
+      std::string filename = t->getFile(*it, broken);
+      Tileset *tileset = Tileset::create(filename);
+      tileset->setSubDir(File::get_basename(*it));
+
+      //extract all the files and remember where we extracted them
+      std::list<std::string> delfiles;
+      delfiles.push_back(filename);
+      std::list<std::string> files;
+      tileset->getFilenames(files);
+      for (std::list<std::string>::iterator i = files.begin(); i != files.end(); i++)
+	delfiles.push_back (t->getFile(*i + ".png", broken));
+
+      std::string subdir = "";
+      guint32 id = 0;
+      Tilesetlist::getInstance()->addToPersonalCollection(tileset, subdir, id);
+
+      for (std::list<std::string>::iterator it = delfiles.begin(); it != delfiles.end(); it++)
+	File::erase(*it);
+  
+      Tilesetlist::getInstance()->getTileset(tileset->getId())->instantiateImages();
+    }
+  return !broken;
+}
+
+bool GameScenario::loadCitysets(Tar_Helper *t)
+{
+  bool broken = false;
+  std::list<std::string> citysets;
+  citysets = t->getFilenamesWithExtension(Cityset::file_extension);
+  for (std::list<std::string>::iterator it = citysets.begin(); 
+       it != citysets.end(); it++)
+    {
+      std::string filename = t->getFile(*it, broken);
+      Cityset *cityset = Cityset::create(filename);
+      cityset->setSubDir(File::get_basename(*it));
+
+      //extract all the files and remember where we extracted them
+      std::list<std::string> delfiles;
+      delfiles.push_back(filename);
+      std::list<std::string> files;
+      cityset->getFilenames(files);
+      for (std::list<std::string>::iterator i = files.begin(); i != files.end(); i++)
+	delfiles.push_back (t->getFile(*i + ".png", broken));
+
+      std::string subdir = "";
+      guint32 id = 0;
+      Citysetlist::getInstance()->addToPersonalCollection(cityset, subdir, id);
+
+      for (std::list<std::string>::iterator it = delfiles.begin(); it != delfiles.end(); it++)
+	File::erase(*it);
+  
+      Citysetlist::getInstance()->getCityset(cityset->getId())->instantiateImages();
+    }
+  return !broken;
+}
+
+bool GameScenario::loadShieldsets(Tar_Helper *t)
+{
+  bool broken = false;
+  std::list<std::string> shieldsets;
+  shieldsets = t->getFilenamesWithExtension(Shieldset::file_extension);
+  for (std::list<std::string>::iterator it = shieldsets.begin(); 
+       it != shieldsets.end(); it++)
+    {
+      std::string filename = t->getFile(*it, broken);
+      Shieldset *shieldset = Shieldset::create(filename);
+      shieldset->setSubDir(File::get_basename(*it));
+
+      //extract all the files and remember where we extracted them
+      std::list<std::string> delfiles;
+      delfiles.push_back(filename);
+      std::list<std::string> files;
+      shieldset->getFilenames(files);
+      for (std::list<std::string>::iterator i = files.begin(); i != files.end(); i++)
+	delfiles.push_back (t->getFile(*i + ".png", broken));
+
+      std::string subdir = "";
+      guint32 id = 0;
+      Shieldsetlist::getInstance()->addToPersonalCollection(shieldset, subdir, id);
+
+      for (std::list<std::string>::iterator it = delfiles.begin(); it != delfiles.end(); it++)
+	File::erase(*it);
+  
+
+      Shieldsetlist::getInstance()->getShieldset(shieldset->getId())->instantiateImages();
+    }
+  return !broken;
 }
 
 GameScenario::GameScenario(XML_Helper &helper, bool& broken)
-  : d_turnmode(true), d_playmode(GameScenario::HOTSEAT), recording_file(""),
+  : d_turnmode(true), d_playmode(GameScenario::HOTSEAT),
     inhibit_autosave_removal(false)
 {
   broken = loadWithHelper(helper);
 }
 
-void GameScenario::quickStart()
+void GameScenario::quickStartEvenlyDivided()
 {
   Playerlist *plist = Playerlist::getInstance();
   Citylist *clist = Citylist::getInstance();
@@ -139,10 +261,81 @@ void GameScenario::quickStart()
 	    continue;
 	  pos = clist->getFirstCity(p)->getPos();
 	  City *c = clist->getNearestNeutralCity(pos);
-	  c->conquer(p);
-	  History_CityWon *item = new History_CityWon();
-	  item->fillData(c);
-	  p->addHistory(item);
+	  if (c)
+	    {
+	      //does the city contain any stacks yet?
+	      //change their allegience to us.
+	      for (unsigned int x = 0 ; x < c->getSize(); x++)
+		{
+		  for (unsigned int y = 0; y < c->getSize(); y++)
+		    {
+		      StackTile *stile = 
+			GameMap::getStacks(c->getPos() + Vector<int>(x,y));
+		      std::list<Stack*> stks = stile->getStacks();
+		      for (std::list<Stack *>::iterator i = stks.begin(); 
+			   i != stks.end(); i++)
+			Stacklist::changeOwnership(*i, p);
+		    }
+		}
+
+	      //now give the city to us.
+	      c->conquer(p);
+	      History_CityWon *item = new History_CityWon();
+	      item->fillData(c);
+	      p->addHistory(item);
+	    }
+	}
+    }
+}
+
+void GameScenario::quickStartAIHeadStart()
+{
+  float head_start_factor = 0.05;
+  //each AI player gets this percent of total cities.
+
+  Playerlist *plist = Playerlist::getInstance();
+  Citylist *clist = Citylist::getInstance();
+  Vector <int> pos;
+
+  unsigned int citycount = clist->size() * head_start_factor;
+  if (citycount == 0)
+    citycount = 1;
+  for (unsigned int i = 0; i < MAX_PLAYERS; i++)
+    {
+      for (unsigned int j = 0; j < citycount; j++)
+	{
+	  Player *p = plist->getPlayer(i);
+	  if (!p)
+	    continue;
+	  if (p == plist->getNeutral())
+	    continue;
+	  if (p->getType() == Player::HUMAN)
+	    continue;
+	  pos = clist->getFirstCity(p)->getPos();
+	  City *c = clist->getNearestNeutralCity(pos);
+	  if (c)
+	    {
+	      //does the city contain any stacks yet?
+	      //change their allegience to us.
+	      for (unsigned int x = 0 ; x < c->getSize(); x++)
+		{
+		  for (unsigned int y = 0; y < c->getSize(); y++)
+		    {
+		      StackTile *stile = 
+			GameMap::getStacks(c->getPos() + Vector<int>(x,y));
+		      std::list<Stack*> stks = stile->getStacks();
+		      for (std::list<Stack *>::iterator i = stks.begin(); 
+			   i != stks.end(); i++)
+			Stacklist::changeOwnership(*i, p);
+		    }
+		}
+
+	      //now give the city to us.
+	      c->conquer(p);
+	      History_CityWon *item = new History_CityWon();
+	      item->fillData(c);
+	      p->addHistory(item);
+	    }
 	}
     }
 }
@@ -264,7 +457,7 @@ bool GameScenario::setupRewards(bool hidden_map)
   return true;
 }
 
-bool GameScenario::setupCities(bool quick_start)
+bool GameScenario::setupCities(GameParameters::QuickStartPolicy quick_start)
 {
   debug("GameScenario::setupCities")
 
@@ -283,8 +476,10 @@ bool GameScenario::setupCities(bool quick_start)
 	}
     }
 
-  if (quick_start)
-    quickStart();
+  if (quick_start == GameParameters::EVENLY_DIVIDED)
+    quickStartEvenlyDivided();
+  else if (quick_start == GameParameters::AI_HEAD_START)
+    quickStartAIHeadStart();
 
   for (Citylist::iterator it = Citylist::getInstance()->begin();
        it != Citylist::getInstance()->end(); it++)
@@ -383,6 +578,7 @@ bool GameScenario::loadWithHelper(XML_Helper& helper)
   if (!helper.parse())
     broken = true;
 
+  GameMap::getInstance()->updateStackPositions();
   GameMap::getInstance()->calculateBlockedAvenues();
 
   return broken;
@@ -415,7 +611,7 @@ GameScenario::~GameScenario()
       inhibit_autosave_removal == false)
     {
       std::string filename = File::getSavePath() + "autosave.sav";
-      remove(filename.c_str());
+      File::erase(filename);
     }
   GameScenarioOptions::s_round = 0;
 } 
@@ -441,30 +637,71 @@ bool GameScenario::saveGame(string filename, string extension) const
   bool retval = true;
   string goodfilename=filename;
 
-  stringTokenizer * strtoken= new stringTokenizer(filename,"/.\\ ");
+  if (File::nameEndsWith(filename, extension) == false)
+    goodfilename += "." + extension;
 
-  if (strtoken->getLastToken() == extension)
-    {
-      debug("The Filename is well formed")
-	std::cerr <<"";  //dummy call if debug statement is commented out
-    }
-  else 
-    {
-      debug("The Filename lacks the extension --> " << extension)
-	goodfilename += "." + extension;
-    }
-
-  delete strtoken;
-
-  XML_Helper helper(goodfilename, ios::out, Configuration::s_zipfiles);
+  std::string tmpfile = "lw.XXXX";
+  int fd = Glib::file_open_tmp(tmpfile, "lw.XXXX");
+  close(fd);
+  XML_Helper helper(tmpfile, ios::out, Configuration::s_zipfiles);
   retval &= saveWithHelper(helper);
   helper.close();
 
-  if (retval)
-    return true;
+  if (retval == false)
+    return false;
+
+  Tar_Helper t(goodfilename, std::ios::out);
+  t.saveFile(tmpfile, File::get_basename(goodfilename, true));
+  File::erase(tmpfile);
+  std::list<std::string> files;
+
+  Cityset *cs = GameMap::getInstance()->getCityset();
+  cs->getFilenames(files);
+  t.saveFile(cs->getConfigurationFile());
+  for (std::list<std::string>::iterator it = files.begin(); it !=files.end();
+       it++)
+    t.saveFile(cs->getFile(*it));
+
+  files.clear();
+  Shieldset *ss = GameMap::getInstance()->getShieldset();
+  ss->getFilenames(files);
+  t.saveFile(ss->getConfigurationFile());
+  for (std::list<std::string>::iterator it = files.begin(); it !=files.end();
+       it++)
+    t.saveFile(ss->getFile(*it));
+
+  files.clear();
+  Tileset *ts = GameMap::getInstance()->getTileset();
+  ts->getFilenames(files);
+  t.saveFile(ts->getConfigurationFile());
+  for (std::list<std::string>::iterator it = files.begin(); it !=files.end();
+       it++)
+    t.saveFile(ts->getFile(*it));
+
+  Playerlist *plist = Playerlist::getInstance();
+  std::list<guint32> armysets;
+  for (Playerlist::iterator it = plist->begin(); it != plist->end(); it++)
+    {
+      guint32 armyset = (*it)->getArmyset();
+      if (std::find(armysets.begin(), armysets.end(), armyset) == 
+	  armysets.end())
+	armysets.push_back(armyset);
+    }
+  for (std::list<guint32>::iterator it = armysets.begin(); it!= armysets.end();
+       it++)
+    {
+      files.clear();
+      Armyset *as = Armysetlist::getInstance()->getArmyset(*it);
+      t.saveFile(as->getConfigurationFile());
+      as->getFilenames(files);
+      for (std::list<std::string>::iterator i = files.begin(); 
+	   i != files.end(); i++)
+	t.saveFile(as->getFile(*i));
+    }
+
+  t.Close();
+  return true;
 
-  std::cerr << "GameScenario: Something went wrong with saving.\n";
-  return false;
 }
 
 bool GameScenario::saveWithHelper(XML_Helper &helper) const
@@ -497,11 +734,14 @@ bool GameScenario::saveWithHelper(XML_Helper &helper) const
   retval &= helper.saveData("id", d_id);
   retval &= helper.saveData("name", d_name);
   retval &= helper.saveData("comment", d_comment);
+  retval &= helper.saveData("copyright", d_copyright);
+  retval &= helper.saveData("license", d_license);
   retval &= helper.saveData("turn", s_round);
   retval &= helper.saveData("turnmode", d_turnmode);
   retval &= helper.saveData("view_enemies", s_see_opponents_stacks);
   retval &= helper.saveData("view_production", s_see_opponents_production);
-  retval &= helper.saveData("quests", s_play_with_quests);
+  std::string quest_policy_str = Configuration::questPolicyToString(GameParameters::QuestPolicy(s_play_with_quests));
+  retval &= helper.saveData("quests", quest_policy_str);
   retval &= helper.saveData("hidden_map", s_hidden_map);
   retval &= helper.saveData("diplomacy", s_diplomacy);
   retval &= helper.saveData("cusp_of_war", s_cusp_of_war);
@@ -542,10 +782,14 @@ bool GameScenario::load(std::string tag, XML_Helper* helper)
       helper->getData(d_turnmode, "turnmode");
       helper->getData(d_name, "name");
       helper->getData(d_comment, "comment");
+      helper->getData(d_copyright, "copyright");
+      helper->getData(d_license, "license");
       helper->getData(s_round, "turn");
       helper->getData(s_see_opponents_stacks, "view_enemies");
       helper->getData(s_see_opponents_production, "view_production");
-      helper->getData(s_play_with_quests, "quests");
+      std::string quest_policy_str;
+      helper->getData(quest_policy_str, "quests");
+      s_play_with_quests = Configuration::questPolicyFromString(quest_policy_str);
       helper->getData(s_hidden_map, "hidden_map");
       helper->getData(s_diplomacy, "diplomacy");
       helper->getData(s_cusp_of_war, "cusp_of_war");
@@ -702,23 +946,7 @@ bool GameScenario::autoSave()
 void GameScenario::nextRound()
 {
   s_round++;
-
-  if (recording_file != "")
-    {
-      XML_Helper helper(recording_file, std::ios::app, 
-			Configuration::s_zipfiles);
-      //what do i do now?
-      Playerlist *pl = Playerlist::getInstance();
-      for (Playerlist::iterator it = pl->begin(); it != pl->end(); it++)
-	{
-	  (*it)->saveNetworkActions(&helper);
-	  //somehow dump the action list of each player, while 
-	  //appending to recording file.
-	}
-      helper.close();
-    }
   autoSave();
-
 }
 
 std::string GameScenario::playModeToString(const GameScenario::PlayMode mode)
@@ -839,6 +1067,10 @@ bool GameScenario::validate(std::list<std::string> &errors, std::list<std::strin
       warnings.push_back(s);
     }
 
+      
+  if (GameMap::getInstance()->checkCityAccessibility() == false)
+    errors.push_back(_("Not all cities are reachable by a non-flying unit."));
+
   if (errors.size() ==  0)
     return true;
   return false;
@@ -858,11 +1090,36 @@ void GameScenario::initialize(GameParameters g)
     Playerlist::getInstance()->turnHumansIntoNetworkPlayers();
   else
     autoSave();
+  GameMap::getInstance()->updateStackPositions();
+
+  if (d_name == "AutoGenerated")
+    {
+      if (GameMap::getInstance()->checkCityAccessibility() == false)
+	exit (0);
+    }
 }
 
 class ParamLoader
 {
 public:
+    ParamLoader(std::string filename, bool &broken) {
+      Tar_Helper t(filename, std::ios::in);
+      std::string tmpfile = t.getFirstFile(broken);
+      XML_Helper helper(tmpfile, std::ios::in, Configuration::s_zipfiles);
+      helper.registerTag(GameMap::d_tag, 
+			 sigc::mem_fun(this, &ParamLoader::loadParam));
+      helper.registerTag(GameScenario::d_tag, 
+			 sigc::mem_fun(this, &ParamLoader::loadParam));
+      helper.registerTag(Playerlist::d_tag, 
+			 sigc::mem_fun(this, &ParamLoader::loadParam));
+      helper.registerTag(Player::d_tag, 
+			 sigc::mem_fun(this, &ParamLoader::loadParam));
+      bool retval = helper.parse();
+      File::erase(tmpfile);
+      if (broken == false)
+	broken = !retval;
+      helper.close();
+    }
     bool loadParam(std::string tag, XML_Helper* helper)
       {
 	if (tag == Playerlist::d_tag)
@@ -925,7 +1182,10 @@ public:
 			    "view_enemies");
 	    helper->getData(game_params.see_opponents_production, 
 			    "view_production");
-	    helper->getData(game_params.play_with_quests, "quests");
+	    std::string quest_policy_str;
+	    helper->getData(quest_policy_str, "quests");
+	    game_params.play_with_quests = 
+	      Configuration::questPolicyFromString(quest_policy_str);
 	    helper->getData(game_params.hidden_map, "hidden_map");
 	    helper->getData(game_params.diplomacy, "diplomacy");
 	    helper->getData(game_params.cusp_of_war, "cusp_of_war");
@@ -951,26 +1211,28 @@ public:
 };
 GameParameters GameScenario::loadGameParameters(std::string filename, bool &broken)
 {
-  ParamLoader loader;
+  ParamLoader loader(filename, broken);
   
-  XML_Helper helper(filename, std::ios::in, Configuration::s_zipfiles);
-  helper.registerTag(GameMap::d_tag, 
-		     sigc::mem_fun(loader, &ParamLoader::loadParam));
-  helper.registerTag(GameScenario::d_tag, 
-		     sigc::mem_fun(loader, &ParamLoader::loadParam));
-  helper.registerTag(Playerlist::d_tag, 
-		     sigc::mem_fun(loader, &ParamLoader::loadParam));
-  helper.registerTag(Player::d_tag, 
-		     sigc::mem_fun(loader, &ParamLoader::loadParam));
-  bool retval = helper.parse();
-
-  broken = !retval;
-  helper.close();
   return loader.game_params;
 }
+
 class PlayModeLoader
 {
 public:
+    PlayModeLoader(std::string filename, bool broken) {
+      play_mode = GameScenario::HOTSEAT;
+      Tar_Helper t(filename, std::ios::in);
+      std::string file = File::get_basename(filename, true);
+      std::string tmpfile = t.getFirstFile(broken);
+      XML_Helper helper(tmpfile, std::ios::in, Configuration::s_zipfiles);
+      helper.registerTag(GameScenario::d_tag, 
+			 sigc::mem_fun(this, &PlayModeLoader::loadParam));
+      bool retval = helper.parse();
+      File::erase(tmpfile);
+      if (broken == false)
+	broken = !retval;
+      helper.close();
+    }
     bool loadParam(std::string tag, XML_Helper* helper)
       {
 	if (tag == GameScenario::d_tag)
@@ -987,24 +1249,64 @@ public:
 
 GameScenario::PlayMode GameScenario::loadPlayMode(std::string filename, bool &broken)
 {
-  PlayModeLoader loader;
-  
-  XML_Helper helper(filename, std::ios::in, Configuration::s_zipfiles);
-  helper.registerTag(GameScenario::d_tag, 
-		     sigc::mem_fun(loader, &PlayModeLoader::loadParam));
-  bool retval = helper.parse();
-
-  broken = !retval;
-  helper.close();
+  PlayModeLoader loader(filename, broken);
   return loader.play_mode;
 }
 
-void GameScenario::startRecordingEventsToFile(std::string filename)
+class DetailsLoader
 {
-  recording_file = filename;
+public:
+    DetailsLoader(std::string filename, bool &broken) {
+      player_count = 0; city_count = 0; name = ""; comment = "";
+      Tar_Helper t(filename, std::ios::in);
+      std::string tmpfile = t.getFirstFile(broken);
+      XML_Helper helper(tmpfile, std::ios::in, Configuration::s_zipfiles);
+      helper.registerTag(GameScenario::d_tag, 
+			 sigc::mem_fun(this, &DetailsLoader::loadDetails));
+      helper.registerTag(Player::d_tag, 
+			 sigc::mem_fun(this, &DetailsLoader::loadDetails));
+      helper.registerTag(City::d_tag, 
+		     sigc::mem_fun(this, &DetailsLoader::loadDetails));
+      bool retval = helper.parse();
+      if (!broken)
+	broken = !retval;
+      File::erase(tmpfile);
+    }
+
+    bool loadDetails(std::string tag, XML_Helper* helper)
+      {
+	if (tag == GameScenario::d_tag)
+	  {
+	    helper->getData(name, "name");
+	    helper->getData(comment, "comment");
+	    return true;
+	  }
+	if (tag == Player::d_tag)
+	  {
+	    player_count++;
+	    return true;
+	  }
+	if (tag == City::d_tag)
+	  {
+	    city_count++;
+	    return true;
+	  }
+	return false;
+      };
+    Tar_Helper *t;
+    std::string name, comment;
+    guint32 player_count, city_count;
+};
+void GameScenario::loadDetails(std::string filename, bool &broken, guint32 &player_count, guint32 &city_count, std::string &name, std::string &comment)
+{
+  DetailsLoader loader(filename, broken);
+  if (broken == false)
+    {
+      player_count = loader.player_count;
+      city_count = loader.city_count;
+      name = loader.name;
+      comment = loader.comment;
+    }
+  return;
 }
 
-void GameScenario::stopRecordingEventsToFile()
-{
-  recording_file = "";
-}
