@@ -35,6 +35,7 @@
 #include "Configuration.h"
 
 class XML_Helper;
+class Tar_Helper;
 
 //! A class to hold several scenario options.
 /** 
@@ -101,6 +102,12 @@ class GameScenario: public GameScenarioOptions
         //! Returns the comment for the scenario.
         std::string getComment(bool translate = true) const;
 
+        //! Returns the copyright for the scenario.
+        std::string getCopyright() const {return d_copyright; };
+
+        //! Returns the license of the scenario.
+        std::string getLicense() const {return d_license;};
+
         //! Increments the turn number and does an autosave. Called by NextTurn
         //! via a signal.
         void nextRound();
@@ -108,9 +115,15 @@ class GameScenario: public GameScenarioOptions
         //! Sets the name of the scenario.
         void setName(std::string name) {d_name = name;}
 
-        //! Sets the name of the scenario.
+        //! Sets the description of the scenario.
         void setComment(std::string comment) {d_comment = comment;}
         
+        //! Sets the copyright of the scenario.
+        void setCopyright(std::string copy) {d_copyright = copy;}
+        
+        //! Sets the license of the scenario.
+        void setLicense(std::string license) {d_license = license;}
+        
         /** Saves the game. See XML_Helper for further explanations.
           * 
           * @param filename     the full name of the save game file
@@ -132,8 +145,7 @@ class GameScenario: public GameScenarioOptions
 
 	static PlayMode loadPlayMode(std::string filename, bool &broken);
 
-	void startRecordingEventsToFile(std::string filename);
-	void stopRecordingEventsToFile();
+	static void loadDetails(std::string filename, bool &broken, guint32 &player_count, guint32 &city_count, std::string &name, std::string &comment);
 
 	void inhibitAutosaveRemoval(bool i) {inhibit_autosave_removal = i;}
     private:
@@ -144,9 +156,10 @@ class GameScenario: public GameScenarioOptions
 	   * @return true if all went well, false otherwise.
 	   */
 	  bool load(std::string tag, XML_Helper* helper);
-	  void quickStart();
+	  void quickStartEvenlyDivided();
+	  void quickStartAIHeadStart();
 	  bool setupFog(bool hidden_map);
-	  bool setupCities(bool quick_start);
+	  bool setupCities(GameParameters::QuickStartPolicy quick_start);
 	  bool setupRewards(bool hidden_map);
 	  bool setupMapRewards();
 	  bool setupRuinRewards();
@@ -155,13 +168,19 @@ class GameScenario: public GameScenarioOptions
 	  void setupDiplomacy(bool diplomacy);
 	  bool autoSave();
 
+	  bool loadArmysets(Tar_Helper *t);
+	  bool loadTilesets(Tar_Helper *t);
+	  bool loadCitysets(Tar_Helper *t);
+	  bool loadShieldsets(Tar_Helper *t);
+
 	  // DATA
 	  std::string d_name;
 	  std::string d_comment;
+	  std::string d_copyright;
+	  std::string d_license;
 	  bool d_turnmode; //see NextTurn for a description of this option
 	  guint32 d_playmode;
 	  std::string d_id; //globally unique id identifying the scenario
-	  std::string recording_file;
 	  bool inhibit_autosave_removal;
 };
 
