@@ -2,7 +2,7 @@
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
-//  the Free Software Foundation; either version 2 of the License, or
+//  the Free Software Foundation; either version 3 of the License, or
 //  (at your option) any later version.
 //
 //  This program is distributed in the hope that it will be useful,
@@ -39,7 +39,7 @@ ItemProto::ItemProto(XML_Helper* helper)
 
 }
 
-ItemProto::ItemProto(std::string name, Uint32 id)
+ItemProto::ItemProto(std::string name, guint32 id)
 	: Renamable(name)
 {
   d_bonus = 0;
@@ -61,7 +61,7 @@ bool ItemProto::save(XML_Helper* helper) const
 
   // A template is never saved, so we assume this class is a real-life item
   retval &= helper->openTag(d_tag);
-  retval &= helper->saveData("name", getName());
+  retval &= helper->saveData("name", getName(false));
   std::string bonus_str = bonusFlagsToString(d_bonus);
   retval &= helper->saveData("bonus", bonus_str);
 
@@ -87,32 +87,42 @@ void ItemProto::removeBonus(ItemProto::Bonus bonus)
 
 std::string ItemProto::getBonusDescription() const
 {
+  guint32 battle = 0;
+  guint32 command = 0;
+  guint32 goldpercity = 0;
   // the attributes column
   std::vector<Glib::ustring> s;
   if (getBonus(ItemProto::ADD1STR))
-    s.push_back(_("+1 Battle"));
+    battle++;
   if (getBonus(ItemProto::ADD2STR))
-    s.push_back(_("+2 Battle"));
+    battle+=2;
   if (getBonus(ItemProto::ADD3STR))
-    s.push_back(_("+3 Battle"));
+    battle+=3;
   if (getBonus(ItemProto::ADD1STACK))
-    s.push_back(_("+1 Command"));
+    command++;
   if (getBonus(ItemProto::ADD2STACK))
-    s.push_back(_("+2 Command"));
+    command+=2;
   if (getBonus(ItemProto::ADD3STACK))
-    s.push_back(_("+3 Command"));
+    command+=3;
   if (getBonus(ItemProto::FLYSTACK))
     s.push_back(_("Allows Flight"));
   if (getBonus(ItemProto::DOUBLEMOVESTACK))
     s.push_back(_("Doubles Movement"));
   if (getBonus(ItemProto::ADD2GOLDPERCITY))
-    s.push_back(_("+2 gold per city"));
+    goldpercity+=2;
   if (getBonus(ItemProto::ADD3GOLDPERCITY))
-    s.push_back(_("+3 gold per city"));
+    goldpercity+=3;
   if (getBonus(ItemProto::ADD4GOLDPERCITY))
-    s.push_back(_("+4 gold per city"));
+    goldpercity+=4;
   if (getBonus(ItemProto::ADD5GOLDPERCITY))
-    s.push_back(_("+5 gold per city"));
+    goldpercity+=5;
+
+  if (battle > 0)
+    s.push_back(String::ucompose(_("+%1 Battle"), battle));
+  if (command > 0)
+    s.push_back(String::ucompose(_("+%1 Command"), command));
+  if (goldpercity > 0)
+    s.push_back(String::ucompose(_("+%1 gold per city"), goldpercity));
 
   Glib::ustring str;
   bool first = true;
@@ -160,7 +170,7 @@ std::string ItemProto::bonusFlagToString(ItemProto::Bonus bonus)
   return "ItemProto::ADD1STR";
 }
 
-std::string ItemProto::bonusFlagsToString(Uint32 bonus)
+std::string ItemProto::bonusFlagsToString(guint32 bonus)
 {
   std::string bonuses;
   if (bonus & ItemProto::ADD1STR)
@@ -190,9 +200,9 @@ std::string ItemProto::bonusFlagsToString(Uint32 bonus)
   return bonuses;
 }
 
-Uint32 ItemProto::bonusFlagsFromString(std::string str)
+guint32 ItemProto::bonusFlagsFromString(std::string str)
 {
-  Uint32 total = 0;
+  guint32 total = 0;
   std::stringstream bonuses;
   bonuses.str(str);
 
