@@ -3,7 +3,7 @@
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
-//  the Free Software Foundation; either version 2 of the License, or
+//  the Free Software Foundation; either version 3 of the License, or
 //  (at your option) any later version.
 //
 //  This program is distributed in the hope that it will be useful,
@@ -122,7 +122,7 @@ bool Itemlist::save(XML_Helper* helper) const
 
 void Itemlist::remove(ItemProto *itemproto)
 {
-  Uint32 index = 0;
+  guint32 index = 0;
   for (iterator it = begin(); it != end(); it++)
     {
       if ((*it).second == itemproto)
