@@ -25,6 +25,8 @@
 #include "stacklist.h"
 #include "stack.h"
 #include "FogMap.h"
+#include "GameMap.h"
+#include "stacktile.h"
 
 #include "xmlhelper.h"
 
@@ -55,6 +57,11 @@ LocationBox::LocationBox(const LocationBox& loc)
 {
 }
 
+LocationBox::LocationBox(const LocationBox& loc, Vector<int> pos)
+  : Immovable(pos), d_size(loc.d_size)
+{
+}
+
 LocationBox::LocationBox(XML_Helper* helper, guint32 size)
     :Immovable(helper)
 {
@@ -67,43 +74,68 @@ LocationBox::~LocationBox()
 
 Stack *LocationBox::addArmy(Army *a) const
 {
-    Stack* stack = getFreeStack(a->getOwner());
+  Vector<int> pos = Vector<int>(-1,-1);
+  Stack* stack = getFreeStack(a->getOwner(), pos);
 
-    // No stack found in the entire location
-    if (!stack)
-      return NULL;
+  //no stacks with enough room for one more army, found lets create one.
+  if (!stack)
+    {
+      // No stack found in the entire location
+      if (pos == Vector<int>(-1,-1))
+	return NULL;
+	    
+      Player *p = a->getOwner();
+      stack = new Stack(p, pos);
+      stack->add(a);
+      p->addStack(stack);
+    }
+  else
+    stack->add(a);
 
-    // add army to stack
-    stack->push_back(a);
-    if (stack->countGroupedArmies() == 0)
-      stack->ungroup();
-    if (stack->size() > 1)
-      stack->sortForViewing(true);
-    stack->setDefending(false);
-    stack->setParked(false);
-    return stack;
+  if (stack->size() > 1)
+    stack->sortForViewing(true);
+  stack->setDefending(false);
+  stack->setParked(false);
+  return stack;
 }
 
-Stack* LocationBox::getFreeStack(Player *p) const
+bool LocationBox::isFull(Player *p) const
 {
-    for (unsigned int i = 0; i < d_size; i++)
-        for (unsigned int j = 0; j < d_size; j++)
-        {
-            Stack* stack = Stacklist::getObjectAt(getPos().x + j, 
-						  getPos().y + i);
+  for (unsigned int i = 0; i < d_size; i++)
+    for (unsigned int j = 0; j < d_size; j++)
+      {
+	Vector<int> pos = getPos() + Vector<int>(j,i);
+	StackTile *stile = GameMap::getInstance()->getTile(pos)->getStacks();
+	if (stile->canAdd(1, p) == true)
+	  return false;
+      }
+    return true;
+}
 
-            if (stack == NULL)
-            {
-                Vector<int> temp;
-                temp.x = getPos().x + j;
-                temp.y = getPos().y + i;
-                stack = new Stack(p, temp);
-                p->addStack(stack);
-                return stack;
-            }
-            else if (stack->size() < MAX_STACK_SIZE) return stack;
+Stack* LocationBox::getFreeStack(Player *p, Vector<int> &tile) const
+{
+  for (unsigned int i = 0; i < d_size; i++)
+    for (unsigned int j = 0; j < d_size; j++)
+      {
+	Vector<int> pos = getPos() + Vector<int>(j,i);
+	if (GameMap::canAddArmy(pos) == false)
+	  continue;
+	StackTile *stile = GameMap::getInstance()->getTile(pos)->getStacks();
+	Stack *stack = stile->getFriendlyStack(p);
+	if (stack == NULL)
+	  {
+	    tile = pos;
+	    return NULL;
+	  }
+	else 
+	  {
+	    Stack *enemy = stile->getEnemyStack(p);
+	    if (!enemy && stack->isFull() == false)
+	      return stack;
+	  }
         }
-    return NULL;
+  tile = Vector<int>(-1,-1);
+  return NULL;
 }
 
 bool LocationBox::isVisible(Player *player) const
@@ -119,7 +151,8 @@ bool LocationBox::isVisible(Player *player) const
       }
   return false;
 }
-void LocationBox::deFog()
+
+void LocationBox::deFog() const
 {
   Player *p = Playerlist::getActiveplayer();
   if (!p)
@@ -128,7 +161,7 @@ void LocationBox::deFog()
   fogmap->alterFogRadius (getPos(), 3, FogMap::OPEN);
 }
 
-void LocationBox::deFog(Player *p)
+void LocationBox::deFog(Player *p) const
 {
   if (!p)
     return;
@@ -156,3 +189,26 @@ bool LocationBox::isCompletelyObscuredByFog(Player *p) const
       }
   return true;
 }
+
+Vector<int> LocationBox::getNearestPos(Movable *m) const
+{
+  return getNearestPos(m->getPos());
+}
+
+Vector<int> LocationBox::getNearestPos(Vector<int> pos) const
+{
+  int min_dist = -1;
+  Vector<int> closest_tile = Vector<int>(-1,-1);
+  for (unsigned int i = 0; i < d_size; i++)
+    for (unsigned int j = 0; j < d_size; j++)
+      {
+        Vector<int> target = Vector<int>(i,j) + getPos();
+        int d = dist(pos, target);
+        if (d < min_dist || min_dist == -1)
+          {
+            min_dist = d;
+            closest_tile = target;
+          }
+      }
+  return closest_tile;
+}
