@@ -23,9 +23,12 @@
 #define LOCATIONLIST_H
 
 #include <gtkmm.h>
+#include <algorithm>
 #include <list>
 #include <map>
+#include "PathCalculator.h"
 #include "vector.h"
+class Stack;
 
 /** A list for object instances
   * 
@@ -61,9 +64,22 @@ template<class T> class LocationList : public std::list<T>
 	    d_object[pos] = t;
 	  }
     }
+  void subtract(T t)
+    {
+      this->erase(std::find(this->begin(), this->end(), t));
+      d_id.erase(d_id.find(t->getId()));
+      int size = t->getSize();
+      for (int i = 0; i < size; i++)
+	for (int j = 0; j < size; j++)
+	  {
+	    Vector<int> pos = t->getPos() + Vector<int>(i,j);
+	    d_object.erase(d_object.find(pos));
+	  }
+      delete t;
+    }
 
   //! Returns the object at position (x,y).  
-  T getObjectAt(int x, int y) 
+  T getObjectAt(int x, int y) const
     {
       Vector<int> pos = Vector<int>(x,y);
   
@@ -74,16 +90,16 @@ template<class T> class LocationList : public std::list<T>
     }
 
   //! Returns the object at position pos.  
-  T getObjectAt(const Vector<int>& pos) 
+  T getObjectAt(const Vector<int>& pos) const
     {
       return getObjectAt(pos.x, pos.y);
     }
 
-  T getNearestObjectInDir(const Vector<int> &pos, const Vector<int> dir)
+  T getNearestObjectInDir(const Vector<int> &pos, const Vector<int> dir) const
     {
       int diff = -1;
-      typename LocationList<T>::iterator diffit;
-      for (typename LocationList<T>::iterator it = this->begin(); it != this->end(); ++it)
+      typename LocationList<T>::const_iterator diffit;
+      for (typename LocationList<T>::const_iterator it = this->begin(); it != this->end(); ++it)
         {
           Vector<int> p = (*it)->getPos();
           int delta = abs(p.x - pos.x) + abs(p.y - pos.y);
@@ -112,11 +128,53 @@ template<class T> class LocationList : public std::list<T>
       return (*diffit);
     }
 
-  T getNearestObject (const Vector<int>& pos, std::list<bool (*)(void*)> *filters)
+  T getClosestObject (const Stack *stack, std::list<bool (*)(void*)> *filters) const
     {
       int diff = -1;
-      typename LocationList<T>::iterator diffit;
-      for (typename LocationList<T>::iterator it = this->begin(); it != this->end(); ++it)
+      typename LocationList<T>::const_iterator diffit;
+      PathCalculator pc(stack, true, 0, 0);
+      for (typename LocationList<T>::const_iterator it = this->begin(); it != this->end(); ++it)
+        {
+          int delta = pc.calculate((*it)->getPos());
+          if (delta <= 0)
+            continue;
+	  if (filters)
+	    {
+	      std::list<bool (*)(void*)>::iterator fit = filters->begin();
+	      bool filtered = false;
+	      for (; fit != filters->end(); fit++)
+	        {
+	          if ((*fit)(*it) == true)
+	            {
+		      filtered = true;
+		      break;
+	            }
+	            
+	        }
+	      if (filtered)
+	        continue;
+	    }
+
+          if ((diff > delta) || (diff == -1))
+            {
+              diff = delta;
+              diffit = it;
+            }
+        }
+      if (diff == -1) return 0;
+      return (*diffit);
+    }
+
+  T getClosestObject (const Stack *stack) const
+    {
+      return getClosestObject (stack, NULL);
+    }
+
+  T getNearestObject (const Vector<int>& pos, std::list<bool (*)(void*)> *filters) const
+    {
+      int diff = -1;
+      typename LocationList<T>::const_iterator diffit;
+      for (typename LocationList<T>::const_iterator it = this->begin(); it != this->end(); ++it)
         {
           Vector<int> p = (*it)->getPos();
           int delta = abs(p.x - pos.x) + abs(p.y - pos.y);
@@ -147,12 +205,12 @@ template<class T> class LocationList : public std::list<T>
       return (*diffit);
     }
 
-  T getNearestObject (const Vector<int>& pos)
+  T getNearestObject (const Vector<int>& pos) const
     {
       return getNearestObject (pos, NULL);
     }
 
-  T getNearestObjectBefore (const Vector<int>& pos, int dist)
+  T getNearestObjectBefore (const Vector<int>& pos, int dist) const
     {
       T t = getNearestObject(pos);
       if (!t)
@@ -164,12 +222,12 @@ template<class T> class LocationList : public std::list<T>
     }
 
   T getNearestObjectAfter(const Vector<int>& pos, int dist, 
-			   std::list<bool (*)(void*)> *filters)
+			   std::list<bool (*)(void*)> *filters) const
     {
       int diff = -1;
-      typename LocationList<T>::iterator diffit;
+      typename LocationList<T>::const_iterator diffit;
 
-      for (typename LocationList<T>::iterator it = this->begin(); it != this->end(); ++it)
+      for (typename LocationList<T>::const_iterator it = this->begin(); it != this->end(); ++it)
         {
 	  if (filters)
 	    {
