@@ -3,7 +3,7 @@
 // Copyright (C) 2004 David Barnsdale
 // Copyright (C) 2003 Michael Bartl
 // Copyright (C) 2004, 2005 Andrea Paternesi
-// Copyright (C) 2006, 2007, 2008, 2009 Ben Asselstine
+// Copyright (C) 2006, 2007, 2008, 2009, 2010 Ben Asselstine
 // Copyright (C) 2008 Janek Kozicki
 //
 //  This program is free software; you can redistribute it and/or modify
@@ -51,6 +51,8 @@
 #include "army.h"
 #include "vector.h"
 #include "RoadPathCalculator.h"
+#include "cityset.h"
+#include "overviewmap.h"
 
 #define debug(x) {std::cerr<<__FILE__<<": "<<__LINE__<<": "<<x<<std::endl<<std::flush;}
 //#define debug(x)
@@ -64,7 +66,7 @@ MapGenerator::MapGenerator()
     //set reasonable default values
     :d_terrain(0), d_building(0), d_pswamp(2), d_pwater(25), d_pforest(3),
     d_phills(5), d_pmountains(5), d_nocities(11), d_notemples(9), d_noruins(20),
-    d_nosignposts(30)
+    d_nosignposts(30), cityset(NULL)
 
 {
     d_xdir[0]=0;d_xdir[1]=-1;d_xdir[2]=-1;d_xdir[3]=-1;d_xdir[4]=0;d_xdir[5]=1;d_xdir[6]=1;d_xdir[7]=1;
@@ -218,8 +220,11 @@ void MapGenerator::makeMap(int width, int height, bool roads)
     progress.emit(.900, _("spawning temples..."));
     makeBuildings(Maptile::TEMPLE,d_notemples);
     debug("building bridges");
-    progress.emit(.950, _("building bridges..."));
-    makeBridges();
+    if (roads == true)
+      {
+        progress.emit(.950, _("building bridges..."));
+        makeBridges();
+      }
     debug("raising signs");
     progress.emit(.990, _("raising signs..."));
     makeBuildings(Maptile::SIGNPOST,d_nosignposts);
@@ -304,13 +309,16 @@ void MapGenerator::makeBridges()
     for (int x = 0; x < d_width; x++)
       {
 	if (d_building[y*d_width + x] == Maptile::CITY)
-	  Citylist::getInstance()->add(new City(Vector<int>(x,y)));
+	  Citylist::getInstance()->add
+	    (new City(Vector<int>(x,y), cityset->getCityTileWidth()));
 	else if (d_building[y*d_width + x] == Maptile::ROAD)
 	  Roadlist::getInstance()->add(new Road(Vector<int>(x,y)));
 	else if (d_building[y*d_width + x] == Maptile::RUIN)
-	  Ruinlist::getInstance()->add(new Ruin(Vector<int>(x,y)));
+	  Ruinlist::getInstance()->add
+	    (new Ruin(Vector<int>(x,y), cityset->getRuinTileWidth()));
 	else if (d_building[y*d_width + x] == Maptile::TEMPLE)
-	  Templelist::getInstance()->add(new Temple(Vector<int>(x,y)));
+	  Templelist::getInstance()->add
+	    (new Temple(Vector<int>(x,y), cityset->getTempleTileWidth()));
 	else if (d_building[y*d_width + x] == Maptile::PORT)
 	  Portlist::getInstance()->add(new Port(Vector<int>(x,y)));
       }
@@ -1102,7 +1110,7 @@ void MapGenerator::makeCities(int cities)
     {
         int x = rand()%(d_width-2);
         int y = rand()%(d_height-2);
-        if (inhospitableTerrain(x, y, City::getWidth()) && (iterations < 1000))
+        if (inhospitableTerrain(x, y, cityset->getCityTileWidth()) && (iterations < 1000))
         {
             iterations++;
             continue;
@@ -1123,8 +1131,8 @@ void MapGenerator::makeCities(int cities)
 
 bool MapGenerator::canPutCity(int x,int y)
 {
-  for (unsigned int i = 0; i < City::getWidth(); i++)
-    for (unsigned int j = 0; j < City::getWidth(); j++)
+  for (unsigned int i = 0; i < cityset->getCityTileWidth(); i++)
+    for (unsigned int j = 0; j < cityset->getCityTileWidth(); j++)
       if (canPutBuilding(x+i,y+j) == false)
         return false;
         
@@ -1136,12 +1144,12 @@ void MapGenerator::putCity(int x, int y, int& city_count)
         d_building[y*d_width + x] = Maptile::CITY;
 
         //cities shall only sit on grass tiles
-	for (unsigned int i = 0; i < City::getWidth(); i++)
-	  for (unsigned int j = 0; j < City::getWidth(); j++)
+	for (unsigned int i = 0; i < cityset->getCityTileWidth(); i++)
+	  for (unsigned int j = 0; j < cityset->getCityTileWidth(); j++)
 	    d_terrain[(y+i)*d_width + (x+j)] = Tile::GRASS;
         //cities cannot neighbor with mountain tiles
-        for (int Y = -1; Y <= (int)City::getWidth(); ++Y )
-            for (int X = -1; X <= (int)City::getWidth(); ++X)
+        for (int Y = -1; Y <= (int)cityset->getCityTileWidth(); ++Y )
+            for (int X = -1; X <= (int)cityset->getCityTileWidth(); ++X)
                 if (d_terrain[(y+Y)*d_width + x+X] == Tile::MOUNTAIN)
                     d_terrain[(y+Y)*d_width + x+X] = Tile::HILLS;
 
@@ -1159,11 +1167,11 @@ void MapGenerator::makeBuildings(Maptile::Building b, int building)
     switch (b)
       {
       case Maptile::CITY:
-	width = City::getWidth(); break;
+	width = cityset->getCityTileWidth(); break;
       case Maptile::RUIN:
-	width = Ruin::getWidth(); break;
+	width = cityset->getRuinTileWidth(); break;
       case Maptile::TEMPLE:
-	width = Temple::getWidth(); break;
+	width = cityset->getTempleTileWidth(); break;
       case Maptile::NONE:
       case Maptile::SIGNPOST:
       case Maptile::ROAD:
@@ -1218,12 +1226,16 @@ bool MapGenerator::canPutBuilding(int x,int y)
     // if the building is on water or mountains, return false
     if (d_terrain[y*d_width +x] != Tile::GRASS )
         return false;
-        
+
+    int tooclose;
+    tooclose = GameMap::calculateTilesPerOverviewMapTile(d_width, d_height);
+    tooclose++;
     //if the building is close to the map boundaries, return false
-    if ((x < 3) || (x > (d_width-3)) || (y < 3) || (y > (d_height-3)))
+    if (x <= tooclose || x >= (d_width - tooclose) || 
+        y <= tooclose || y >= (d_height - tooclose))
         return false;
 
-    int dist = (int)City::getWidth() + 1;
+    int dist = (int)cityset->getCityTileWidth() + tooclose;
     //if there is another building too close, return false
     for (int locx = x-dist; locx <= x+dist; locx++)
         for (int locy = y-dist; locy <= y+dist; locy++)
@@ -1359,11 +1371,9 @@ int MapGenerator::tryRoad(int src_x, int src_y, int dest_x, int dest_y)
   Path *p = new Path();
   Stack s(NULL, src);
 
-  Armysetlist *al = Armysetlist::getInstance();
-  guint32 armyset = al->getArmysetId(_("Default"), 
-				    Tileset::getDefaultTileSize());
-  const ArmyProto* basearmy = Armysetlist::getInstance()->getArmy(armyset, 1);
+  ArmyProto *basearmy = ArmyProto::createScout();
   Army *a = Army::createNonUniqueArmy(*basearmy);
+  delete basearmy;
   s.push_back(a);
   // try to get there with a scout
   guint32 moves = p->calculate(&s, dest, false);
@@ -1434,14 +1444,12 @@ bool MapGenerator::isAccessible (int src_x, int src_y, int dest_x, int dest_y)
   Path *p = new Path();
   Stack s(NULL, src);
 
-  Armysetlist *al = Armysetlist::getInstance();
-  guint32 armyset = al->getArmysetId(_("Default"), 
-				    Tileset::getDefaultTileSize());
-  const ArmyProto* basearmy = Armysetlist::getInstance()->getArmy(armyset, 1);
+  ArmyProto *basearmy = ArmyProto::createScout();
   Army *a = Army::createNonUniqueArmy(*basearmy);
+  delete basearmy;
   s.push_back(a);
   // try to get there with a scout
-  if (p->calculate(&s, dest, true) == 0)
+  if (p->calculate(&s, dest, true) <= 0)
     retval = false;
   delete p;
 
@@ -1463,15 +1471,23 @@ bool MapGenerator::makeAccessible(int src_x, int src_y, int dest_x, int dest_y)
   Path *p = new Path();
   Stack s(NULL, src);
 
-  Armysetlist *al = Armysetlist::getInstance();
-  guint32 armyset = al->getArmysetId(_("Default"), 
-				    Tileset::getDefaultTileSize());
-  const ArmyProto* basearmy = Armysetlist::getInstance()->getArmy(armyset, 16);
+  ArmyProto *basearmy = ArmyProto::createScout();
   Army *a = Army::createNonUniqueArmy(*basearmy);
+  delete basearmy;
   s.push_back(a);
-  // try to get there with a giant bat
   guint32 moves = p->calculate(&s, dest, false);
 
+  if (moves <= 0)
+    {
+      s.clear();
+      delete a;
+      ArmyProto *basearmy = ArmyProto::createBat();
+      a = Army::createNonUniqueArmy(*basearmy);
+      delete basearmy;
+      s.push_back(a);
+      moves = p->calculate(&s, dest, false);
+    }
+
   if (moves != 0)
     {
       Path::iterator it = p->begin();
@@ -1496,7 +1512,7 @@ bool MapGenerator::makeAccessible(int src_x, int src_y, int dest_x, int dest_y)
 	    {
 	      if (placePort(x, y) == true)
 		{
-		  if (isAccessible(x, y, dest.x, dest.y))
+		  if (isAccessible(src_x, src_y, dest.x, dest.y))
 		    {
 		      retval = true;
 		      break;
@@ -1508,7 +1524,7 @@ bool MapGenerator::makeAccessible(int src_x, int src_y, int dest_x, int dest_y)
 	    {
 	      if (placePort(nextx, nexty) == true)
 		{
-		  if (isAccessible(nextx, nexty, dest.x, dest.y))
+		  if (isAccessible(src_x, src_y, dest.x, dest.y))
 		    {
 		      retval = true;
 		      break;
@@ -1520,8 +1536,8 @@ bool MapGenerator::makeAccessible(int src_x, int src_y, int dest_x, int dest_y)
     }
   else
     retval = false;
-  delete p;
 
+  delete p;
   return retval;
 }
 
@@ -1628,7 +1644,7 @@ void MapGenerator::makeRoads()
 
   GameMap::setWidth(d_width);
   GameMap::setHeight(d_height);
-  GameMap::getInstance("default", "default", "default")->fill(this);
+  GameMap::getInstance("default", "default", cityset->getSubDir())->fill(this);
   Roadlist::getInstance();
   //the game map class smooths the map, so let's take what it smoothed.
   for (int y = 0; y < d_height; y++)
@@ -1640,7 +1656,8 @@ void MapGenerator::makeRoads()
     for (int x = 0; x < d_width; x++)
       {
 	if (d_building[y*d_width + x] == Maptile::CITY)
-	  Citylist::getInstance()->add(new City(Vector<int>(x,y)));
+	  Citylist::getInstance()->add
+	    (new City(Vector<int>(x,y), cityset->getCityTileWidth()));
       }
   GameMap::getInstance()->calculateBlockedAvenues();
 
@@ -1660,16 +1677,18 @@ void MapGenerator::makeRoads()
       makeRoad(src, dest);
       progress.emit(.810, _("paving roads..."));
     }
-
+  
   //make all cities accessible by allowing movement to a central city
-  Vector<int> pos = cl->calculateCenterOfTerritory(NULL);
+  Vector<int> pos = GameMap::getCenterOfMap();
   City *center = cl->getNearestCity(pos);
   for (Citylist::iterator it = cl->begin(); it != cl->end(); it++)
     {
       if (center == *it)
 	continue;
       if (isAccessible(center->getPos(), (*it)->getPos()) == false)
+	{
 	makeAccessible(center->getPos(), (*it)->getPos());
+	}
       progress.emit(.810, _("paving roads..."));
     }
 
