@@ -39,10 +39,14 @@ class NamedLocation: public Location, public Namable
 		   std::string desc);
      //! Copy constructor.
      NamedLocation(const NamedLocation&);
+
+     NamedLocation(const NamedLocation&, Vector<int> pos);
+
      //! Loading constructor.
      NamedLocation(XML_Helper* helper, guint32 size);
+
      //! Destructor.
-    ~NamedLocation();
+    virtual ~NamedLocation();
 
     std::string getDescription() const {return d_description;};
  private:
