@@ -28,6 +28,8 @@
 #include "armysetlist.h"
 #include "armyprodbase.h"
 
+#define ENEMY_CITY_WEIGHT 10
+#define ENEMY_STACK_WEIGHT 10
 using namespace std;
 //#define debug(x) {cerr<<__FILE__<<": "<<__LINE__<<": "<<x<<flush<<endl;}
 #define debug(x)
@@ -77,12 +79,12 @@ void PathCalculator::populateNodeMap()
       nodes[i].moves = -1;
       nodes[i].moves_left = 0;
       nodes[i].turns = 0;
-      if (isBlocked(Vector<int>(i % width, i/width)))
+      if (isBlocked(Vector<int>(i % width, i / width)))
 	nodes[i].moves = -2;
     }
   int idx = start.toIndex();
   nodes[idx].moves = 0;
-  nodes[idx].moves_left = stack->getGroupMoves();
+  nodes[idx].moves_left = stack->getMoves();
   nodes[idx].turns = 0;
 
   // now the main loop
@@ -99,17 +101,17 @@ void PathCalculator::populateNodeMap()
     }
 }
 
-PathCalculator::PathCalculator(Stack *s, bool zig)
+PathCalculator::PathCalculator(const Stack *s, bool zig, int city_avoidance, int stack_avoidance)
 :stack(s), flying(s->isFlying()), d_bonus(s->calculateMoveBonus()),
-    land_reset_moves(s->getMaxGroupLandMoves()),
-    boat_reset_moves(s->getMaxGroupBoatMoves()), zigzag(zig), on_ship(stack->hasShip()), delete_stack(false)
+    land_reset_moves(s->getMaxLandMoves()),
+    boat_reset_moves(s->getMaxBoatMoves()), zigzag(zig), on_ship(stack->hasShip()), enemy_city_avoidance(city_avoidance), enemy_stack_avoidance(stack_avoidance), delete_stack(false)
 {
   populateNodeMap();
 }
 
-PathCalculator::PathCalculator(Player *p, Vector<int> src, const ArmyProdBase *prodbase, bool zig)
+PathCalculator::PathCalculator(Player *p, Vector<int> src, const ArmyProdBase *prodbase, bool zig, int city_avoidance, int stack_avoidance)
 {
-  stack = Stack::createNonUniqueStack(p, src);
+  Stack *new_stack = Stack::createNonUniqueStack(p, src);
   Army *army;
   if (!prodbase)
     {
@@ -123,25 +125,30 @@ PathCalculator::PathCalculator(Player *p, Vector<int> src, const ArmyProdBase *p
 
   if (!army)
     return;
-  stack->push_back(army);
+  new_stack->push_back(army);
+  stack = new_stack;
   flying = stack->isFlying();
   d_bonus = stack->calculateMoveBonus();
-  land_reset_moves = stack->getMaxGroupLandMoves();
-  boat_reset_moves = stack->getMaxGroupBoatMoves();
+  land_reset_moves = stack->getMaxLandMoves();
+  boat_reset_moves = stack->getMaxBoatMoves();
   zigzag = zig; 
+  enemy_city_avoidance = city_avoidance;
+  enemy_stack_avoidance = stack_avoidance;
   on_ship = stack->hasShip();
   delete_stack = true;
   populateNodeMap();
 }
 
-PathCalculator::PathCalculator(const Stack &s, bool zig)
+PathCalculator::PathCalculator(const Stack &s, bool zig, int city_avoidance, int stack_avoidance)
 {
   stack = new Stack(s);
   flying = stack->isFlying();
   d_bonus = stack->calculateMoveBonus();
-  land_reset_moves = stack->getMaxGroupLandMoves();
-  boat_reset_moves = stack->getMaxGroupBoatMoves();
+  land_reset_moves = stack->getMaxLandMoves();
+  boat_reset_moves = stack->getMaxBoatMoves();
   zigzag = zig; 
+  enemy_city_avoidance = city_avoidance;
+  enemy_stack_avoidance = stack_avoidance;
   on_ship = stack->hasShip();
   delete_stack = true;
   populateNodeMap();
@@ -152,7 +159,7 @@ PathCalculator::PathCalculator(const PathCalculator &p)
 :stack(new Stack(*p.stack)), flying(p.flying), d_bonus(p.d_bonus),
     land_reset_moves(p.land_reset_moves),
     boat_reset_moves(p.boat_reset_moves), zigzag(p.zigzag), on_ship(p.on_ship),
-    delete_stack(p.delete_stack)
+    enemy_city_avoidance(p.enemy_city_avoidance), enemy_stack_avoidance(p.enemy_stack_avoidance), delete_stack(p.delete_stack)
 {
   int width = GameMap::getWidth();
   int height = GameMap::getHeight();
@@ -340,11 +347,10 @@ std::list<Vector<int> > PathCalculator::calcMoves(Vector<int> pos)
 
 bool PathCalculator::load_or_unload(Vector<int> src, Vector<int> dest, bool &on_ship)
 {
+  Stack *new_stack = Stack::createNonUniqueStack(stack->getOwner(), src);
   //do we load or unload if we step from SRC to DEST?
-  Vector<int> old = stack->getPos();
-  stack->setPos(src);
-  bool retval = stack->isMovingToOrFromAShip(dest, on_ship);
-  stack->setPos(old);
+  bool retval = new_stack->isMovingToOrFromAShip(dest, on_ship);
+  delete new_stack;
   return retval;
 }
 int PathCalculator::pointsToMoveTo(Vector<int> pos, Vector<int> next) const
@@ -357,6 +363,19 @@ int PathCalculator::pointsToMoveTo(Vector<int> pos, Vector<int> next) const
 
   moves = tile->getMoves();
 
+  if (enemy_city_avoidance >= 1)
+    {
+      //We will still try to avoid enemy cities a little.
+      if (GameMap::getEnemyCity(pos))
+	moves += enemy_city_avoidance;
+    }
+  if (enemy_stack_avoidance >= 1)
+    {
+      //We will still try to avoid enemy stacks a little.
+      if (GameMap::getEnemyStack(pos))
+	moves += enemy_stack_avoidance;
+    }
+
   // does everything in the stack have a bonus to move onto this square?
   if (tile->getMaptileType() & d_bonus && moves != 1)
     return 2;
@@ -402,33 +421,24 @@ bool PathCalculator::isBlockedDir(Vector<int> pos, Vector<int> next)
   return false;
 }
 
-bool PathCalculator::isBlocked(Stack *s, Vector<int> pos)
+bool PathCalculator::isBlocked(const Stack *s, Vector<int> pos, bool enemy_cities_block, bool enemy_stacks_block)
 {
   const Maptile* tile = GameMap::getInstance()->getTile(pos);
 
   // Return true on every condition which may prevent the stack from
   // entering the tile, which are...
 
-  Stack* target = Stacklist::getObjectAt(pos);
-  if (target)
+  // ...enemy stacks which stand in the way...
+  if (enemy_stacks_block == true)
     {
-      // ...enemy stacks which stand in the way...
-      if (s->getOwner() != target->getOwner())
+      Stack* target = GameMap::getEnemyStack(pos);
+      if (target)
 	return true;
-
-      //the computer walks around any too-big stacks of it's own.
-      //if (s->getOwner() == target->getOwner()
-	  //&& s->canJoin(target) == false && 
-	  //s->getOwner()->getType() != Player::HUMAN) 
-	//{
-	  //return true;
-	//}
-
     }
 
   //...enemy cities
   // saves some computation time here
-  if (tile->getBuilding() == Maptile::CITY)
+  if (tile->getBuilding() == Maptile::CITY && enemy_cities_block == true)
     {
       City* c = GameMap::getCity(pos);
       if (c && (c->getOwner() != s->getOwner()))
@@ -441,7 +451,7 @@ bool PathCalculator::isBlocked(Stack *s, Vector<int> pos)
 
 bool PathCalculator::isBlocked(Vector<int> pos)
 {
-  return isBlocked(stack, pos);
+  return isBlocked(stack, pos, enemy_city_avoidance < 0, enemy_stack_avoidance < 0);
 }
 
 bool PathCalculator::canMoveThere(Vector<int> dest)
@@ -572,7 +582,7 @@ Path* PathCalculator::calculate(Vector<int> dest, guint32 &moves, guint32 &turns
 
   //calculate when the waypoints show no more movement possible
   guint32 pathcount = 0;
-  guint32 moves_left = stack->getGroupMoves();
+  guint32 moves_left = stack->getMoves();
   for (Path::iterator it = path->begin(); it != path->end(); it++)
     {
       guint32 moves = stack->calculateTileMovementCost(*it);
@@ -653,7 +663,7 @@ Path *PathCalculator::calculateToCity (City *c, guint32 &moves, guint32 &turns,
 	if (checkJoin == true)
 	  {
 	    Stack *other_stack = GameMap::getStack(c->getPos() + Vector<int>(i,j));
-	    if (other_stack && stack->canJoin(other_stack) == false)
+	    if (other_stack && GameMap::canJoin(stack,other_stack) == false)
 	      continue;
 	  }
 	int distance = dist (stack->getPos(), c->getPos() + Vector<int>(i, j));
@@ -679,7 +689,7 @@ Path *PathCalculator::calculateToCity (City *c, guint32 &moves, guint32 &turns,
 	if (checkJoin == true)
 	  {
 	    Stack *other_stack = GameMap::getStack(c->getPos() + Vector<int>(i,j));
-	    if (other_stack && stack->canJoin(other_stack) == false)
+	    if (other_stack && GameMap::canJoin(stack, other_stack) == false)
 	      continue;
 	  }
 	p = calculate(c->getPos() + Vector<int>(i,j), moves, turns, zigzag);
@@ -697,3 +707,15 @@ Path *PathCalculator::calculateToCity (City *c, guint32 &moves, guint32 &turns,
   return calculate(shortest, moves, turns, zigzag);
 }
 
+std::list<Vector<int> > PathCalculator::getReachablePositions(int mp)
+{
+  std::list<Vector<int> > positions;
+  int width = GameMap::getWidth();
+  int height = GameMap::getHeight();
+  for (int i = 0; i < width*height; i++)
+    {
+      if (nodes[i].moves < mp || mp == 0)
+	positions.push_back(Vector<int>(i % width, i / width));
+    }
+  return positions;
+}
