@@ -23,6 +23,7 @@
 #include <stdlib.h>
 #include <algorithm>
 #include <fstream>
+#include <assert.h>
 
 #include "ai_fast.h"
 
@@ -44,6 +45,11 @@
 #include "hero.h"
 #include "vectoredunitlist.h"
 #include "PathCalculator.h"
+#include "stacktile.h"
+#include "armyprodbase.h"
+#include "QuestsManager.h"
+#include "Quest.h"
+#include "Sage.h"
 
 using namespace std;
 
@@ -52,7 +58,7 @@ using namespace std;
 
 AI_Fast::AI_Fast(string name, guint32 armyset, Gdk::Color color, int width, int height, int player_no)
     :RealPlayer(name, armyset, color, width, height, Player::AI_FAST, player_no), d_join(true),
-    d_maniac(false), d_analysis(0), d_diplomacy(0), d_abort_requested(false)
+    d_maniac(false), d_analysis(0), d_diplomacy(0)
 {
 }
 
@@ -61,11 +67,10 @@ AI_Fast::AI_Fast(const Player& player)
     d_diplomacy(0)
 {
     d_type = AI_FAST;
-    d_abort_requested = false;
 }
 
 AI_Fast::AI_Fast(XML_Helper* helper)
-    :RealPlayer(helper), d_analysis(0), d_diplomacy(0), d_abort_requested(false)
+    :RealPlayer(helper), d_analysis(0), d_diplomacy(0)
 {
     helper->getData(d_join, "join");
     helper->getData(d_maniac, "maniac");
@@ -94,7 +99,7 @@ bool AI_Fast::save(XML_Helper* helper) const
 
 void AI_Fast::abortTurn()
 {
-  d_abort_requested = true;
+  abort_requested = true;
   if (surrendered)
     aborted_turn.emit();
   else if (Playerlist::getInstance()->countPlayersAlive() == 1)
@@ -104,13 +109,12 @@ void AI_Fast::abortTurn()
 bool AI_Fast::startTurn()
 {
     sbusy.emit();
-    AI_maybeBuyScout();
 
     sbusy.emit();
-    // maniac AI's never recruit heroes, otherwise take everything we can get
-    if (!d_maniac)
-      maybeRecruitHero();
-    
+    if (getStacklist()->getHeroes().size() == 0 &&
+        Citylist::getInstance()->countCities(this) == 1)
+      AI_maybeBuyScout(Citylist::getInstance()->getFirstCity(this));
+    sbusy.emit();
 
     debug(getName() << ": AI_Fast::start_turn")
     debug("being in " <<(d_maniac?"maniac":"normal") <<" mode")
@@ -121,34 +125,51 @@ bool AI_Fast::startTurn()
 
     d_diplomacy->considerCuspOfWar();
     
-    if (getUpkeep() > getIncome() + getIncome())
+    d_maniac = false;
+    float ratio = 2.0;
+    if (getUpkeep() > getIncome() * ratio)
       d_maniac = true;
-    else
-      d_maniac = false;
 
     //setup production
-    if (getIncome() > getUpkeep())
+    debug("examining cities");
+    Citylist *cl = Citylist::getInstance();
+    for (Citylist::iterator cit = cl->begin(); cit != cl->end(); ++cit)
       {
-	Citylist *cl = Citylist::getInstance();
-	for (Citylist::iterator cit = cl->begin(); cit != cl->end(); ++cit)
-	  {
-	    City *c = *cit;
-	    if (c->getOwner() != this || c->isBurnt())
-	      continue;
-	    if (c->getActiveProductionSlot() == -1)
-	      {
-		if (c->getProductionBase(0))
-		  c->setActiveProductionSlot(0);
-	      }
-	  }
+        City *c = *cit;
+        if (c->getOwner() != this || c->isBurnt())
+          continue;
+        if (c->getActiveProductionSlot() == -1)
+          setBestProduction(c);
       }
 
     //setup vectoring
+    debug("setting up vectoring");
     if (!d_maniac)
 	AI_setupVectoring(18, 3, 30);
 
     sbusy.emit();
 
+    debug("trying to complete quests");
+    //try to complete our quests
+    std::vector<Quest*> q = QuestsManager::getInstance()->getPlayerQuests(this);
+    for (std::vector<Quest*>::iterator it = q.begin(); it != q.end(); it++)
+      {
+        Quest *quest = *it;
+        if (quest->isPendingDeletion())
+          continue;
+        Stack *s = getStacklist()->getArmyStackById(quest->getHeroId());
+        if (!s)
+          continue;
+        Vector<int> dest = AI_getQuestDestination(quest, s);
+        if (dest == Vector<int>(-1,-1))
+            continue;
+        d_stacklist->setActivestack(s);
+        s->getPath()->calculate(s, dest);
+        bool stack_moved = stackMove(s);
+        if (d_stacklist->getActivestack() && stack_moved)
+          GameMap::groupStacks(s);
+      }
+
     while (computerTurn() == true)
       {
 	sbusy.emit();
@@ -164,19 +185,22 @@ bool AI_Fast::startTurn()
 		if (mp <= 0)
 		  continue;
 		debug ("AI_FAST stack " << s->getId() << " can still potentially move");
-		debug ("moving from " << s->getPos().x << "," << s->getPos().y
+		debug ("moving from (" << s->getPos().x << "," << s->getPos().y
 		       << ") to (" <<s->getFirstPointInPath().x << "," <<
-		       s->getFirstPointInPath().y << ") with " << s->getGroupMoves() <<" left");
+		       s->getFirstPointInPath().y << ") with " << s->getMoves() <<" left");
 
 	    
 		found = true;
 	      }
+	    //are there any stacks without paths that still have some moves?
+	    else if (s->getPath()->size() == 0 && s->getMoves() > 1)
+	      found = true;
 	  }
 	if (!found)
 	  break;
 	if (found)
 	  found = false;
-	if (d_abort_requested)
+	if (abort_requested)
 	  break;
       }
 
@@ -189,43 +213,126 @@ bool AI_Fast::startTurn()
     if (GameScenarioOptions::s_diplomacy)
       d_diplomacy->makeProposals();
 
-    if (d_abort_requested)
+    if (abort_requested)
       aborted_turn.emit();
     return true;
 }
 
+int AI_Fast::scoreArmyType(const ArmyProdBase *a)
+{
+  int max_strength = a->getStrength();
+
+  int production;
+  if (a->getProduction() == 1)
+    production = 6;
+  else if (a->getProduction() == 2)
+    production = 2;
+  else
+    production = 1;
+
+  int upkeep = 0;
+  if (a->getUpkeep() < 5)
+    upkeep = 6;
+  else if (a->getUpkeep() < 10)
+    upkeep = 2;
+  else 
+    upkeep = 1;
+
+  int newcost = 0;
+  if (a->getProductionCost() < 5)
+    newcost = 6;
+  else if (a->getProductionCost() < 10)
+    newcost = 2;
+  else
+    newcost = 1;
+
+  //we prefer armies that move farther
+  int move_bonus = 0;
+  if (a->getMaxMoves() >  10)
+    move_bonus += 2;
+  if (a->getMaxMoves() >=  20)
+    move_bonus += 4;
+
+  return max_strength + move_bonus + production + upkeep + newcost;
+}
+
+int AI_Fast::setBestProduction(City *c)
+{
+  int select = -1;
+  int score = -1;
+
+  // we try to determine the most attractive basic production
+  for (guint32 i = 0; i < c->getMaxNoOfProductionBases(); i++)
+    {
+      if (c->getArmytype(i) == -1)    // no production in this slot
+        continue;
+
+      const ArmyProdBase *proto = c->getProductionBase(i);
+      if (scoreArmyType(proto) > score)
+        {
+          select = i;
+          score = scoreArmyType(proto);
+        }
+    }
+
+
+  if (select != c->getActiveProductionSlot())
+    {
+      cityChangeProduction(c, select);
+      debug(getName() << " Set production to slot " << select << " in " << c->getName())
+    }
+
+  return c->getActiveProductionSlot();
+}
+
 void AI_Fast::invadeCity(City* c)
 {
+  CityDefeatedAction action = CITY_DEFEATED_OCCUPY;
+  bool quest_preference = AI_invadeCityQuestPreference(c, action);
   debug("Invaded city " <<c->getName());
 
-  // There are three modes: maniac razes all cities, non-maniac just
-  // occupies them...
-  // but even a maniac won't raze a city if money is needed
-  if (getIncome() < getUpkeep())
+  if (quest_preference == false)
     {
-      debug("Occupying it");
-      cityOccupy(c);
+      if (getIncome() < getUpkeep())
+        action = CITY_DEFEATED_OCCUPY;
+      else if (d_maniac)
+        action = CITY_DEFEATED_RAZE;
+      else
+        action = CITY_DEFEATED_OCCUPY;
     }
-  else if (d_maniac)
+  int gold = 0;
+  int pillaged_army_type = -1;
+  std::list<guint32> sacked_army_types;
+  switch (action)
     {
-      debug ("Razing it");
+    case CITY_DEFEATED_OCCUPY:
+      cityOccupy(c);
+      setBestProduction(c);
+      break;
+    case CITY_DEFEATED_PILLAGE:
+      cityPillage(c, gold, &pillaged_army_type);
+      AI_maybeBuyScout(c);
+      setBestProduction(c);
+      break;
+    case CITY_DEFEATED_RAZE:
       cityRaze(c);
-    }
-  else
-    {
-      debug("Occupying it");
-      cityOccupy(c);
+      break;
+    case CITY_DEFEATED_SACK:
+      citySack(c, gold, &sacked_army_types);
+      AI_maybeBuyScout(c);
+      setBestProduction(c);
+      break;
     }
 }
 
-void AI_Fast::levelArmy(Army* a)
+void AI_Fast::heroGainsLevel(Hero * a)
 {
     debug("Army raised a level, id = " <<a->getId())
     
     //advancing a level
     // increase the strength attack (uninnovative, but enough here)
     Army::Stat stat = Army::STRENGTH;
-    doLevelArmy(a, stat);
+    doHeroGainsLevel(a, stat);
 
     Action_Level* item = new Action_Level();
     item->fillData(a, stat);
@@ -234,19 +341,43 @@ void AI_Fast::levelArmy(Army* a)
 
 Stack *AI_Fast::findNearOwnStackToJoin(Stack *s, int max_distance)
 {
+  int min_mp = -1;
+  std::list<Stack*> stks;
+  stks = GameMap::getNearbyFriendlyStacks(s->getPos(), max_distance);
+  if (stks.size() <= 0)
+    return NULL;
+  PathCalculator pc(s);
   Stack* target = NULL;
-  for (Stacklist::iterator it2 = d_stacklist->begin(); it2 != d_stacklist->end(); it2++)
+  for (std::list<Stack*>::iterator it = stks.begin(); it != stks.end(); it++)
     {
-      if (s->size() + (*it2)->size() > MAX_STACK_SIZE || s == (*it2))
+      //is this us?
+      if (s == (*it))
+	continue;
+      //is this a stack that is co-located?
+      if (s->getPos() == (*it)->getPos())
+	return s;
+
+      //does the destination have few enough army units to join?
+      if (GameMap::canJoin(s, (*it)) == false)
 	continue;
 
-      int distance = dist(s->getPos(), (*it2)->getPos());
+      //is the tile distance under the threshold?
+      int distance = dist(s->getPos(), (*it)->getPos());
 
       if (distance <= max_distance)
 	{
-	  target = (*it2);
-	  break;
+	  //can we actually get there?
+	  int mp = pc.calculate((*it)->getPos());
+	  if (mp <= 0)
+	    continue;
+	  if (mp < min_mp || min_mp == -1)
+	    {
+	      target = (*it);
+	      min_mp = mp;
+	    }
 	}
+
+
     }
   return target;
 }
@@ -267,7 +398,6 @@ bool AI_Fast::computerTurn()
     //          resupply
     //      if (!maniac)
     //          find next enemy city
-    //          if it's too far away then disband
     //      else
     //          find next enemy unit with preference to cities
     //      attack
@@ -278,30 +408,58 @@ bool AI_Fast::computerTurn()
     // and we want the freshly created stacks join the veterans and not the other
     // way round.
     //d_stacklist->dump();
-    for (Stacklist::reverse_iterator it = d_stacklist->rbegin(); it != d_stacklist->rend(); it++)
+ 
+  std::list<Vector<int> > points = d_stacklist->getPositions();
+    for (std::list<Vector<int> >::iterator it = points.begin(); 
+         it != points.end(); it++)
     {
-        d_stacklist->setActivestack(*it);
-        Stack* s = *it;
-	s->group();
+      Stack *s = GameMap::getFriendlyStack(*it);
+      if (!s)
+        continue;
+
+        d_stacklist->setActivestack(s);
         
-	//go to a temple
+	//go to a temple or ruin
 	if (!d_maniac)
 	  {
 	    bool stack_died = false;
 	    bool blessed = false;
-	    if (Citylist::getInstance()->getObjectAt(s->getPos()) == NULL)
+	    if (s->isOnCity() == false && s->hasHero() == false)
 	      {
 		stack_moved = AI_maybeVisitTempleForBlessing
-		  (s, s->getGroupMoves(), s->getGroupMoves() + 7, 50.0, 
+		  (s, s->getMoves(), s->getMoves() + 7, 50.0, 
 		   blessed, stack_died);
 		if (stack_died)
 		  return true;
 		s = d_stacklist->getActivestack();
-		if (blessed && stack_moved)
-		  stack_moved = false; //do this so we move it later on
-		else if (stack_moved)
-		  continue;
+		if (stack_moved)
+                  {
+                    GameMap::groupStacks(s);
+                    s->clearPath();
+                    continue;
+                  }
 	      }
+            else if (s->isOnCity() == false && s->hasHero() == true)
+              {
+		stack_moved = AI_maybeVisitTempleForQuest
+		  (s, s->getMoves(), s->getMoves() + 15, stack_died);
+		if (stack_died)
+		  return true;
+                if (!stack_moved)
+                  {
+                    stack_moved = AI_maybeVisitRuin
+                      (s, s->getMoves(), s->getMoves() + 15, stack_died);
+                    if (stack_died)
+                      return true;
+                  }
+		s = d_stacklist->getActivestack();
+		if (stack_moved)
+                  {
+                    GameMap::groupStacks(s);
+                    s->clearPath();
+                    continue;
+                  }
+              }
 	  }
 
 	//pick up items
@@ -310,8 +468,8 @@ bool AI_Fast::computerTurn()
 	    bool stack_died = false;
 	    bool picked_up = false;
 
-	    stack_moved = AI_maybePickUpItems(s, s->getGroupMoves(), 
-					       s->getGroupMoves() + 7, 
+	    stack_moved = AI_maybePickUpItems(s, s->getMoves(), 
+					       s->getMoves() + 7, 
 					       picked_up, stack_died);
 	    if (stack_died)
 	      return true;
@@ -326,7 +484,7 @@ bool AI_Fast::computerTurn()
 	       <<"," <<s->getPos().y <<") containing " <<s->size() << " armies ?")
 
         // join armies if close
-        if (d_join && s->size() < MAX_STACK_SIZE)
+        if (d_join && s->isFull() == false)
         {
             Stack* target = NULL;
 	    target = findNearOwnStackToJoin(s, 5);
@@ -336,6 +494,14 @@ bool AI_Fast::computerTurn()
                 debug("Joining with stack " <<target->getId() <<" at (" <<target->getPos().x <<"," <<target->getPos().y <<")")
 	       	s->getPath()->calculate(s, target->getPos());
                 stack_moved |= stackMove(s);
+		//in case we lost our stack
+		if (!d_stacklist->getActivestack())
+		  return true;
+		if (s->getPos() == target->getPos())
+		  {
+		    GameMap::groupStacks(s);
+		    continue;
+		  }
 		continue;
             }
         }
@@ -344,7 +510,7 @@ bool AI_Fast::computerTurn()
         if (!d_maniac)
         {
             City *target = Citylist::getInstance()->getNearestFriendlyCity(s->getPos());
-            if (s->size() < MAX_STACK_SIZE && target)
+            if (s->isFull() == false && target)
             {
                 debug("Restocking in " <<target->getName())
                 // try to move to the north west part of the city (where the units
@@ -352,6 +518,7 @@ bool AI_Fast::computerTurn()
 
 		if (target->contains(s->getPos()) == false)
 		  {
+		    debug("Stack is not in " << target->getName() << " yet" <<endl);
 		    int mp = s->getPath()->calculateToCity(s, target);
 		    if (mp > 0)
 		      {
@@ -360,34 +527,46 @@ bool AI_Fast::computerTurn()
 			// the stack could have joined another stack waiting there
 			if (!d_stacklist->getActivestack())
 			  return true;
-			continue;
-		      }
-		    else
-		      {
-			d_maniac = true;
+			if (stack_moved)
+			  {
+			    GameMap::groupStacks(s);
+			    s->clearPath();
+			    continue;
+			  }
 		      }
 		  }
 		else if (s->getPos() != target->getPos())
 		  {
+		    debug("Stack is inside " << target->getName() << endl);
 		    //if we're not in the upper right corner
-		    int mp = s->getPath()->calculateToCity(s, target);
-		    if (mp)
+		    s->getPath()->calculate(s, target->getPos());
+		    //go there, and take as many as we can
+                    Stack *new_stack = NULL;
+		    stack_moved |= stackSplitAndMove(s, new_stack);
+		    //in case we lost our stack
+		    if (!d_stacklist->getActivestack())
+		      return true;
+		    if (stack_moved)
 		      {
-		    //then try to go there
-		    stack_moved |= stackMove(s);
-		    continue;
+			    GameMap::groupStacks(s);
+			    s->clearPath();
+			    GameMap::groupStacks(target->getPos());
+			    return true;
 		      }
-		    else
-		      d_maniac = true;
 		  }
+		else
+		  {
 		//otherwise just stay put in the city
+		    GameMap::groupStacks(s);
+		    continue;
+		  }
 	    }
 
 	    // third step: non-maniac players attack only enemy cities
 	    else
 	      {
 		target = NULL;
-		PathCalculator pc(s);
+		PathCalculator pc(s, true, 10, -1);
 		guint32 moves1 = 0, turns1 = 0, moves2 = 0, turns2 = 0;
 		Path *target1_path = NULL;
 		Path *target2_path = NULL;
@@ -440,29 +619,6 @@ bool AI_Fast::computerTurn()
 		  int moves = s->getPath()->calculateToCity(s, target);
 		debug("Moves to enemy city: " << moves);
 
-		if (moves > 60)
-		  {
-		    //are we in a city?
-		    City *c = Citylist::getInstance()->getObjectAt(s->getPos());
-		    if (c)
-		      {
-			//is nearest friendly city to the enemy not our city?
-			if (cl->getNearestFriendlyCity(target->getPos()) != c)
-			  {
-			    bool disbanded;
-			    bool killed = false;
-			    guint32 id = s->getId();
-			    disbanded = AI_maybeDisband(s, c, 3, 18, killed);
-			    if (disbanded)
-			      {
-				debug ("disbanded stack " << id << " in " << c->getName() <<"\n");
-				return true;
-			      }
-			    if (killed)
-			      return true;
-			  }
-		      }
-		  }
 		if (moves >= 1)
 		  {
 		    stack_moved |= stackMove(s);
@@ -474,11 +630,18 @@ bool AI_Fast::computerTurn()
 		      {
 			//and the target city is empty
 			if (target->countDefenders() == 0)
-			  {
-			    //attack it if we can reach it.
-			    int moved = stackSplitAndMove(s);
-			    stack_moved |=  moved;
-			  }
+                          {
+                            //attack it if we can reach it.
+                            Stack *new_stack = NULL;
+                            int moved = stackSplitAndMove(s, new_stack);
+                            stack_moved |=  moved;
+                            if (moved)
+                              {
+                                GameMap::groupStacks(s);
+                                GameMap::groupStacks(target->getPos());
+                                return true;
+                              }
+                          }
 		      }
 		  }
 		else
@@ -488,7 +651,9 @@ bool AI_Fast::computerTurn()
 		    //for some reason we can't set parked on this thing
 		    //and have it realize it, after we return true.
 		    //why is that?
-		    printf("crap, it happened\n");
+		    printf("crap, it happened with a stack at %d,%d\n", s->getPos().x, s->getPos().y);
+		    printf("moves is %d\n", moves);
+		    printf("Destination was %d,%d (%s)\n", target->getPos().x, target->getPos().y, target->getName().c_str());
 		    stackDisband(s);
 		    return true;
 		  }
@@ -524,7 +689,7 @@ bool AI_Fast::computerTurn()
 		  continue;
 
 		guint32 mp = s->getPath()->calculate(s, threatpos);
-		if ((int)mp <= 0 || mp > s->getGroupMoves())
+		if ((int)mp <= 0 || mp > s->getMoves())
 		  continue;
 
 		target = *tit;
@@ -559,10 +724,13 @@ bool AI_Fast::computerTurn()
 	      {
 	      //printf ("stack %d at %d,%d moving %d with %d moves\n",
 		      //s->getId(), s->getPos().x, s->getPos().y,
-		      //mp, s->getGroupMoves());
+		      //mp, s->getMoves());
 		bool moved = stackMove(s);
 		//printf("result of move: %d\n", moved);
 		stack_moved |= moved;
+		//in case we lost our stack
+		if (!d_stacklist->getActivestack())
+		  return true;
 		s = d_stacklist->getActivestack();
 	      }
 	    else
@@ -576,7 +744,12 @@ bool AI_Fast::computerTurn()
 		  {
 		    mp = s->getPath()->calculate(s, friendly_city->getPos());
 		    if (mp > 0)
+		      {
 		      stack_moved |= stackMove(s);
+		//in case we lost our stack
+		if (!d_stacklist->getActivestack())
+		  return true;
+		      }
 		    else
 		      stack_moved |= false;
 		  }
@@ -593,17 +766,39 @@ bool AI_Fast::computerTurn()
 	    continue;
 
 	  }
-	if (d_abort_requested)
+	if (abort_requested)
 	  break;
     }
     return stack_moved;
 }
 
-bool AI_Fast::treachery (Stack *stack, Player *player, Vector <int> pos, DiplomaticState state)
+bool AI_Fast::chooseTreachery (Stack *stack, Player *player, Vector <int> pos)
 {
   bool performTreachery = true;
   return performTreachery;
 }
 
+bool AI_Fast::chooseHero(HeroProto *hero, City *city, int gold)
+{
+  return true;
+}
 
+Reward *AI_Fast::chooseReward(Ruin *ruin, Sage *sage, Stack *stack)
+{
+  //always pick the money.
+  for (Sage::iterator it = sage->begin(); it != sage->end(); it++)
+    if ((*it)->getType() == Reward::GOLD)
+      return (*it);
+  return sage->front();
+}
+
+Army::Stat AI_Fast::chooseStat(Hero *hero)
+{
+  return Army::STRENGTH;
+}
+
+bool AI_Fast::chooseQuest(Hero *hero)
+{
+  return true;
+}
 // End of file
