@@ -70,6 +70,8 @@ class AI_Fast : public RealPlayer
 	//! Destructor.
         ~AI_Fast();
         
+	virtual bool isComputer() const {return true;};
+
         //! Saves data, the method is for saving additional data.
         bool save(XML_Helper* helper) const;
 
@@ -88,9 +90,12 @@ class AI_Fast : public RealPlayer
 	virtual void abortTurn();
         virtual bool startTurn();
         virtual void invadeCity(City* c);
-        virtual void levelArmy(Army* a);
-	virtual bool treachery (Stack *stack, Player *player, Vector <int> pos, 
-				DiplomaticState state);
+        virtual bool chooseHero(HeroProto *hero, City* c, int gold);
+        virtual Reward *chooseReward(Ruin *ruin, Sage *sage, Stack *stack);
+        virtual void heroGainsLevel(Hero * a);
+	virtual bool chooseTreachery (Stack *stack, Player *player, Vector <int> pos);
+        virtual Army::Stat chooseStat(Hero *hero);
+        virtual bool chooseQuest(Hero *hero);
 
     private:
         //! The actual core function of the ai's logic.
@@ -99,6 +104,10 @@ class AI_Fast : public RealPlayer
 	//! search through our stacklist for a stack we can join
 	Stack *findNearOwnStackToJoin(Stack *s, int max_distance);
 
+        //! produce the best low-turn high strength army unit.
+        int setBestProduction(City *c);
+
+        int scoreArmyType(const ArmyProdBase *a);
 
 	//! Determines whether to join units or move them separately.
         bool d_join;
@@ -108,7 +117,6 @@ class AI_Fast : public RealPlayer
 
         AI_Analysis* d_analysis;
         AI_Diplomacy* d_diplomacy;
-	bool d_abort_requested;
 };
 
 #endif // AI_FAST_H
