@@ -91,24 +91,38 @@ class AI_Smart : public RealPlayer
 	//! Destructor.
         ~AI_Smart();
 
+	virtual bool isComputer() const {return true;};
 	virtual void abortTurn();
         virtual bool startTurn();
         virtual void invadeCity(City* c);
-        virtual void levelArmy(Army* a);
+        virtual bool chooseHero(HeroProto *hero, City* c, int gold);
+        virtual Reward *chooseReward(Ruin *ruin, Sage *sage, Stack *stack);
+        virtual void heroGainsLevel(Hero * a);
+	virtual bool chooseTreachery (Stack *stack, Player *player, Vector <int> pos);
+        virtual Army::Stat chooseStat(Hero *hero);
+        virtual bool chooseQuest(Hero *hero);
 
     private:
         // Choose a new type of army to buy production for.
-        int chooseArmyTypeToBuy(City *c);
+        int chooseArmyTypeToBuy(City *c, bool quick);
 
         // Consider buying new production for this city
-        int maybeBuyProduction(City *c);
+        int maybeBuyProduction(City *c, bool quick);
         
         // Set the city to produce the best armies possible
+        void setProduction(City *c);
         int setBestProduction(City *c);
+        int setQuickProduction(City *c);
         
         // assign a score to an army type to try to figure out which is best
-        int scoreArmyType(const ArmyProto *proto);
-        int scoreArmyType(const ArmyProdBase *proto);
+        // which is best for buying
+        int scoreBestArmyType(const ArmyProto *proto);
+        //which is best for producing
+        int scoreBestArmyType(const ArmyProdBase *a);
+        //which is quickest for producing.
+        int scoreQuickArmyType(const ArmyProdBase *proto);
+        //which is quickest for buying
+        int scoreQuickArmyType(const ArmyProto *proto);
 
         // suggest somewhere that a hero stack might like to visit
         Location *getAlternateHeroTarget(Stack *s);
@@ -122,10 +136,11 @@ class AI_Smart : public RealPlayer
 	// buy a scout if we need to
 	void maybeBuyScout();
 
+        bool cityNewlyTaken(City *city, guint32 turns = 2) const;
+
         // DATA
         int d_mustmakemoney;  // used to avoid to buy new production 
                               // and to reinforce cities to earn more money
-	bool d_abort_requested;
 
 };
 
