@@ -132,10 +132,14 @@ class Army :public ArmyBase, public UniquelyIdentified, public Ownable, public s
 	static Army* createNonUniqueArmy(const ArmyProto& a, Player *p=NULL);
 	static Army* createNonUniqueArmy(const ArmyProdBase& a, Player *p=NULL);
 
-        // Set functions:
+
+        // Set Methods
         
         //! Set the Id of Armyset and type that this Army belongs to.
         void setArmyset(guint32 armyset, guint32 type);
+	
+	//! Change the armyset that the army type for this army belongs to.
+	void setArmyset(guint32 armyset_id) {d_armyset = armyset_id;};
 
         //! Set an Army statistic.
         void setStat(Stat stat, guint32 value);
@@ -143,15 +147,6 @@ class Army :public ArmyBase, public UniquelyIdentified, public Ownable, public s
         //! Set the current number of hitpoints of this army.
         void setHP(guint32 hp) {d_hp = hp;}
 
-        /** 
-         * When you select a stack on the map, you can toggle the armies 
-	 * which will be moved when you click somewhere else on the map.
-         * Grouped==true means that this army will be among the selected
-         * armies.
-         */
-	//!Set the grouped state of this Army.
-        void setGrouped(bool grouped){d_grouped = grouped;}
-
         //! Sets whether or not the Army has a particular medal.
         void setMedalBonus(guint32 index, bool value) 
 	  {d_medal_bonus[index]=value;}
@@ -171,7 +166,8 @@ class Army :public ArmyBase, public UniquelyIdentified, public Ownable, public s
 	//! Sets whether or not this Army unit is in a tower.
 	void setFortified (bool f);
 
-        // Get functions
+
+        // Get Methods 
         
 	//! Get the Id of the Armyset to which the Army's type belongs.
         guint32 getArmyset() const {return d_armyset;}
@@ -208,9 +204,6 @@ class Army :public ArmyBase, public UniquelyIdentified, public Ownable, public s
         //! Get the current level of the Army.
         guint32 getLevel() const {return d_level;}
 
-        //! Returns the grouped state of the Army within a Stack.
-        bool isGrouped() const {return d_grouped;}
-
         //! Return which medals this Army unit has.
         bool* getMedalBonuses() const {return (bool*)&d_medal_bonus;}
 
@@ -227,11 +220,24 @@ class Army :public ArmyBase, public UniquelyIdentified, public Ownable, public s
         double getNumberHasBeenHit() const {return d_number_hasbeenhit;}
 
 	//! Return whether or not the Army is in a tower.
-	bool getFortified ();
+	bool getFortified () const;
 
-	//! Returns how many experience points the next level requires.
-        guint32 getXpNeededForNextLevel() const;
+	//! Is this army a hero?
+	/**
+	 * isHero is overridden by the Hero class.
+	 */
+	virtual bool isHero() const {return false;};
 
+	//! This army is of an army type that can be awarded as a reward.
+	bool getAwardable() const;
+
+	//! This army is of an army type that can be the keeper in a ruin.
+	bool getDefendsRuins() const;
+
+	//! This army is of an army type that has this name.
+	virtual std::string getName() const;
+
+	//Methods that operate on class data and modify the class data
         /** 
 	 * Regenerate an amount of the Army unit's hitpoints but not 
 	 * exceeding the maximum number of hitpoints.
@@ -283,39 +289,20 @@ class Army :public ArmyBase, public UniquelyIdentified, public Ownable, public s
         //! Increases the experience points of the Army by the given amount.
         void gainXp(double n);
 
-        //! Checks whether or not the Army unit can advance a level.
-        bool canGainLevel() const;
+	//! Make this army look and behave like another one.
+	void morph(const ArmyProto *armyproto);
 
-        /** 
-	 * Increase the Army unit's level, and increase one of three stats;
-	 * Stat::STRENGTH, Stat::MOVES, or Stat::SIGHT.
-	 *
-         * @param stat     The stat to increase.
-	 *
-         * @return How much the statistic increases or -1 upon error 
-	 *         (e.g. because the XP is not enough).
-         */
-	//! Increase the Army's level, and increase a given stat.
-        int gainLevel(Stat stat);
-
-	/**
-	 * Calculate how much a stat is increased because the Army unit
-	 * has increased it's level.
-	 *
-	 * @param stat  One of Stat::STRENGTH, Stat::MOVES, or Stat::SIGHT.
-	 *
-	 * @return The new value of the stat after it is increased.
-	 */
-	//! Return how much the stat would be boosted by gaining a level.
-        int computeLevelGain(Stat stat);
+	//Methods that operate on class data and do not modify the class data
 
 	//! Returns whether or not the army was blessed at the given temple.
-        bool blessedAtTemple(guint32 temple_id);
+        bool blessedAtTemple(guint32 temple_id) const;
 
         void printAllDebugInfo() const;
 
         //! Saves the Army to an opened saved-game file.
         virtual bool save(XML_Helper* helper) const;
+
+	//signals
         
 	/**
 	 * @note This signal is static because sometimes the army doesn't 
@@ -326,13 +313,6 @@ class Army :public ArmyBase, public UniquelyIdentified, public Ownable, public s
 	//! Emitted when an Army has died.
         static sigc::signal<void, Army*> sdying;
 
-	virtual bool isHero() const {return false;};
-
-	//take these values from the army type that the instance points to.
-	bool getAwardable() const;
-	bool getDefendsRuins() const;
-	virtual std::string getName() const;
-
     protected:
 
         //! Generic method for saving Army data.  Useful to the Hero class.
@@ -439,18 +419,6 @@ class Army :public ArmyBase, public UniquelyIdentified, public Ownable, public s
 	 */
         guint32 d_level;
 
-	//! Whether or not the Army unit is grouped into it's parent Stack.
-	/**
-	 * When splitting stacks, it is necessary to know which Army units
-	 * are staying in the original Stack, and which ones are going into
-	 * the new Stack.
-	 *
-	 * If an Army is grouped, it is going into the new Stack.
-	 *
-	 * When attacking, only the grouped Army units in the Stack attack.
-	 */
-        bool d_grouped;
-
 	/**
 	 * There are three different medals that an Army unit can win.
 	 *
