@@ -29,6 +29,8 @@
 #include "defs.h"
 #include "shield.h"
 #include "File.h"
+#include "set.h"
+#include "hero.h"
 
 using namespace std;
 
@@ -48,7 +50,7 @@ using namespace std;
  *
  * Armysets are most often referred to by their Id (Armyset::d_id), but may 
  * sometimes be referred to by their name (Armyset::d_name) or subdirectory 
- * name (Armyset::d_dir).
+ * name (Armyset::d_subdir).
  *
  * Armyset objects are loaded from an armyset configuration file.
  *
@@ -58,14 +60,15 @@ using namespace std;
  * player's forces, but in practise there is only one Armyset per scenario.
  *
  * The armyset configuration file is a same named XML file inside the Armyset's
- * directory.  E.g. army/${Armyset::d_dir}/${Armyset::d_dir}.xml.
+ * directory.  E.g. army/${Armyset::d_subdir}/${Armyset::d_subdir}.xml.
  */
-class Armyset: public std::list<ArmyProto *>, public sigc::trackable
+class Armyset: public std::list<ArmyProto *>, public sigc::trackable, public Set
 {
     public:
 
 	//! The xml tag of this object in an armyset configuration file.
 	static std::string d_tag; 
+	static std::string file_extension; 
 
 	//! Default constructor.
 	/**
@@ -80,9 +83,10 @@ class Armyset: public std::list<ArmyProto *>, public sigc::trackable
 	/**
 	 * Load armyset XML entities from armyset configuration files.
 	 */
-        Armyset(XML_Helper* helper, bool private_collection = false);
+        Armyset(XML_Helper* helper, std::string directory);
+
+	static Armyset *create(std::string filename);
 
-	static Armyset *create(std::string filename, bool private_collection = false);
 	//! Destructor.
         ~Armyset();
 
@@ -116,9 +120,6 @@ class Armyset: public std::list<ArmyProto *>, public sigc::trackable
         guint32 getId() const {return d_id;}
 
 	//! Set the unique identifier for this armyset.
-	/**
-	 * @note This method is only used in the armyset editor.  
-	 */
         void setId(guint32 id) {d_id = id;}
 
 	//! Returns the name of the armyset.
@@ -136,6 +137,18 @@ class Armyset: public std::list<ArmyProto *>, public sigc::trackable
 	 */
         void setName(std::string name) {d_name = name;}
 
+	//! Get the copyright holders for this armyset.
+	std::string getCopyright() const {return d_copyright;};
+
+	//! Set the copyright holders on the armyset.
+	void setCopyright(std::string copy) {d_copyright = copy;};
+
+	//! Get the license of this armyset.
+	std::string getLicense() const {return d_license;};
+
+	//! Set the license for this armyset.
+	void setLicense(std::string license) {d_license = license;};
+
 	//! Get the subdirectory name of the armyset.
 	/**
 	 * This value does not contain a path (e.g. no slashes).  It is the
@@ -143,10 +156,10 @@ class Armyset: public std::list<ArmyProto *>, public sigc::trackable
 	 *
 	 * @return The name of the subdirectory the Armyset is held in.
 	 */
-        std::string getSubDir() const {return d_dir;}
+        std::string getSubDir() const {return d_subdir;}
 
 	//! Set the subdirectory that the armyset is in.
-        void setSubDir(std::string dir) {d_dir = dir;}
+        void setSubDir(std::string dir) {d_subdir = dir;}
 
 	//! Get the image of the stack in a ship (minus the mask).
 	PixMask* getShipPic() const {return d_ship;}
@@ -160,6 +173,12 @@ class Armyset: public std::list<ArmyProto *>, public sigc::trackable
 	//! Set the mask portion of the image of the stack in a ship.
 	void setShipMask(PixMask* shipmask) {d_shipmask = shipmask;};
 
+	//! Get the image of the bag.
+	PixMask* getBagPic() const {return d_bag;}
+
+	//! Set the image of the bag.
+	void setBagPic(PixMask* s) {d_bag = s;};
+
 	//! Get the image of the planted standard (minus the mask).
 	PixMask* getStandardPic() const {return d_standard;}
 
@@ -184,6 +203,12 @@ class Armyset: public std::list<ArmyProto *>, public sigc::trackable
 	//! Get the name of the file holding the image of the hero's flag.
 	std::string getStandardImageName() {return d_standard_name;};
 
+	//! Set the name of the file holding the image of the bag.
+	void setBagImageName(std::string n) {d_bag_name = n;};
+
+	//! Get the name of the file holding the image of the bag.
+	std::string getBagImageName() {return d_bag_name;};
+
 	//! Find an army with a type in this armyset.
 	/**
 	 * Scan the Army prototype objects in this Armyset and return it.
@@ -197,12 +222,15 @@ class Armyset: public std::list<ArmyProto *>, public sigc::trackable
 	 * @return The Army with the given army type id, or NULL if none
 	 *         could be found.
 	 */
-	ArmyProto * lookupArmyByType(guint32 army_type);
+	ArmyProto * lookupArmyByType(guint32 army_type) const;
 
-	//! Return whether this is an armyset in the user's personal collection.
-	bool fromPrivateCollection() {return private_collection;};
+	ArmyProto * lookupArmyByName(std::string name) const;
 
+	ArmyProto * lookupArmyByStrengthAndTurns(guint32 str, guint32 turns) const;
 
+	ArmyProto * lookupArmyByGender(Hero::Gender gender) const;
+
+	ArmyProto * lookupSimilarArmy(ArmyProto *army) const;
 	//! can this armyset be used within the game?
 	bool validate();
 	bool validateSize();
@@ -212,22 +240,32 @@ class Armyset: public std::list<ArmyProto *>, public sigc::trackable
 	bool validateAwardables();
 	bool validateShip();
 	bool validateStandard();
+	bool validateBag();
 	bool validateArmyUnitImages();
 	bool validateArmyUnitImage(ArmyProto *a, Shield::Colour &c);
 	bool validateArmyUnitNames();
 	bool validateArmyUnitName(ArmyProto *a);
 	//! get filenames in this armyset, excepting the configuration file.
 	void getFilenames(std::list<std::string> &files);
+	void instantiateImages();
+	void uninstantiateImages();
+	void loadStandardPic(std::string image_filename);
+	void loadShipPic(std::string image_filename);
+	void loadBagPic(std::string image_filename);
+
+	std::string getConfigurationFile();
+	static std::list<std::string> scanUserCollection();
+	static std::list<std::string> scanSystemCollection();
+	static void switchArmyset(Army *army, const Armyset *armyset);
+	static void switchArmyset(ArmyProdBase *army, const Armyset *armyset);
+	static void switchArmysetForRuinKeeper(Army *army, const Armyset *armyset);
+	const ArmyProto * getRandomRuinKeeper() const;
+	const ArmyProto *getRandomAwardableAlly() const;
+
     private:
 
         //! Callback function for the army tag (see XML_Helper)
         bool loadArmyProto(std::string tag, XML_Helper* helper);
-
-	//! Load the image and mask of the stack's ship picture.
-	void loadShipPic();
-
-	//! Load the image and mask of the planted standard.
-	void loadStandardPic();
         
 	//! The unique Id of this armyset.
 	/**
@@ -243,13 +281,19 @@ class Armyset: public std::list<ArmyProto *>, public sigc::trackable
 	 */
         std::string d_name;
 
+	//! The armyset has these copyright holders.
+	std::string d_copyright;
+
+	//! The license of the armyset.
+	std::string d_license;
+
 	//! The subdirectory of the Armyset.
 	/**
 	 * This is the name of the subdirectory that the Armyset files are
 	 * residing in.  It does not contain a path (e.g. no slashes).
 	 * Armyset files sit in the army/ directory.
 	 */
-        std::string d_dir;
+        std::string d_subdir;
 
 	//! The size of each army tile as rendered in the game.
 	/**
@@ -274,14 +318,17 @@ class Armyset: public std::list<ArmyProto *>, public sigc::trackable
 	//! The mask of what to shade with the player's colour on the standard.
 	PixMask* d_standard_mask;
 
+	//! The picture of an item when it's lying on the ground.
+	PixMask *d_bag;
+
 	//! The name of the file that holds the picture of the hero's flag.
 	std::string d_standard_name;
 
 	//! The name of the file that holds the picture of stack on water.
 	std::string d_stackship_name;
 
-	//! Whether this is a system armyset, or one that the user made.
-	bool private_collection;
+	//! The name of the file that holds the picture of the sack of items.
+	std::string d_bag_name;
 };
 
 #endif // ARMYSET_H
