@@ -46,8 +46,9 @@ using namespace std;
 //#define debug(x) {cerr<<__FILE__<<": "<<__LINE__<<": "<<x<<endl<<flush;}
 #define debug(x)
 
-City::City(Vector<int> pos, string name, guint32 gold, guint32 numslots)
-    :Ownable((Player *)0), Location(pos, City::getWidth()), Renamable(name),
+City::City(Vector<int> pos, guint32 width, string name, guint32 gold, 
+	   guint32 numslots)
+    :Ownable((Player *)0), Location(pos, width), Renamable(name),
     ProdSlotlist(numslots), d_gold(gold), d_defense_level(1), d_burnt(false), 
     d_vectoring(false), d_vector(Vector<int>(-1,-1)), 
     d_capital(false), d_capital_owner(0)
@@ -62,8 +63,8 @@ City::City(Vector<int> pos, string name, guint32 gold, guint32 numslots)
       }
 }
 
-City::City(XML_Helper* helper)
-    :Ownable(helper), Location(helper, City::getWidth()), Renamable(helper),
+City::City(XML_Helper* helper, guint32 width)
+    :Ownable(helper), Location(helper, width), Renamable(helper),
     ProdSlotlist(helper)
 {
     //initialize the city
@@ -109,6 +110,14 @@ City::City(const City& c)
 {
 }
 
+City::City(const City& c, Vector<int> pos)
+    :Ownable(c), Location(c, pos), Renamable(c), ProdSlotlist(c),
+    d_gold(c.d_gold), d_defense_level(c.d_defense_level), d_burnt(c.d_burnt),
+    d_vectoring(c.d_vectoring),d_vector(c.d_vector), d_capital(c.d_capital), 
+    d_capital_owner(c.d_capital_owner)
+{
+}
+
 City::~City()
 {
 }
@@ -140,34 +149,6 @@ bool City::save(XML_Helper* helper) const
     return retval;
 }
 
-bool City::raiseDefense()
-{
-    if (d_defense_level == (int) CITY_LEVELS)
-    {
-        // no further progress possible
-        return false;
-    }
-
-    // increase the defense level, the income and the possible production
-    d_defense_level++;
-
-    return true;
-}
-
-bool City::reduceDefense()
-{
-    if (d_defense_level == 1)
-    {
-        // no further reduction possible
-        return false;
-    }
-
-    // the same as raiseDefense, but the other way round
-    d_defense_level--;
-
-    return true;
-}
-
 void City::conquer(Player* newowner)
 {
   Citylist::getInstance()->stopVectoringTo(this);
@@ -191,8 +172,7 @@ void City::produceStrongestProductionBase()
   if (getNoOfProductionBases() == 0)
     return;
 
-  Stack* stack = getFreeStack(d_owner);
-  if (stack)
+  if (!isFull(d_owner))
     {
       unsigned int max_strength = 0;
       int strong_idx = -1;
@@ -211,7 +191,8 @@ void City::produceStrongestProductionBase()
 
       int savep = d_active_production_slot;
       setActiveProductionSlot(strong_idx);
-      produceArmy();
+      Vector<int> pos;
+      produceArmy(pos);
       setActiveProductionSlot(savep);
       return;
     }
@@ -234,8 +215,7 @@ void City::produceWeakestProductionBase()
   if (getNoOfProductionBases() == 0)
     return;
 
-  Stack* stack = getFreeStack(d_owner);
-  if (stack)
+  if (!isFull(d_owner))
     {
       unsigned int min_strength = 100;
       int weak_idx = -1;
@@ -254,13 +234,14 @@ void City::produceWeakestProductionBase()
 
       int savep = d_active_production_slot;
       setActiveProductionSlot(weak_idx);
-      produceArmy();
+      Vector<int> pos;
+      produceArmy(pos);
       setActiveProductionSlot(savep);
       return;
     }
 }
 
-const Army *City::armyArrives()
+const Army *City::armyArrives(Vector<int> &pos)
 {
   // vector the army to the new spot
   if (d_vectoring)
@@ -278,7 +259,7 @@ const Army *City::armyArrives()
     }
   else //or make it here
     {
-      return produceArmy();
+      return produceArmy(pos);
     }
   return NULL;
 }
@@ -305,17 +286,6 @@ void City::nextTurn()
     }
 }
 
-int City::getGoldNeededForUpgrade() const
-{
-  if (d_defense_level == 1)
-    return 1000;
-  else if (d_defense_level == 2)
-    return 2000;
-  else if (d_defense_level == 3)
-    return 3000;
-  return -1;
-}
-
 void City::setVectoring(Vector<int> p) 
 {
   d_vector = p;
@@ -329,7 +299,7 @@ void City::setVectoring(Vector<int> p)
     }
 }
 
-Army *City::produceArmy()
+Army *City::produceArmy(Vector<int> &pos)
 {
   // add produced army to stack
   if (d_active_production_slot == -1)
@@ -344,7 +314,8 @@ Army *City::produceArmy()
     return NULL;
 
   Army *a = new Army(*(getProductionBase(d_active_production_slot)), d_owner);
-  GameMap::getInstance()->addArmy(this, a);
+  Stack *s = GameMap::getInstance()->addArmy(this, a);
+  pos = s->getPos();
 
   if (d_owner == Playerlist::getInstance()->getNeutral()) 
     {
@@ -361,12 +332,12 @@ Army *City::produceArmy()
   return a;
 }
 
-bool City::canAcceptMoreVectoring()
+bool City::canAcceptMoreVectoring() const
 {
   return canAcceptMoreVectoring(0);
 }
 
-bool City::canAcceptMoreVectoring(guint32 number_of_cities)
+bool City::canAcceptMoreVectoring(guint32 number_of_cities) const
 {
   //here we presume that it's one unit per city
   guint32 num = Citylist::getInstance()->countCitiesVectoringTo(this);
@@ -383,10 +354,15 @@ bool City::changeVectorDestination(Vector<int> dest)
   return true;
 }
 
-guint32 City::countDefenders()
+std::vector<Stack *> City::getDefenders() const
+{
+  return getOwner()->getStacklist()->getDefendersInCity(this);
+}
+
+guint32 City::countDefenders() const
 {
   std::vector<Stack*> defenders;
-  defenders = getOwner()->getStacklist()->defendersInCity(this);
+  defenders = getDefenders();
 
   guint32 armies = 0;
   std::vector<Stack*>::iterator it = defenders.begin();
@@ -539,4 +515,21 @@ void City::setRandomArmytypes(bool produce_allies, int likely)
   sortProduction();
 }
 
+guint32 City::calculateDefenseLevel() const
+{
+  int num_production_bases = getNoOfProductionBases();
+  if (isBurnt()) 
+    return 0;
+  else if (num_production_bases <= 2 && 
+	   getOwner() == Playerlist::getInstance()->getNeutral())
+    return 1;
+  else if (num_production_bases <= 2)
+    return 2;
+  else if (num_production_bases > 2 && 
+	   getOwner() == Playerlist::getInstance()->getNeutral())
+    return 2;
+  else if (num_production_bases > 2)
+    return 3;
+  return 0;
+}
 // End of file
