@@ -42,7 +42,6 @@ class Hero;
 
 #define DEFAULT_CITY_NAME "Noname"
 #define DEFAULT_CITY_INCOME 20
-#define CITY_TILE_WIDTH 2
 
 //! A City on the game map.
 /**
@@ -77,28 +76,66 @@ class City : public Ownable, public Location, public Renamable,
           * Make a new city object.
 	  *
           * @param pos       The location of the city on the game map.
+	  * @param width     The number of tiles this city spans.
           * @param name      The name of the city.
           * @param gold      The amount of gold the city produces each turn.
 	  * @param numslots  The number of production slots for this city.
           */
-        City(Vector<int> pos, std::string name = DEFAULT_CITY_NAME, 
+        City(Vector<int> pos, guint32 width, 
+	     std::string name = DEFAULT_CITY_NAME, 
 	     guint32 gold = DEFAULT_CITY_INCOME, 
 	     guint32 numslots = MAX_PRODUCTION_SLOTS_IN_A_CITY);
+
 	//! Copy constructor.
         City(const City&);
+
+	//! Alternative copy constructor.
+	/**
+	 * Make a city by copying another one, but it has a different position.
+	 */
+        City(const City&, Vector<int> pos);
+
         //! Loading constructor.
 	/**
 	 * Make a new city object by reading it from a saved-game file.
 	 *
 	 * @param helper The opened saved-game file to load the City from.
 	 */
-        City(XML_Helper* helper);
+        City(XML_Helper* helper, guint32 width);
+
 	//! Destructor.
         ~City();
 
-        //! Save the city to an opened saved-game file.
-        bool save(XML_Helper* helper) const;
-        
+	//Get Methods
+	
+        //! Return the defense level of the city.
+        int getDefenseLevel() const {return calculateDefenseLevel();}
+
+        //! Return the income of the city per turn.
+        guint32 getGold() const {return d_gold;}
+
+        //! Returns whether or not the city has been destroyed.
+        bool isBurnt() const {return d_burnt;}
+
+        //! Returns whether or not the city is a capital city.
+        bool isCapital() const {return d_capital;}
+
+        //! Returns the original owner of this capital city.
+        Player *getCapitalOwner() const {return d_capital_owner;}
+
+        //! Get the position where the city will send produced armies.
+	/**
+	 * @return The position on the map where the city will send it's newly
+	 *         produced armies.  Returns (-1,-1) if the city is not
+	 *         vectoring.
+	 */
+        Vector<int> getVectoring() const {return d_vector;}
+
+	bool isUnnamed() const {return getName() == getDefaultName() ? true : false;};
+
+
+	//Set Methods
+
         //! Set the gold the city produces each turn.
         void setGold(guint32 gold){d_gold = gold;}
 
@@ -111,14 +148,19 @@ class City : public Ownable, public Location, public Renamable,
         //! Sets whether the city is a capital.
         void setCapitalOwner(Player *p) {d_capital_owner = p;}
 
-        //! Raise the defense level by one. Return true on success.
-        bool raiseDefense();
-
-        //! Lower the defense level by one. Return true on success.
-	/**
-	 * This method is not used.
+        //! Set the point where the city will send the produced armies.
+        /**
+	 * @note This method does not check to see if the destination point
+	 * can receive yet another vectored unit.
+	 *
+	 * @param pos  The position on the map to send the produced units to.
+	 *             The position must point to another city, or a planted
+	 *             standard.  Set to (-1,-1) if not vectoring.
 	 */
-        bool reduceDefense();
+        void setVectoring(Vector<int> pos);
+
+
+	// Methods that operate on class data and modify the class.
 
         //! Changes the owner of the city and prepares it for takeover.
 	/**
@@ -160,76 +202,6 @@ class City : public Ownable, public Location, public Renamable,
 	 */
         void nextTurn();
 
-        //! Return the defense level of the city.
-        int getDefenseLevel() const {return d_defense_level;}
-
-	//! Returns the amount of gold needed to increase the defense level.
-	/**
-	 * @note This method isn't called.  Cities are not upgraded.
-	 *
-	 * Returns the number of gold pieces needed to increase the defense
-	 *         level, or -1 if city can't be upgraded.
-	 */
-	int getGoldNeededForUpgrade() const; 
-	
-        //! Return the income of the city per turn.
-        guint32 getGold() const {return d_gold;}
-
-        //! Returns whether or not the city has been destroyed.
-        bool isBurnt() const {return d_burnt;}
-
-        //! Returns whether or not the city is a capital city.
-        bool isCapital() const {return d_capital;}
-
-        //! Returns the original owner of this capital city.
-        Player *getCapitalOwner() const {return d_capital_owner;}
-
-        //! Set the point where the city will send the produced armies.
-        /**
-	 * @note This method does not check to see if the destination point
-	 * can receive yet another vectored unit.
-	 *
-	 * @param pos  The position on the map to send the produced units to.
-	 *             The position must point to another city, or a planted
-	 *             standard.  Set to (-1,-1) if not vectoring.
-	 */
-        void setVectoring(Vector<int> pos);
-
-        //! Get the position where the city will send produced armies.
-	/**
-	 * @return The position on the map where the city will send it's newly
-	 *         produced armies.  Returns (-1,-1) if the city is not
-	 *         vectoring.
-	 */
-        Vector<int> getVectoring() const {return d_vector;}
-
-	//! Returns true if the city isn't accepting too many vectored armies.
-	/**
-	 * Scans all of the cities vectoring to this city.  If vectoring
-	 * to this city would accrue the count to 
-	 * MAX_CITIES_VECTORED_TO_ONE_CITY, this method returns false.
-	 *
-	 * @return True if the city can have another city vectoring to it.
-	 *         Otherwise false.
-	 */
-	bool canAcceptMoreVectoring();
-
-	//! Returns true if the city can accept vectoring from a set of cities.
-	/**
-	 * Instead of checking to see if one unit can be vectored here like in 
-	 * City::canAcceptMoreVectoring, this method checks if the city can
-	 * receive multiple units (from an equal number of cities, because a
-	 * city can only produce and vector one army at a time).
-	 *
-	 * @param number_of_cities The number of cities to check to
-	 *                         see if this city can receive on top of what
-	 *                         it's already receiving.
-	 *
-	 * @return True if the city can have this many more cities vectoring 
-	 *         to it.  Otherwise false.
-	 */
-	bool canAcceptMoreVectoring(guint32 number_of_cities);
-
 	//! Changes the vectoring destination, but for en-route units too.
 	/**
 	 * This method acts like City::setVectoring but also changes the
@@ -246,24 +218,60 @@ class City : public Ownable, public Location, public Renamable,
 	 */
 	bool changeVectorDestination(Vector<int> dest);
 
+	//! Callback that makes the army show up.
+	const Army *armyArrives(Vector<int> &pos);
+
+	
+	// Methods that operate on class data and do not modify the class.
+
+        //! Save the city to an opened saved-game file.
+        bool save(XML_Helper* helper) const;
+        
+	//! Returns true if the city isn't accepting too many vectored armies.
+	/**
+	 * Scans all of the cities vectoring to this city.  If vectoring
+	 * to this city would accrue the count to 
+	 * MAX_CITIES_VECTORED_TO_ONE_CITY, this method returns false.
+	 *
+	 * @return True if the city can have another city vectoring to it.
+	 *         Otherwise false.
+	 */
+	bool canAcceptMoreVectoring() const;
+
+	//! Returns true if the city can accept vectoring from a set of cities.
+	/**
+	 * Instead of checking to see if one unit can be vectored here like in 
+	 * City::canAcceptMoreVectoring, this method checks if the city can
+	 * receive multiple units (from an equal number of cities, because a
+	 * city can only produce and vector one army at a time).
+	 *
+	 * @param number_of_cities The number of cities to check to
+	 *                         see if this city can receive on top of what
+	 *                         it's already receiving.
+	 *
+	 * @return True if the city can have this many more cities vectoring 
+	 *         to it.  Otherwise false.
+	 */
+	bool canAcceptMoreVectoring(guint32 number_of_cities) const;
+
 	//! Return how many armies are in the city.
-	guint32 countDefenders();
+	guint32 countDefenders() const;
 
-	//! This makes the army show up.  called when it's time.
-	const Army *armyArrives();
-
-	bool isUnnamed() {return getName() == getDefaultName() ? true : false;};
+	//! Return the stacks that are inside the city walls.
+	std::vector<Stack *> getDefenders() const;
 
+	// Static Methods
+	
+	//! Get the default name of any city.
 	static std::string getDefaultName() {return _(DEFAULT_CITY_NAME);};
 
-	static guint32 getWidth() {return CITY_TILE_WIDTH;};
-
     private:
 
+	//! Callback for loading city objects from a saved-game file.
 	bool load(std::string tag, XML_Helper *helper);
 
         //! Produces the currently active Army production base.
-        Army * produceArmy();
+        Army * produceArmy(Vector<int> &pos);
 
 	//! Makes an Army production base be a little different than expected.
 	/**
@@ -282,6 +290,13 @@ class City : public Ownable, public Location, public Renamable,
 	 */
 	void sortProduction();
 
+	//! Calculate defense level.
+	/**
+	 * The defense level is a function of how many production slots
+	 * the city has.
+	 */
+	guint32 calculateDefenseLevel() const;
+
         // DATA
 
 	//! The City gives the Player this much gold per turn.
