@@ -20,7 +20,6 @@
 //  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 
 //  02110-1301, USA.
 
-#include <assert.h>
 #include <sigc++/functors/mem_fun.h>
 
 #include "citylist.h"
@@ -31,6 +30,9 @@
 #include "hero.h"
 #include "stack.h"
 #include "armyprodbase.h"
+#include "GameMap.h"
+#include "cityset.h"
+#include "PathCalculator.h"
 
 //#define debug(x) {cerr<<__FILE__<<": "<<__LINE__<<": "<<x<<endl<<flush;}
 #define debug(x)
@@ -106,7 +108,7 @@ int Citylist::countCities(Player* player) const
     return cities;
 }
 
-void Citylist::collectTaxes(Player* p)
+void Citylist::collectTaxes(Player* p) const
 {
   // Collect the taxes
   for (const_iterator it = begin(); it != end(); it++)
@@ -116,7 +118,7 @@ void Citylist::collectTaxes(Player* p)
 }
 
 //calculate the amount of money new armies will cost in the upcoming turn.
-guint32 Citylist::calculateUpcomingUpkeep(Player *p)
+guint32 Citylist::calculateUpcomingUpkeep(Player *p) const
 {
   guint32 total = 0;
   for (const_iterator it = begin(); it != end(); it++)
@@ -129,6 +131,7 @@ guint32 Citylist::calculateUpcomingUpkeep(Player *p)
 	      continue;
 	    const ArmyProdBase *a = (*it)->getProductionBase(slot);
 	    total += a->getUpkeep();
+	    total += a->getProductionCost();
 	  }
       }
   return total;
@@ -144,12 +147,10 @@ void Citylist::nextTurn(Player* p)
     //we've already collected taxes this round, so hopefully our
     //treasury has enough money to pay our city upkeep.
 
-    //fixme: here we want the upkeep that we would pay out to all cities.
-    //and not the total upkeep.
-    guint32 upkeep_for_new_armies = calculateUpcomingUpkeep(p);
-    if (p->getGold() < (int)upkeep_for_new_armies)
+    guint32 cost_of_new_armies = calculateUpcomingUpkeep(p);
+    if (p->getGold() < (int)cost_of_new_armies)
       {
-	int diff = upkeep_for_new_armies - p->getGold();
+	int diff = cost_of_new_armies - p->getGold();
 	//then we have to turn off enough production to make up for diff
 	//gold pieces.
 	for (iterator it = begin(); it != end(); it++)
@@ -181,7 +182,7 @@ void Citylist::nextTurn(Player* p)
 
 static bool isFogged(void *object)
 {
-  return ((City*)object)->isVisible(Playerlist::getActiveplayer()) == false;
+  return ((City*)object)->isVisible(Playerlist::getViewingplayer()) == false;
 }
 
 static bool isBurnt(void *object)
@@ -225,7 +226,7 @@ static bool canNotAcceptMoreVectoring(void *object)
   return true;
 }
 
-City* Citylist::getNearestEnemyCity(const Vector<int>& pos)
+City* Citylist::getNearestEnemyCity(const Vector<int>& pos) const
 {
   std::list<bool (*)(void *)> filters;
   filters.push_back(isBurnt);
@@ -233,8 +234,23 @@ City* Citylist::getNearestEnemyCity(const Vector<int>& pos)
   return getNearestObject(pos, &filters);
 }
 
+City* Citylist::getClosestEnemyCity(const Stack *stack) const
+{
+  std::list<bool (*)(void *)> filters;
+  filters.push_back(isBurnt);
+  filters.push_back(isNotOwnedByEnemy);
+  return getClosestObject(stack, &filters);
+}
 
-City* Citylist::getNearestForeignCity(const Vector<int>& pos)
+City* Citylist::getClosestForeignCity(const Stack *stack) const
+{
+  std::list<bool (*)(void *)> filters;
+  filters.push_back(isBurnt);
+  filters.push_back(isOwnedByActivePlayer);
+  return getClosestObject(stack, &filters);
+}
+
+City* Citylist::getNearestForeignCity(const Vector<int>& pos) const
 {
   std::list<bool (*)(void *)> filters;
   filters.push_back(isBurnt);
@@ -242,7 +258,7 @@ City* Citylist::getNearestForeignCity(const Vector<int>& pos)
   return getNearestObject(pos, &filters);
 }
 
-City* Citylist::getNearestCity(const Vector<int>& pos, int dist)
+City* Citylist::getNearestCity(const Vector<int>& pos, int dist) const
 {
   City *c = getNearestCity(pos);
   if (!c)
@@ -253,7 +269,7 @@ City* Citylist::getNearestCity(const Vector<int>& pos, int dist)
   return NULL;
 }
 
-City* Citylist::getNearestFriendlyCity(const Vector<int>& pos, int dist)
+City* Citylist::getNearestFriendlyCity(const Vector<int>& pos, int dist) const 
 {
   City *c = getNearestFriendlyCity(pos);
   if (!c)
@@ -264,18 +280,18 @@ City* Citylist::getNearestFriendlyCity(const Vector<int>& pos, int dist)
   return NULL;
 }
 
-City* Citylist::getNearestFriendlyCity(const Vector<int>& pos)
+City* Citylist::getNearestFriendlyCity(const Vector<int>& pos) const
 {
     Player* p = Playerlist::getInstance()->getActiveplayer();
     return getNearestCity (pos, p);
 }
 
-City* Citylist::getNearestCity(const Vector<int>& pos, Player *p)
+City* Citylist::getNearestCity(const Vector<int>& pos, Player *p) const
 {
     int diff = -1;
-    iterator diffit;
+    const_iterator diffit;
     
-    for (iterator it = begin(); it != end(); ++it)
+    for (const_iterator it = begin(); it != end(); ++it)
     {
         if ((*it)->isBurnt())
             continue;
@@ -299,14 +315,55 @@ City* Citylist::getNearestCity(const Vector<int>& pos, Player *p)
     return (*diffit);
 }
 
-City* Citylist::getNearestCity(const Vector<int>& pos)
+City* Citylist::getClosestCity(const Stack *stack, Player *p) const
+{
+    int diff = -1;
+    const_iterator diffit;
+    PathCalculator pc(stack, true, 0, 0);
+    
+    for (const_iterator it = begin(); it != end(); ++it)
+    {
+        if ((*it)->isBurnt())
+            continue;
+
+        if ((*it)->getOwner() == p)
+        {
+            int delta = pc.calculate((*it)->getPos());
+            if (delta <= 0)
+              continue;
+            
+            if ((diff > delta) || (diff == -1))
+            {
+                diff = delta;
+                diffit = it;
+            }
+        }
+    }
+    
+    if (diff == -1) return 0;
+    return (*diffit);
+}
+
+City* Citylist::getClosestFriendlyCity(const Stack *stack) const
+{
+    Player* p = Playerlist::getInstance()->getActiveplayer();
+    return getClosestCity (stack, p);
+}
+City* Citylist::getClosestCity(const Stack *stack) const
+{
+  std::list<bool (*)(void *)> filters;
+  filters.push_back(isBurnt);
+  return getClosestObject(stack, &filters);
+}
+
+City* Citylist::getNearestCity(const Vector<int>& pos) const
 {
   std::list<bool (*)(void *)> filters;
   filters.push_back(isBurnt);
   return getNearestObject(pos, &filters);
 }
 
-City* Citylist::getNearestVisibleCity(const Vector<int>& pos)
+City* Citylist::getNearestVisibleCity(const Vector<int>& pos) const
 {
   std::list<bool (*)(void *)> filters;
   filters.push_back(isBurnt);
@@ -314,7 +371,7 @@ City* Citylist::getNearestVisibleCity(const Vector<int>& pos)
   return getNearestObject(pos, &filters);
 }
 
-City* Citylist::getNearestVisibleCity(const Vector<int>& pos, int dist)
+City* Citylist::getNearestVisibleCity(const Vector<int>& pos, int dist) const
 {
   City *c = getNearestVisibleCity(pos);
   if (!c)
@@ -325,7 +382,7 @@ City* Citylist::getNearestVisibleCity(const Vector<int>& pos, int dist)
   return NULL;
 }
 
-City* Citylist::getNearestVisibleFriendlyCity(const Vector<int>& pos, int dist)
+City* Citylist::getNearestVisibleFriendlyCity(const Vector<int>& pos, int dist) const
 {
   City *c = getNearestFriendlyCity(pos);
   if (!c)
@@ -337,7 +394,7 @@ City* Citylist::getNearestVisibleFriendlyCity(const Vector<int>& pos, int dist)
 }
 
 
-City* Citylist::getNearestVisibleFriendlyCity(const Vector<int>& pos)
+City* Citylist::getNearestVisibleFriendlyCity(const Vector<int>& pos) const
 {
   std::list<bool (*)(void *)> filters;
   filters.push_back(isBurnt);
@@ -346,7 +403,7 @@ City* Citylist::getNearestVisibleFriendlyCity(const Vector<int>& pos)
   return getNearestObject(pos, &filters);
 }
 
-City* Citylist::getNearestNeutralCity(const Vector<int>& pos)
+City* Citylist::getNearestNeutralCity(const Vector<int>& pos) const
 {
   std::list<bool (*)(void *)> filters;
   filters.push_back(isBurnt);
@@ -354,7 +411,7 @@ City* Citylist::getNearestNeutralCity(const Vector<int>& pos)
   return getNearestObject(pos, &filters);
 }
 
-City* Citylist::getNearestFriendlyVectorableCity(const Vector<int>& pos)
+City* Citylist::getNearestFriendlyVectorableCity(const Vector<int>& pos) const
 {
   std::list<bool (*)(void *)> filters;
   filters.push_back(isBurnt);
@@ -363,9 +420,9 @@ City* Citylist::getNearestFriendlyVectorableCity(const Vector<int>& pos)
   return getNearestObject(pos, &filters);
 }
 
-City* Citylist::getFirstCity(Player* p)
+City* Citylist::getFirstCity(Player* p) const
 {
-    for (iterator it = begin(); it != end(); it++)
+    for (const_iterator it = begin(); it != end(); it++)
         if ((*it)->getOwner() == p)
             return (*it);
 
@@ -390,7 +447,8 @@ bool Citylist::load(std::string tag, XML_Helper* helper)
 {
     if (tag == City::d_tag)
       {
-	City *c = new City(helper);
+	Cityset *cs = GameMap::getInstance()->getCityset();
+	City *c = new City(helper, cs->getCityTileWidth());
 	add(c);
 	return true;
       }
@@ -423,36 +481,9 @@ void Citylist::stopVectoringTo(City *c)
   return;
 }
 
-Vector<int> Citylist::calculateCenterOfTerritory (Player *p)
+bool Citylist::isVectoringTarget(City *target) const
 {
-  int n = INT_MAX;
-  int s = 0;
-  int w = INT_MAX;
-  int e = 0;
-  int count = 0;
-  for (iterator it = begin(); it != end(); it++)
-    {
-      if (p && (*it)->getOwner() == p)
-	continue;
-      Vector<int> pos = (*it)->getPos();
-      count++;
-      if (pos.x > e)
-	e = pos.x;
-      if (pos.x < w)
-	w = pos.x;
-      if (pos.y > s)
-	s = pos.y;
-      if (pos.y < n)
-	n = pos.y;
-    }
-  if (count == 0)
-    return Vector<int>(-1, -1);
-  return Vector<int>((s-n)/2, (e-w)/2);
-}
-
-bool Citylist::isVectoringTarget(City *target)
-{
-  for (iterator it = begin(); it != end(); it++)
+  for (const_iterator it = begin(); it != end(); it++)
     {
       if ((*it)->getOwner() != target->getOwner())
 	continue;
@@ -462,20 +493,21 @@ bool Citylist::isVectoringTarget(City *target)
   return false;
 }
 
-std::list<City*> Citylist::getCitiesVectoringTo(Vector<int> target)
+std::list<City*> Citylist::getCitiesVectoringTo(Vector<int> target) const
 {
   std::list<City*> cities;
-  for (iterator it = begin(); it != end(); it++)
+  for (const_iterator it = begin(); it != end(); it++)
     {
       if (target == (*it)->getVectoring())
 	cities.push_back((*it));
     }
   return cities;
 }
-std::list<City*> Citylist::getCitiesVectoringTo(City *target)
+
+std::list<City*> Citylist::getCitiesVectoringTo(City *target) const
 {
   std::list<City*> cities;
-  for (iterator it = begin(); it != end(); it++)
+  for (const_iterator it = begin(); it != end(); it++)
     {
       if ((*it)->getOwner() != target->getOwner())
 	continue;
@@ -485,17 +517,17 @@ std::list<City*> Citylist::getCitiesVectoringTo(City *target)
   return cities;
 }
 
-City* Citylist::getNearestCityPast(const Vector<int>& pos, int dist)
+City* Citylist::getNearestCityPast(const Vector<int>& pos, int dist) const
 {
   std::list<bool (*)(void *)> filters;
   filters.push_back(isBurnt);
   return getNearestObjectAfter(pos, dist, &filters);
 }
 
-guint32 Citylist::countCitiesVectoringTo(City *dest)
+guint32 Citylist::countCitiesVectoringTo(const City *dest) const
 {
   guint32 count = 0;
-  for (iterator it = begin(); it != end(); it++)
+  for (const_iterator it = begin(); it != end(); it++)
     {
       City *c = *it;
       if (c->getOwner() != dest->getOwner())
@@ -507,4 +539,71 @@ guint32 Citylist::countCitiesVectoringTo(City *dest)
   return count;
 }
 
+std::list<City*> Citylist::getNearestFriendlyCities(Player *player, Vector<int> pos) const
+{
+  std::list<City*> cities;
+  for (const_iterator it = begin(); it != end(); it++)
+    {
+      City *c = *it;
+      if (c->getOwner() != player)
+        continue;
+      if (c->isBurnt() == true)
+        continue;
+      cities.push_back(c);
+    }
+  if (cities.size() == 0)
+    return cities;
+  std::list<int> distances;
+
+  if (pos == Vector<int>(-1,-1))
+    pos = getFirstCity(player)->getPos();
+
+  for (std::list<City*>::iterator it = cities.begin(); it != cities.end(); it++)
+    distances.push_back(dist((*it)->getNearestPos(pos), pos));
+
+  bool sorted = false;
+
+  while (!sorted)
+    {
+      sorted = true;
+
+      // setup
+      std::list<int>::iterator dit = distances.begin();
+      std::list<int>::iterator dnextit = distances.begin();
+      dnextit++;
+
+      std::list<City*>::iterator it = cities.begin();
+      std::list<City*>::iterator nextit = it;
+      nextit++;
+
+      for (; nextit != cities.end(); it++, nextit++, dit++, dnextit++)
+        if ((*dit) > (*dnextit))
+          {
+            // exchange the items in both lists
+            sorted = false;
+
+            City* tmp = (*nextit);
+            cities.erase(nextit);
+            nextit = it;
+            it = cities.insert(nextit, tmp);
+
+            int val = (*dnextit);
+            distances.erase(dnextit);
+            dnextit = dit;
+            dit = distances.insert(dnextit, val);
+          }
+    }
+  return cities;
+}
+
+City *Citylist::getCapitalCity(Player *player) const
+{
+  for (const_iterator it = begin(); it != end(); it++)
+    {
+      City *c = *it;
+      if (c->getCapitalOwner() == player)
+        return c;
+    }
+  return NULL;
+}
 // End of file
