@@ -38,7 +38,6 @@
 #include "defs.h"
 #include "Configuration.h"
 #include "GraphicsCache.h"
-#include "GraphicsLoader.h"
 #include "armysetlist.h"
 #include "Tile.h"
 #include "File.h"
@@ -46,19 +45,22 @@
 #include "shieldsetlist.h"
 
 #include "ucompose.hpp"
-#include "rgb_shift.h"
 
 #include "glade-helpers.h"
+#include "image-editor-dialog.h"
 
 
 ArmySetWindow::ArmySetWindow(std::string load_filename)
 {
+  needs_saving = false;
   d_armyset = NULL;
     Glib::RefPtr<Gtk::Builder> xml
 	= Gtk::Builder::create_from_file(get_glade_path() + "/armyset-window.ui");
 
     xml->get_widget("window", window);
     window->set_icon_from_file(File::getMiscFile("various/castle_icon.png"));
+    window->signal_delete_event().connect
+      (sigc::mem_fun(*this, &ArmySetWindow::on_window_closed));
 
     xml->get_widget("white_image", white_image);
     xml->get_widget("green_image", green_image);
@@ -112,23 +114,55 @@ ArmySetWindow::ArmySetWindow(std::string load_filename)
     neutral_image_filechooserbutton->signal_selection_changed().connect
       (sigc::mem_fun(this, &ArmySetWindow::on_neutral_image_changed));
     xml->get_widget("production_spinbutton", production_spinbutton);
+    production_spinbutton->set_range
+      (double(MIN_PRODUCTION_TURNS_FOR_ARMY_UNITS), 
+       double(MAX_PRODUCTION_TURNS_FOR_ARMY_UNITS));
     production_spinbutton->signal_changed().connect
       (sigc::mem_fun(this, &ArmySetWindow::on_production_changed));
+    production_spinbutton->signal_insert_text().connect
+      (sigc::mem_fun(this, &ArmySetWindow::on_production_text_changed));
     xml->get_widget("cost_spinbutton", cost_spinbutton);
+    cost_spinbutton->set_range(double(MIN_COST_FOR_ARMY_UNITS), 
+			       double(MAX_COST_FOR_ARMY_UNITS));
     cost_spinbutton->signal_changed().connect
       (sigc::mem_fun(this, &ArmySetWindow::on_cost_changed));
+    cost_spinbutton->signal_insert_text().connect
+      (sigc::mem_fun(this, &ArmySetWindow::on_cost_text_changed));
+    xml->get_widget("new_cost_spinbutton", new_cost_spinbutton);
+    new_cost_spinbutton->set_range(double(MIN_NEW_COST_FOR_ARMY_UNITS), 
+				   double(MAX_NEW_COST_FOR_ARMY_UNITS));
+    new_cost_spinbutton->signal_changed().connect
+      (sigc::mem_fun(this, &ArmySetWindow::on_new_cost_changed));
+    new_cost_spinbutton->signal_insert_text().connect
+      (sigc::mem_fun(this, &ArmySetWindow::on_new_cost_text_changed));
     xml->get_widget("upkeep_spinbutton", upkeep_spinbutton);
+    upkeep_spinbutton->set_range (double(MIN_UPKEEP_FOR_ARMY_UNITS), 
+				  double(MAX_UPKEEP_FOR_ARMY_UNITS));
     upkeep_spinbutton->signal_changed().connect
       (sigc::mem_fun(this, &ArmySetWindow::on_upkeep_changed));
+    upkeep_spinbutton->signal_insert_text().connect
+      (sigc::mem_fun(this, &ArmySetWindow::on_upkeep_text_changed));
     xml->get_widget("strength_spinbutton", strength_spinbutton);
+    strength_spinbutton->set_range (double(MIN_STRENGTH_FOR_ARMY_UNITS), 
+				    double(MAX_STRENGTH_FOR_ARMY_UNITS));
     strength_spinbutton->signal_changed().connect
       (sigc::mem_fun(this, &ArmySetWindow::on_strength_changed));
+    strength_spinbutton->signal_insert_text().connect
+      (sigc::mem_fun(this, &ArmySetWindow::on_strength_text_changed));
     xml->get_widget("moves_spinbutton", moves_spinbutton);
+    moves_spinbutton->set_range(double(MIN_MOVES_FOR_ARMY_UNITS), 
+				double(MAX_MOVES_FOR_ARMY_UNITS));
     moves_spinbutton->signal_changed().connect
       (sigc::mem_fun(this, &ArmySetWindow::on_moves_changed));
+    moves_spinbutton->signal_insert_text().connect
+      (sigc::mem_fun(this, &ArmySetWindow::on_moves_text_changed));
     xml->get_widget("exp_spinbutton", exp_spinbutton);
+    exp_spinbutton->set_range(double(MIN_EXP_FOR_ARMY_UNITS), 
+			      double(MAX_EXP_FOR_ARMY_UNITS));
     exp_spinbutton->signal_changed().connect
       (sigc::mem_fun(this, &ArmySetWindow::on_exp_changed));
+    exp_spinbutton->signal_insert_text().connect
+      (sigc::mem_fun(this, &ArmySetWindow::on_exp_text_changed));
     xml->get_widget("gender_none_radiobutton", gender_none_radiobutton);
     gender_none_radiobutton->signal_toggled().connect
       (sigc::mem_fun(this, &ArmySetWindow::on_gender_none_toggled));
@@ -145,8 +179,12 @@ ArmySetWindow::ArmySetWindow(std::string load_filename)
     defends_ruins_checkbutton->signal_toggled().connect
       (sigc::mem_fun(this, &ArmySetWindow::on_defends_ruins_toggled));
     xml->get_widget("sight_spinbutton", sight_spinbutton);
+    sight_spinbutton->set_range(double(MIN_SIGHT_FOR_ARMY_UNITS), 
+				double(MAX_SIGHT_FOR_ARMY_UNITS));
     sight_spinbutton->signal_changed().connect
       (sigc::mem_fun(this, &ArmySetWindow::on_sight_changed));
+    sight_spinbutton->signal_insert_text().connect
+      (sigc::mem_fun(this, &ArmySetWindow::on_sight_text_changed));
     xml->get_widget("move_forests_checkbutton", move_forests_checkbutton);
     move_forests_checkbutton->signal_toggled().connect
       (sigc::mem_fun(this, &ArmySetWindow::on_move_forests_toggled));
@@ -233,6 +271,9 @@ ArmySetWindow::ArmySetWindow(std::string load_filename)
 		    edit_standard_picture_menuitem);
     edit_standard_picture_menuitem->signal_activate().connect
       (sigc::mem_fun(this, &ArmySetWindow::on_edit_standard_picture_activated));
+    xml->get_widget("edit_bag_picture_menuitem", edit_bag_picture_menuitem);
+    edit_bag_picture_menuitem->signal_activate().connect
+      (sigc::mem_fun(this, &ArmySetWindow::on_edit_bag_picture_activated));
     xml->get_widget("edit_ship_picture_menuitem", edit_ship_picture_menuitem);
     edit_ship_picture_menuitem->signal_activate().connect
       (sigc::mem_fun(this, &ArmySetWindow::on_edit_ship_picture_activated));
@@ -279,8 +320,7 @@ ArmySetWindow::update_armyset_menuitems()
     }
   else
     {
-      std::string file = 
-	File::getArmysetDir(d_armyset) + d_armyset->getSubDir() + ARMYSET_EXT;
+      std::string file = d_armyset->getConfigurationFile();
       if (File::exists(file) == false)
 	save_armyset_menuitem->set_sensitive(true);
       else if (File::is_writable(file) == false)
@@ -318,7 +358,8 @@ ArmySetWindow::update_army_panel()
       name_entry->set_text("");
       description_textview->get_buffer()->set_text("");
       production_spinbutton->set_value(MIN_PRODUCTION_TURNS_FOR_ARMY_UNITS);
-      cost_spinbutton->set_value(0);
+      cost_spinbutton->set_value(MIN_COST_FOR_ARMY_UNITS);
+      new_cost_spinbutton->set_value(MIN_NEW_COST_FOR_ARMY_UNITS);
       upkeep_spinbutton->set_value(MIN_UPKEEP_FOR_ARMY_UNITS);
       strength_spinbutton->set_value(MIN_STRENGTH_FOR_ARMY_UNITS);
       moves_spinbutton->set_value(MIN_MOVES_FOR_ARMY_UNITS);
@@ -394,14 +435,6 @@ bool ArmySetWindow::on_delete_event(GdkEventAny *e)
   return true;
 }
 
-
-bool ArmySetWindow::load(std::string tag, XML_Helper *helper)
-{
-  if (tag == Armyset::d_tag)
-    d_armyset = new Armyset(helper);
-  return true;
-}
-
 void ArmySetWindow::on_new_armyset_activated()
 {
   std::string name = "";
@@ -419,9 +452,11 @@ void ArmySetWindow::on_new_armyset_activated()
     delete d_armyset;
   d_armyset = armyset;
   armies_list->clear();
-  current_save_filename = File::getArmyset(d_armyset);
+  std::string dir = File::getUserArmysetDir() + "/" + d_armyset->getSubDir();
+  d_armyset->setDirectory(dir);
+  File::create_dir(dir);
+  current_save_filename = d_armyset->getConfigurationFile();
 
-  File::create_dir(File::getUserArmysetDir() + "/" + d_armyset->getSubDir());
   std::string imgpath = Glib::get_home_dir();
   //we will copy an image from where the user points into the new armyset dir.
   white_image_filechooserbutton->set_current_folder(imgpath);
@@ -440,11 +475,12 @@ void ArmySetWindow::on_new_armyset_activated()
   XML_Helper helper(current_save_filename, std::ios::out, false);
   d_armyset->save(&helper);
   helper.close();
+  needs_saving = true;
 }
 
 void ArmySetWindow::update_filechooserbutton(Gtk::FileChooserButton *b, ArmyProto *a, Shield::Colour c)
 {
-  std::string file = File::getArmysetFile (d_armyset, a->getImageName(c));
+  std::string file = d_armyset->getFile(a->getImageName(c));
   if (File::exists(file) == false)
     b->set_sensitive(true);
   else if (File::is_writable(file) == false)
@@ -506,6 +542,9 @@ void ArmySetWindow::on_validate_armyset_activated()
   valid = d_armyset->validateStandard();
   if (!valid)
     msgs.push_back(_("The hero's standard (the flag) image must be set."));
+  valid = d_armyset->validateBag();
+  if (!valid)
+    msgs.push_back(_("The picture for the bag of items must be set."));
 
   for (Armyset::iterator it = d_armyset->begin(); it != d_armyset->end(); it++)
     {
@@ -539,7 +578,7 @@ void ArmySetWindow::on_validate_armyset_activated()
 void ArmySetWindow::on_save_armyset_activated()
 {
   if (current_save_filename.empty())
-    current_save_filename = File::getArmysetDir(d_armyset);
+    current_save_filename = d_armyset->getConfigurationFile();
   
   //Reorder the armyset according to the treeview
   d_armyset->clear();
@@ -550,62 +589,76 @@ void ArmySetWindow::on_save_armyset_activated()
   XML_Helper helper(current_save_filename, std::ios::out, false);
   d_armyset->save(&helper);
   helper.close();
+  needs_saving = false;
 }
 
-void ArmySetWindow::on_quit_activated()
-{
-  // FIXME: ask
-  bool end = true;
-
-  if (end) {
-  }
-  window->hide();
-}
 void ArmySetWindow::on_edit_ship_picture_activated()
 {
   std::string filename = "";
   if (d_armyset->getShipImageName() != "")
-    filename = File::getArmysetFile(d_armyset, d_armyset->getShipImageName());
+    filename = d_armyset->getFile(d_armyset->getShipImageName());
   MaskedImageEditorDialog d(filename);
   d.set_icon_from_file(File::getMiscFile("various/castle_icon.png"));
   d.run();
   if (d.get_selected_filename() != "")
     {
       std::string file = File::get_basename(d.get_selected_filename());
-      if (d.get_selected_filename() != File::getArmysetFile(d_armyset, file))
+      if (d.get_selected_filename() != d_armyset->getFile(file))
 	{
 	  //fixme:warn on overrwite.
 	  File::copy (d.get_selected_filename(), 
-		      File::getArmysetFile(d_armyset, file));
+		      d_armyset->getFile(file));
 	}
       d_armyset->setShipImageName(file);
+      needs_saving = true;
     }
 }
 void ArmySetWindow::on_edit_standard_picture_activated()
 {
   std::string filename = "";
   if (d_armyset->getStandardImageName() != "")
-    filename = File::getArmysetFile(d_armyset, d_armyset->getStandardImageName());
+    filename = d_armyset->getFile(d_armyset->getStandardImageName());
   MaskedImageEditorDialog d(filename);
   d.set_icon_from_file(File::getMiscFile("various/castle_icon.png"));
   d.run();
   if (d.get_selected_filename() != "")
     {
       std::string file = File::get_basename(d.get_selected_filename());
-      if (d.get_selected_filename() != File::getArmysetFile(d_armyset, file))
+      if (d.get_selected_filename() != d_armyset->getFile(file))
 	{
 	  //fixme:warn on overrwite.
 	  File::copy (d.get_selected_filename(), 
-		      File::getArmysetFile(d_armyset, file));
+		      d_armyset->getFile(file));
 	}
       d_armyset->setStandardImageName(file);
+      needs_saving = true;
+    }
+}
+void ArmySetWindow::on_edit_bag_picture_activated()
+{
+  std::string filename = "";
+  if (d_armyset->getBagImageName().empty() == false)
+    filename = d_armyset->getFile(d_armyset->getBagImageName());
+  ImageEditorDialog d(filename, 1);
+  d.set_icon_from_file(File::getMiscFile("various/tileset_icon.png"));
+  d.set_parent_window(*window);
+  int response = d.run();
+  if (response == Gtk::RESPONSE_ACCEPT)
+    {
+      std::string filename = d.get_selected_filename();
+      std::string name = File::get_basename(filename);
+      File::copy(filename, d_armyset->getFile(name));
+      d_armyset->setBagImageName(name);
+      needs_saving = true;
     }
 }
 void ArmySetWindow::on_edit_armyset_info_activated()
 {
   ArmySetInfoDialog d(d_armyset, true);
   d.set_parent_window(*window);
-  d.run();
+  int response = d.run();
+  if (response == Gtk::RESPONSE_ACCEPT)
+    needs_saving = true;
 }
 
 void ArmySetWindow::on_help_about_activated()
@@ -620,7 +673,7 @@ void ArmySetWindow::on_help_about_activated()
   dialog->set_icon_from_file(File::getMiscFile("various/castle_icon.png"));
 
   dialog->set_version(PACKAGE_VERSION);
-  dialog->set_logo(GraphicsLoader::getMiscPicture("castle_icon.png")->to_pixbuf());
+  dialog->set_logo(GraphicsCache::getMiscPicture("castle_icon.png")->to_pixbuf());
   dialog->show_all();
   dialog->run();
   delete dialog;
@@ -650,12 +703,12 @@ void ArmySetWindow::fill_army_image(Gtk::FileChooserButton *button, Gtk::Image *
     {
       image->property_pixbuf() = army->getImage(c)->to_pixbuf();
     
-      std::string path = File::getArmysetFile(d_armyset, army->getImageName(c));
+      std::string path = d_armyset->getFile(army->getImageName(c));
       button->set_filename(path);
     }
   else
     {
-      std::string imgpath = File::getArmysetDir(d_armyset);
+      std::string imgpath = d_armyset->getDirectory();
       button->set_filename("");
       button->set_current_folder(imgpath);
       image->clear();
@@ -686,6 +739,7 @@ void ArmySetWindow::fill_army_info(ArmyProto *army)
   double turns = army->getProduction();
   production_spinbutton->set_value(turns);
   cost_spinbutton->set_value(army->getProductionCost());
+  new_cost_spinbutton->set_value(army->getNewProductionCost());
   upkeep_spinbutton->set_value(army->getUpkeep());
   strength_spinbutton->set_value(army->getStrength());
   moves_spinbutton->set_value(army->getMaxMoves());
@@ -763,6 +817,7 @@ void ArmySetWindow::on_name_changed()
       ArmyProto *a = row[armies_columns.army];
       a->setName(name_entry->get_text());
       row[armies_columns.name] = name_entry->get_text();
+      needs_saving = true;
     }
 }
 
@@ -776,6 +831,7 @@ void ArmySetWindow::on_description_changed()
       Gtk::TreeModel::Row row = *iterrow;
       ArmyProto *a = row[armies_columns.army];
       a->setDescription(description_textview->get_buffer()->get_text());
+      needs_saving = true;
     }
 }
 
@@ -795,29 +851,27 @@ void ArmySetWindow::on_image_changed(Gtk::FileChooserButton *button, Gtk::Image
       Gtk::TreeModel::Row row = *iterrow;
       ArmyProto *a = row[armies_columns.army];
       std::string file = File::get_basename(button->get_filename());
-      if (button->get_filename() != File::getArmysetFile(d_armyset, file))
+      if (button->get_filename() != d_armyset->getFile(file))
 	{
 	  //fixme:warn on overrwite.
-	  File::copy (button->get_filename(), 
-		      File::getArmysetFile(d_armyset, file));
+	  File::copy (button->get_filename(), d_armyset->getFile(file));
 	}
       a->setImageName(c, file);
-      struct rgb_shift shifts;
-      shifts = Shieldsetlist::getInstance()->getMaskColorShifts(1, c);
-      //GraphicsLoader::instantiateImages(d_armyset);
-      //fixme, this needs to be added back in somehow.
-      GraphicsLoader::instantiateImages(d_armyset, a, c);
+      Gdk::Color colour = Shieldsetlist::getInstance()->getColor(1, c);
+      a->instantiateImages(d_armyset->getTileSize(), c, 
+			   d_armyset->getFile(a->getImageName(c)));
       if (c != Shield::NEUTRAL)
 	{
 	  PixMask *army_image = GraphicsCache::applyMask(a->getImage(c), 
 							 a->getMask(c), 
-							 shifts, false);
+							 colour, false);
 	  image->property_pixbuf() = army_image->to_pixbuf();
 	  delete army_image;
 	}
       else
 	image->property_pixbuf() = a->getImage(c)->to_pixbuf();
 
+      needs_saving = true;
     }
 }
 void ArmySetWindow::on_white_image_changed()
@@ -863,6 +917,11 @@ void ArmySetWindow::on_neutral_image_changed()
 		   Shield::NEUTRAL);
 }
 
+void ArmySetWindow::on_production_text_changed(const Glib::ustring &s, int* p)
+{
+  production_spinbutton->set_value(atoi(production_spinbutton->get_text().c_str()));
+  on_production_changed();
+}
 void ArmySetWindow::on_production_changed()
 {
   Glib::RefPtr<Gtk::TreeSelection> selection = armies_treeview->get_selection();
@@ -880,9 +939,15 @@ void ArmySetWindow::on_production_changed()
 	production_spinbutton->set_value(MAX_PRODUCTION_TURNS_FOR_ARMY_UNITS);
       else
 	a->setProduction(int(production_spinbutton->get_value()));
+      needs_saving = true;
     }
 }
 
+void ArmySetWindow::on_cost_text_changed(const Glib::ustring &s, int* p)
+{
+  cost_spinbutton->set_value(atoi(cost_spinbutton->get_text().c_str()));
+  on_cost_changed();
+}
 void ArmySetWindow::on_cost_changed()
 {
   Glib::RefPtr<Gtk::TreeSelection> selection = armies_treeview->get_selection();
@@ -892,10 +957,41 @@ void ArmySetWindow::on_cost_changed()
     {
       Gtk::TreeModel::Row row = *iterrow;
       ArmyProto *a = row[armies_columns.army];
-      a->setProductionCost(int(cost_spinbutton->get_value()));
+      if (cost_spinbutton->get_value() < MIN_COST_FOR_ARMY_UNITS)
+	cost_spinbutton->set_value(MIN_COST_FOR_ARMY_UNITS);
+      else if (strength_spinbutton->get_value() > MAX_COST_FOR_ARMY_UNITS)
+	cost_spinbutton->set_value(MAX_COST_FOR_ARMY_UNITS);
+      else
+	a->setProductionCost(int(cost_spinbutton->get_value()));
+      needs_saving = true;
     }
 }
 
+
+void ArmySetWindow::on_new_cost_text_changed(const Glib::ustring &s, int* p)
+{
+  new_cost_spinbutton->set_value(atoi(new_cost_spinbutton->get_text().c_str()));
+  on_new_cost_changed();
+}
+void ArmySetWindow::on_new_cost_changed()
+{
+  Glib::RefPtr<Gtk::TreeSelection> selection = armies_treeview->get_selection();
+  Gtk::TreeModel::iterator iterrow = selection->get_selected();
+
+  if (iterrow) 
+    {
+      Gtk::TreeModel::Row row = *iterrow;
+      ArmyProto *a = row[armies_columns.army];
+      a->setNewProductionCost(int(new_cost_spinbutton->get_value()));
+      needs_saving = true;
+    }
+}
+
+void ArmySetWindow::on_upkeep_text_changed(const Glib::ustring &s, int* p)
+{
+  upkeep_spinbutton->set_value(atoi(upkeep_spinbutton->get_text().c_str()));
+  on_upkeep_changed();
+}
 void ArmySetWindow::on_upkeep_changed()
 {
   Glib::RefPtr<Gtk::TreeSelection> selection = armies_treeview->get_selection();
@@ -911,9 +1007,15 @@ void ArmySetWindow::on_upkeep_changed()
 	upkeep_spinbutton->set_value(MAX_UPKEEP_FOR_ARMY_UNITS);
       else
 	a->setUpkeep(int(upkeep_spinbutton->get_value()));
+      needs_saving = true;
     }
 }
 
+void ArmySetWindow::on_strength_text_changed(const Glib::ustring &s, int* p)
+{
+  strength_spinbutton->set_value(atoi(strength_spinbutton->get_text().c_str()));
+  on_strength_changed();
+}
 void ArmySetWindow::on_strength_changed()
 {
   Glib::RefPtr<Gtk::TreeSelection> selection = armies_treeview->get_selection();
@@ -929,9 +1031,15 @@ void ArmySetWindow::on_strength_changed()
 	strength_spinbutton->set_value(MAX_STRENGTH_FOR_ARMY_UNITS);
       else
 	a->setStrength(int(strength_spinbutton->get_value()));
+      needs_saving = true;
     }
 }
 
+void ArmySetWindow::on_moves_text_changed(const Glib::ustring &s, int* p)
+{
+  moves_spinbutton->set_value(atoi(moves_spinbutton->get_text().c_str()));
+  on_moves_changed();
+}
 void ArmySetWindow::on_moves_changed()
 {
   Glib::RefPtr<Gtk::TreeSelection> selection = armies_treeview->get_selection();
@@ -947,9 +1055,15 @@ void ArmySetWindow::on_moves_changed()
 	moves_spinbutton->set_value(MAX_MOVES_FOR_ARMY_UNITS);
       else
 	a->setMaxMoves(int(moves_spinbutton->get_value()));
+      needs_saving = true;
     }
 }
 
+void ArmySetWindow::on_exp_text_changed(const Glib::ustring &s, int* p)
+{
+  exp_spinbutton->set_value(atoi(exp_spinbutton->get_text().c_str()));
+  on_exp_changed();
+}
 void ArmySetWindow::on_exp_changed()
 {
   Glib::RefPtr<Gtk::TreeSelection> selection = armies_treeview->get_selection();
@@ -960,9 +1074,15 @@ void ArmySetWindow::on_exp_changed()
       Gtk::TreeModel::Row row = *iterrow;
       ArmyProto *a = row[armies_columns.army];
       a->setXpReward(int(exp_spinbutton->get_value()));
+      needs_saving = true;
     }
 }
 
+void ArmySetWindow::on_sight_text_changed(const Glib::ustring &s, int* p)
+{
+  sight_spinbutton->set_value(atoi(sight_spinbutton->get_text().c_str()));
+  on_sight_changed();
+}
 void ArmySetWindow::on_sight_changed()
 {
   Glib::RefPtr<Gtk::TreeSelection> selection = armies_treeview->get_selection();
@@ -973,6 +1093,7 @@ void ArmySetWindow::on_sight_changed()
       Gtk::TreeModel::Row row = *iterrow;
       ArmyProto *a = row[armies_columns.army];
       a->setSight(int(sight_spinbutton->get_value()));
+      needs_saving = true;
     }
 }
 
@@ -986,6 +1107,7 @@ void ArmySetWindow::on_gender_none_toggled()
       Gtk::TreeModel::Row row = *iterrow;
       ArmyProto *a = row[armies_columns.army];
       a->setGender(Hero::NONE);
+      needs_saving = true;
     }
 }
 
@@ -999,6 +1121,7 @@ void ArmySetWindow::on_gender_male_toggled()
       Gtk::TreeModel::Row row = *iterrow;
       ArmyProto *a = row[armies_columns.army];
       a->setGender(Hero::MALE);
+      needs_saving = true;
     }
 }
 
@@ -1012,6 +1135,7 @@ void ArmySetWindow::on_gender_female_toggled()
       Gtk::TreeModel::Row row = *iterrow;
       ArmyProto *a = row[armies_columns.army];
       a->setGender(Hero::FEMALE);
+      needs_saving = true;
     }
 }
 
@@ -1025,6 +1149,7 @@ void ArmySetWindow::on_awardable_toggled()
       Gtk::TreeModel::Row row = *iterrow;
       ArmyProto *a = row[armies_columns.army];
       a->setAwardable(awardable_checkbutton->get_active());
+      needs_saving = true;
     }
 }
 
@@ -1038,6 +1163,7 @@ void ArmySetWindow::on_defends_ruins_toggled()
       Gtk::TreeModel::Row row = *iterrow;
       ArmyProto *a = row[armies_columns.army];
       a->setDefendsRuins(defends_ruins_checkbutton->get_active());
+      needs_saving = true;
     }
 }
 
@@ -1063,6 +1189,7 @@ void ArmySetWindow::on_move_forests_toggled()
 	    bonus ^= Tile::FOREST;
 	}
       a->setMoveBonus(bonus);
+      needs_saving = true;
     }
 }
 
@@ -1088,6 +1215,7 @@ void ArmySetWindow::on_move_marshes_toggled()
 	    bonus ^= Tile::SWAMP;
 	}
       a->setMoveBonus(bonus);
+      needs_saving = true;
     }
 }
 
@@ -1113,6 +1241,7 @@ void ArmySetWindow::on_move_hills_toggled()
 	    bonus ^= Tile::HILLS;
 	}
       a->setMoveBonus(bonus);
+      needs_saving = true;
     }
 }
 
@@ -1138,6 +1267,7 @@ void ArmySetWindow::on_move_mountains_toggled()
 	    bonus ^= Tile::MOUNTAIN;
 	}
       a->setMoveBonus(bonus);
+      needs_saving = true;
     }
 }
 
@@ -1161,6 +1291,7 @@ void ArmySetWindow::on_can_fly_toggled()
 	  bonus = 0;
 	}
       a->setMoveBonus(bonus);
+      needs_saving = true;
     }
 }
 
@@ -1182,6 +1313,7 @@ void ArmySetWindow::on_add1strinopen_toggled()
 	    bonus ^= Army::ADD1STRINOPEN;
 	}
       a->setArmyBonus(bonus);
+      needs_saving = true;
     }
 }
 
@@ -1203,6 +1335,7 @@ void ArmySetWindow::on_add2strinopen_toggled()
 	    bonus ^= Army::ADD2STRINOPEN;
 	}
       a->setArmyBonus(bonus);
+      needs_saving = true;
     }
 }
 
@@ -1224,6 +1357,7 @@ void ArmySetWindow::on_add1strinforest_toggled()
 	    bonus ^= Army::ADD1STRINFOREST;
 	}
       a->setArmyBonus(bonus);
+      needs_saving = true;
     }
 }
 
@@ -1245,6 +1379,7 @@ void ArmySetWindow::on_add1strinhills_toggled()
 	    bonus ^= Army::ADD1STRINHILLS;
 	}
       a->setArmyBonus(bonus);
+      needs_saving = true;
     }
 }
 
@@ -1266,6 +1401,7 @@ void ArmySetWindow::on_add1strincity_toggled()
 	    bonus ^= Army::ADD1STRINCITY;
 	}
       a->setArmyBonus(bonus);
+      needs_saving = true;
     }
 }
 
@@ -1287,6 +1423,7 @@ void ArmySetWindow::on_add2strincity_toggled()
 	    bonus ^= Army::ADD2STRINCITY;
 	}
       a->setArmyBonus(bonus);
+      needs_saving = true;
     }
 }
 
@@ -1308,6 +1445,7 @@ void ArmySetWindow::on_add1stackinhills_toggled()
 	    bonus ^= Army::ADD1STACKINHILLS;
 	}
       a->setArmyBonus(bonus);
+      needs_saving = true;
     }
 }
 
@@ -1329,6 +1467,7 @@ void ArmySetWindow::on_suballcitybonus_toggled()
 	    bonus ^= Army::SUBALLCITYBONUS;
 	}
       a->setArmyBonus(bonus);
+      needs_saving = true;
     }
 }
 
@@ -1350,6 +1489,7 @@ void ArmySetWindow::on_sub1enemystack_toggled()
 	    bonus ^= Army::SUB1ENEMYSTACK;
 	}
       a->setArmyBonus(bonus);
+      needs_saving = true;
     }
 }
 
@@ -1371,6 +1511,7 @@ void ArmySetWindow::on_add1stack_toggled()
 	    bonus ^= Army::ADD1STACK;
 	}
       a->setArmyBonus(bonus);
+      needs_saving = true;
     }
 }
 
@@ -1392,6 +1533,7 @@ void ArmySetWindow::on_add2stack_toggled()
 	    bonus ^= Army::ADD2STACK;
 	}
       a->setArmyBonus(bonus);
+      needs_saving = true;
     }
 }
 
@@ -1413,6 +1555,7 @@ void ArmySetWindow::on_suballnonherobonus_toggled()
 	    bonus ^= Army::SUBALLNONHEROBONUS;
 	}
       a->setArmyBonus(bonus);
+      needs_saving = true;
     }
 }
 
@@ -1434,6 +1577,7 @@ void ArmySetWindow::on_suballherobonus_toggled()
 	    bonus ^= Army::SUBALLHEROBONUS;
 	}
       a->setArmyBonus(bonus);
+      needs_saving = true;
     }
 }
 
@@ -1447,6 +1591,7 @@ void ArmySetWindow::on_add_army_clicked()
   (*i)[armies_columns.name] = a->getName();
   (*i)[armies_columns.army] = a;
   d_armyset->push_back(a);
+  needs_saving = true;
 
 }
 
@@ -1463,32 +1608,17 @@ void ArmySetWindow::on_remove_army_clicked()
       ArmyProto *a = row[armies_columns.army];
       armies_list->erase(iterrow);
       d_armyset->remove(a);
+      needs_saving = true;
     }
 }
+
 void ArmySetWindow::load_armyset(std::string filename)
 {
   current_save_filename = filename;
 
-  bool priv = true;
-  std::string dir = Glib::path_get_dirname(filename);
-  dir = Glib::path_get_dirname(dir) + "/";
-  if (dir == File::getArmysetDir())
-    priv = false;
-  else if (dir == File::getUserArmysetDir())
-    priv = true;
-  else
-    {
-      std::string msg;
-      msg = "Armysets can only be loaded from:\n" + File::getArmysetDir() +
-	"\n or \n" + File::getUserArmysetDir();
-      Gtk::MessageDialog dialog(*window, msg);
-      dialog.run();
-      dialog.hide();
-      return;
-    }
   std::string name = File::get_basename(filename);
 
-  Armyset *armyset = Armyset::create(name, priv);
+  Armyset *armyset = Armyset::create(filename);
   if (armyset == NULL)
     {
       std::string msg;
@@ -1503,10 +1633,8 @@ void ArmySetWindow::load_armyset(std::string filename)
     delete d_armyset;
   d_armyset = armyset;
 
-  //was this from the user's own private collection?
-
   d_armyset->setSubDir(name);
-  GraphicsLoader::instantiateImages(d_armyset);
+  d_armyset->instantiateImages();
   guint32 max = d_armyset->getSize();
   for (unsigned int i = 0; i < max; i++)
     addArmyType(i);
@@ -1542,7 +1670,7 @@ void ArmySetWindow::load_armyset(std::string filename)
       update_filechooserbutton(neutral_image_filechooserbutton, a, 
 			       Shield::NEUTRAL);
     }
-  std::string imgpath = File::getArmysetDir(d_armyset);
+  std::string imgpath = d_armyset->getDirectory();
   white_image_filechooserbutton->set_current_folder(imgpath);
   green_image_filechooserbutton->set_current_folder(imgpath);
   yellow_image_filechooserbutton->set_current_folder(imgpath);
@@ -1553,3 +1681,37 @@ void ArmySetWindow::load_armyset(std::string filename)
   black_image_filechooserbutton->set_current_folder(imgpath);
   neutral_image_filechooserbutton->set_current_folder(imgpath);
 }
+bool ArmySetWindow::quit()
+{
+  if (needs_saving == true)
+    {
+      Gtk::Dialog* dialog;
+      Glib::RefPtr<Gtk::Builder> xml
+	= Gtk::Builder::create_from_file(get_glade_path() + 
+					 "/editor-quit-dialog.ui");
+      xml->get_widget("dialog", dialog);
+      dialog->set_transient_for(*window);
+      int response = dialog->run();
+      dialog->hide();
+      delete dialog;
+      
+      if (response == Gtk::RESPONSE_CANCEL) //we don't want to quit
+	return false;
+
+      else if (response == Gtk::RESPONSE_ACCEPT) // save and quit
+	on_save_armyset_activated();
+      //else if (Response == Gtk::CLOSE) // don't save just quit
+      window->hide();
+    }
+  else
+    window->hide();
+  return true;
+}
+bool ArmySetWindow::on_window_closed(GdkEventAny*)
+{
+  return !quit();
+}
+void ArmySetWindow::on_quit_activated()
+{
+  quit();
+}
