@@ -15,8 +15,8 @@
 //  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 
 //  02110-1301, USA.
 
-#ifndef GUI_ARMYLIST_WINDOW_H
-#define GUI_ARMYLIST_WINDOW_H
+#ifndef GUI_ARMYSET_WINDOW_H
+#define GUI_ARMYSET_WINDOW_H
 
 #include <memory>
 #include <vector>
@@ -45,6 +45,7 @@ class ArmySetWindow: public sigc::trackable
     std::string current_save_filename;
     Armyset *d_armyset; //current armyset
     ArmyProto *d_army; //current army
+    bool needs_saving;
     Gtk::Image *white_image;
     Gtk::Image *green_image;
     Gtk::Image *yellow_image;
@@ -68,6 +69,7 @@ class ArmySetWindow: public sigc::trackable
     Gtk::FileChooserButton *neutral_image_filechooserbutton;
     Gtk::SpinButton *production_spinbutton;
     Gtk::SpinButton *cost_spinbutton;
+    Gtk::SpinButton *new_cost_spinbutton;
     Gtk::SpinButton *upkeep_spinbutton;
     Gtk::SpinButton *strength_spinbutton;
     Gtk::SpinButton *moves_spinbutton;
@@ -106,6 +108,7 @@ class ArmySetWindow: public sigc::trackable
     Gtk::MenuItem *edit_armyset_info_menuitem;
     Gtk::MenuItem *edit_ship_picture_menuitem;
     Gtk::MenuItem *edit_standard_picture_menuitem;
+    Gtk::MenuItem *edit_bag_picture_menuitem;
     Gtk::MenuItem *quit_menuitem;
     Gtk::MenuItem *help_about_menuitem;
 
@@ -132,16 +135,17 @@ class ArmySetWindow: public sigc::trackable
     void on_save_armyset_activated();
     void on_validate_armyset_activated();
     void on_quit_activated();
+    bool on_window_closed(GdkEventAny*);
+    bool quit();
     void on_edit_armyset_info_activated();
     void on_edit_standard_picture_activated();
+    void on_edit_bag_picture_activated();
     void on_edit_ship_picture_activated();
     void on_help_about_activated();
     void on_army_selected();
     void fill_army_image(Gtk::FileChooserButton *button, Gtk::Image *image, Shield::Colour c, ArmyProto *army);
     void fill_army_info(ArmyProto *army);
 
-    bool load(std::string tag, XML_Helper *helper);
-
     //callbacks
     void on_name_changed();
     void on_description_changed();
@@ -156,12 +160,21 @@ class ArmySetWindow: public sigc::trackable
     void on_black_image_changed();
     void on_neutral_image_changed();
     void on_production_changed();
+    void on_production_text_changed(const Glib::ustring &s, int* p);
     void on_cost_changed();
+    void on_cost_text_changed(const Glib::ustring &s, int* p);
+    void on_new_cost_changed();
+    void on_new_cost_text_changed(const Glib::ustring &s, int* p);
     void on_upkeep_changed();
+    void on_upkeep_text_changed(const Glib::ustring &s, int* p);
     void on_strength_changed();
+    void on_strength_text_changed(const Glib::ustring &s, int* p);
     void on_moves_changed();
+    void on_moves_text_changed(const Glib::ustring &s, int* p);
     void on_exp_changed();
+    void on_exp_text_changed(const Glib::ustring &s, int* p);
     void on_sight_changed();
+    void on_sight_text_changed(const Glib::ustring &s, int* p);
     void on_gender_none_toggled();
     void on_gender_male_toggled();
     void on_gender_female_toggled();
