@@ -261,10 +261,10 @@ void ItemlistDialog::set_parent_window(Gtk::Window &parent)
     //dialog->set_position(Gtk::WIN_POS_CENTER_ON_PARENT);
 }
 
-void ItemlistDialog::run()
+int ItemlistDialog::run()
 {
     dialog->show_all();
-    dialog->run();
+    return dialog->run();
 }
 
 void ItemlistDialog::on_checkbutton_toggled(Gtk::CheckButton *checkbutton, 
