@@ -40,7 +40,7 @@ class CreateScenarioRandomize;
 class MainWindow: public sigc::trackable
 {
  public:
-    MainWindow();
+    MainWindow(std::string load_filename = "");
     ~MainWindow();
 
     void show();
@@ -50,6 +50,7 @@ class MainWindow: public sigc::trackable
     void show_initial_map();
     Gtk::Window &get_window() { return *window; }
 
+
  private:
     Gtk::Window* window;
     EditorBigMap* bigmap;
@@ -64,6 +65,7 @@ class MainWindow: public sigc::trackable
     Gtk::MenuItem *load_map_menuitem;
     Gtk::MenuItem *save_map_menuitem;
     Gtk::MenuItem *save_map_as_menuitem;
+    Gtk::MenuItem *import_map_from_sav_menuitem;
     Gtk::MenuItem *export_as_bitmap_menuitem;
     Gtk::MenuItem *export_as_bitmap_no_game_objects_menuitem;
     Gtk::MenuItem *validate_menuitem;
@@ -74,6 +76,7 @@ class MainWindow: public sigc::trackable
     Gtk::MenuItem *toggle_grid_menuitem;
     Gtk::MenuItem *smooth_map_menuitem;
     Gtk::MenuItem *smooth_screen_menuitem;
+    Gtk::MenuItem *switch_sets_menuitem;
     Gtk::MenuItem *edit_items_menuitem ;
     Gtk::MenuItem *edit_rewards_menuitem;
     Gtk::MenuItem *random_all_cities_menuitem;
@@ -88,10 +91,12 @@ class MainWindow: public sigc::trackable
     Gtk::HBox *terrain_tile_style_hbox;
     Gtk::DrawingArea *map_drawingarea;
     std::string current_save_filename;
+    bool needs_saving;
     Gtk::Table *terrain_type_table;
     Gtk::Label *mouse_position_label;
     Gtk::RadioButton *pointer_radiobutton;
     Gtk::Tooltips tooltips;
+    Gtk::Box *players_hbox;
 
     
     GdkEventButton *button_event;
@@ -109,15 +114,18 @@ class MainWindow: public sigc::trackable
     void on_new_map_activated();
     void on_load_map_activated();
     void on_save_map_activated();
+    void on_import_map_activated();
     void on_export_as_bitmap_activated();
     void on_export_as_bitmap_no_game_objects_activated();
     void on_validate_activated();
     void on_save_map_as_activated();
     void on_quit_activated();
+    bool quit();
     void on_edit_map_info_activated();
     void on_edit_players_activated();
     void on_smooth_map_activated();
     void on_smooth_screen_activated();
+    void on_switch_sets_activated();
     void on_edit_items_activated();
     void on_edit_rewards_activated();
 
@@ -164,6 +172,8 @@ class MainWindow: public sigc::trackable
     void on_terrain_radiobutton_toggled();
     void on_tile_style_radiobutton_toggled();
     
+
+    void clear_save_file_of_scenario_specific_data();
     Tile::Type get_terrain();
     int get_tile_style_id();
     void setup_pointer_radiobutton(Glib::RefPtr<Gtk::Builder> xml,
@@ -203,9 +213,20 @@ class MainWindow: public sigc::trackable
     bool on_bigmap_exposed(GdkEventExpose *event);
     bool on_smallmap_exposed(GdkEventExpose *event);
     void on_bigmap_surface_changed(Gtk::Allocation box);
+    void redraw();
+    void fill_players();
+
+    struct PlayerItem
+    {
+	Gtk::ToggleButton *button;
+	int player_id;
+    };
+    std::list<PlayerItem> player_buttons;
+    void on_player_toggled(PlayerItem item);
 
     int d_width;
     int d_height;
+    std::string d_load_filename;// filename given on the command line.
     
 };
 
