@@ -186,9 +186,9 @@ void RewardlistDialog::set_parent_window(Gtk::Window &parent)
     //dialog->set_position(Gtk::WIN_POS_CENTER_ON_PARENT);
 }
 
-void RewardlistDialog::run()
+int RewardlistDialog::run()
 {
     dialog->show_all();
-    dialog->run();
+    return dialog->run();
 }
 
