@@ -1,8 +1,8 @@
-//  Copyright (C) 2008, Ben Asselstine
+//  Copyright (C) 2008, 2009 Ben Asselstine
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
-//  the Free Software Foundation; either version 2 of the License, or
+//  the Free Software Foundation; either version 3 of the License, or
 //  (at your option) any later version.
 //
 //  This program is distributed in the hope that it will be useful,
@@ -21,15 +21,7 @@
 #include <memory>
 #include <vector>
 #include <sigc++/trackable.h>
-#include <gtkmm/dialog.h>
-#include <gtkmm/container.h>
-#include <gtkmm/liststore.h>
-#include <gtkmm/treemodelcolumn.h>
-#include <gtkmm/treeview.h>
-#include <gtkmm/textview.h>
-#include <gtkmm/label.h>
-#include <gtkmm/button.h>
-
+#include <gtkmm.h>
 
 class ItemProto;
 
@@ -38,6 +30,7 @@ class SelectItemDialog: public sigc::trackable
 {
  public:
     SelectItemDialog();
+    ~SelectItemDialog();
 
     void set_parent_window(Gtk::Window &parent);
 
@@ -46,7 +39,7 @@ class SelectItemDialog: public sigc::trackable
     const ItemProto *get_selected_item() { return selected_item; }
     
  private:
-    std::auto_ptr<Gtk::Dialog> dialog;
+    Gtk::Dialog* dialog;
     Gtk::Button *select_button;
 
     const ItemProto *selected_item;
@@ -55,9 +48,10 @@ class SelectItemDialog: public sigc::trackable
     class ItemsColumns: public Gtk::TreeModelColumnRecord {
     public:
 	ItemsColumns() 
-        { add(name); add(item);}
+        { add(name); add(attributes);add(item);}
 	
 	Gtk::TreeModelColumn<Glib::ustring> name;
+	Gtk::TreeModelColumn<Glib::ustring> attributes;
 	Gtk::TreeModelColumn<ItemProto *> item;
     };
     const ItemsColumns items_columns;
