@@ -35,6 +35,7 @@
 #include "playerlist.h"
 #include "stacklist.h"
 #include "hero-editor-dialog.h"
+#include "GameMap.h"
 
 #include "select-army-dialog.h"
 
@@ -124,12 +125,15 @@ StackEditorDialog::StackEditorDialog(Stack *s, int m)
 
     xml->get_widget("add_button", add_button);
     xml->get_widget("remove_button", remove_button);
+    xml->get_widget("copy_button", copy_button);
     xml->get_widget("edit_hero_button", edit_hero_button);
 
     add_button->signal_clicked().connect(
 	sigc::mem_fun(this, &StackEditorDialog::on_add_clicked));
     remove_button->signal_clicked().connect(
 	sigc::mem_fun(this, &StackEditorDialog::on_remove_clicked));
+    copy_button->signal_clicked().connect(
+	sigc::mem_fun(this, &StackEditorDialog::on_copy_clicked));
     edit_hero_button->signal_clicked().connect(
 	sigc::mem_fun(this, &StackEditorDialog::on_edit_hero_clicked));
 
@@ -151,7 +155,7 @@ void StackEditorDialog::set_parent_window(Gtk::Window &parent)
     //dialog->set_position(Gtk::WIN_POS_CENTER_ON_PARENT);
 }
 
-void StackEditorDialog::run()
+int StackEditorDialog::run()
 {
     dialog->show_all();
     int response = dialog->run();
@@ -189,7 +193,6 @@ void StackEditorDialog::run()
 	    a->setStat(Army::MOVES, (*j)[army_columns.moves]);
 	    a->setUpkeep((*j)[army_columns.upkeep]);
 	}
-	stack->group();
 	bool ship = stack->hasShip();
 	// add added armies to stack
 	for (Gtk::TreeIter j = army_list->children().begin(),
@@ -208,18 +211,17 @@ void StackEditorDialog::run()
 	{
 	    Player *player = get_selected_player();
 
-	    Player *stack_player = stack->getOwner();
+	    Player *old_active = Playerlist::getActiveplayer();
+	    Playerlist::getInstance()->setActiveplayer(player);
 	    Stack *new_stack = new Stack(*stack);
-	    if (stack_player)
-		stack_player->getStacklist()->flRemove(stack);
+	    GameMap::getInstance()->removeStack(stack);
 	    stack = new_stack;
-	
-	    player->addStack(stack);
 	    stack->setPlayer(player);
-	    for (Stack::iterator it = stack->begin(); it != stack->end(); it++)
-	      (*it)->setOwner(player);
+	    GameMap::getInstance()->putStack(stack);
+	    Playerlist::getInstance()->setActiveplayer(old_active);
 	}
     }
+    return response;
 }
 
 Player *StackEditorDialog::get_selected_player()
@@ -236,6 +238,18 @@ Player *StackEditorDialog::get_selected_player()
   return player;
 }
 
+void StackEditorDialog::on_copy_clicked()
+{
+    Gtk::TreeIter i = army_treeview->get_selection()->get_selected();
+    if (i)
+      {
+	Player *player = get_selected_player();
+	Army *army = (*i)[army_columns.army];
+	add_army(new Army(*army, player));
+      }
+
+    set_button_sensitivity();
+}
 void StackEditorDialog::on_add_clicked()
 {
     SelectArmyDialog d(stack->getOwner(), true);
@@ -314,12 +328,16 @@ void StackEditorDialog::set_button_sensitivity()
     Gtk::TreeIter i = army_treeview->get_selection()->get_selected();
     int armies = army_list->children().size();
     add_button->set_sensitive(armies < max_stack_size);
+    copy_button->set_sensitive(armies < max_stack_size);
     remove_button->set_sensitive(armies > min_size && i);
     if (i)
       {
 	Army *army = (*i)[army_columns.army];
 	if (army->isHero())
-	  edit_hero_button->set_sensitive(true);
+	  {
+	    edit_hero_button->set_sensitive(true);
+	    copy_button->set_sensitive(false);
+	  }
 	else
 	  edit_hero_button->set_sensitive(false);
       }
