@@ -2,7 +2,7 @@
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
-//  the Free Software Foundation; either version 2 of the License, or
+//  the Free Software Foundation; either version 3 of the License, or
 //  (at your option) any later version.
 //
 //  This program is distributed in the hope that it will be useful,
@@ -64,7 +64,7 @@ GameClient::~GameClient()
     network_connection->send(MESSAGE_TYPE_PARTICIPANT_DISCONNECT, d_nickname);
 }
 
-void GameClient::start(std::string host, Uint32 port, std::string nick)
+void GameClient::start(std::string host, guint32 port, std::string nick)
 {
   d_host = host;
   d_port = port;
@@ -386,7 +386,7 @@ void GameClient::request_seat_manifest()
     
 void GameClient::gotTurnOrder (std::string payload)
 {
-  std::list<Uint32> player_ids;
+  std::list<guint32> player_ids;
   std::stringstream players;
   players.str(payload);
 
