@@ -2,7 +2,7 @@
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
-//  the Free Software Foundation; either version 2 of the License, or
+//  the Free Software Foundation; either version 3 of the License, or
 //  (at your option) any later version.
 //
 //  This program is distributed in the hope that it will be useful,
@@ -45,7 +45,7 @@ public:
   //! Deletes the singleton instance.
   static void deleteInstance();
 
-  void start(std::string host, Uint32 port, std::string nick);
+  void start(std::string host, guint32 port, std::string nick);
   void request_seat_manifest();
 
   sigc::signal<void> client_connected;
@@ -57,7 +57,7 @@ public:
   void chat(std::string message);
 
   std::string getHost() const{return d_host;};
-  Uint32 getPort() const{return d_port;};
+  guint32 getPort() const{return d_port;};
 
   void sendRoundOver();
 protected:
@@ -90,7 +90,7 @@ private:
   static GameClient * s_instance;
   bool d_connected;
   std::string d_host;
-  Uint32 d_port;
+  guint32 d_port;
 };
 
 #endif
