@@ -635,9 +635,7 @@ void GameServer::sendMap(Participant *part)
     {
       bool connected = false;
       players.push_back(*i);
-      if ((*i)->getType() == Player::AI_FAST ||
-	  (*i)->getType() == Player::AI_SMART ||
-	  (*i)->getType() == Player::AI_DUMMY)
+      if ((*i)->isComputer() == true)
 	connected = true;
       NetworkPlayer *new_p = new NetworkPlayer(**i);
       new_p->setConnected(connected);
@@ -645,12 +643,18 @@ void GameServer::sendMap(Participant *part)
     }
 
 
-  // send the map
-  std::ostringstream os;
-  XML_Helper helper(&os);
-  d_game_scenario->saveWithHelper(helper);
+  // send the map, and save it to a file somewhere temporarily
+  std::string tmpfile = "lw.XXXX";
+  int fd = Glib::file_open_tmp(tmpfile, "lw.XXXX");
+  close(fd);
+  File::erase(tmpfile);
+  tmpfile += ".sav";
+
+  d_game_scenario->saveGame(tmpfile, "sav");
+
   std::cerr << "sending map" << std::endl;
-  network_server->send(part->conn, MESSAGE_TYPE_SENDING_MAP, os.str());
+  network_server->send(part->conn, MESSAGE_TYPE_SENDING_MAP, tmpfile);
+  File::erase (tmpfile);
 
   // unhack the players
   std::vector<Player*>::iterator j = players.begin();
