@@ -68,9 +68,14 @@
 #include "LocationBox.h"
 #include "Backpack.h"
 #include "MapBackpack.h"
+#include "stacktile.h"
 
 #include "herotemplates.h"
 #include "GameScenarioOptions.h"
+#include "ai_fast.h"
+#include "ai_smart.h"
+#include "Sage.h"
+#include "Commentator.h"
 
 Game *Game::current_game = 0;
 
@@ -90,11 +95,9 @@ void Game::addPlayer(Player *p)
   if (p->getType() == Player::HUMAN)
     {
       connections[p->getId()].push_back
-	(p->snewLevelArmy.connect(sigc::mem_fun(this, &Game::newLevelArmy)));
+	(p->sheroGainsLevel.connect(sigc::mem_fun(this, &Game::heroGainsLevel)));
       connections[p->getId()].push_back
 	(p->snewMedalArmy.connect(sigc::mem_fun(this, &Game::newMedalArmy)));
-      connections[p->getId()].push_back
-	(p->srecruitingHero.connect(sigc::mem_fun(this, &Game::recruitHero)));
 
       connections[p->getId()].push_back
 	(p->streachery.connect
@@ -117,7 +120,26 @@ void Game::addPlayer(Player *p)
 	 (sigc::mem_fun(this, &Game::on_stack_grouped)));
     }
       
+      
   //now do all of the common connections
+      
+  connections[p->getId()].push_back
+    (p->getStacklist()->snewpos.connect
+     (sigc::mem_fun(stack_moves, &sigc::signal<void, Stack*, Vector<int> >::emit)));
+  connections[p->getId()].push_back
+    (p->srecruitingHero.connect(sigc::mem_fun(this, &Game::recruitHero)));
+  connections[p->getId()].push_back
+    (p->svisitingTemple.connect
+     (sigc::mem_fun(this, &Game::stack_searches_temple)));
+  connections[p->getId()].push_back
+    (p->ssearchingRuin.connect
+     (sigc::mem_fun(this, &Game::stack_searches_ruin)));
+  connections[p->getId()].push_back
+    (p->getStacklist()->snewpos.connect
+     (sigc::mem_fun(this, &Game::stack_arrives_on_tile)));
+  connections[p->getId()].push_back
+    (p->getStacklist()->soldpos.connect
+     (sigc::mem_fun(this, &Game::stack_leaves_tile)));
   connections[p->getId()].push_back
     (p->aborted_turn.connect (sigc::mem_fun
 	   (game_stopped, &sigc::signal<void>::emit)));
@@ -190,8 +212,12 @@ Game::Game(GameScenario* gameScenario, NextTurn *nextTurn)
 	sigc::mem_fun(this, &Game::on_stack_grouped_or_ungrouped));
     bigmap->path_set.connect(
 	sigc::mem_fun(this, &Game::update_control_panel));
+    bigmap->city_visited.connect(
+	sigc::mem_fun(this, &Game::on_city_visited));
     bigmap->city_queried.connect(
 	sigc::mem_fun(this, &Game::on_city_queried));
+    bigmap->city_unqueried.connect(
+	sigc::mem_fun(this, &Game::on_city_unqueried));
     bigmap->ruin_queried.connect(
 	sigc::mem_fun(this, &Game::on_ruin_queried));
     bigmap->signpost_queried.connect(
@@ -200,6 +226,8 @@ Game::Game(GameScenario* gameScenario, NextTurn *nextTurn)
 	sigc::mem_fun(this, &Game::on_temple_queried));
     bigmap->stack_queried.connect(
 	sigc::mem_fun(this, &Game::on_stack_queried));
+    bigmap->stack_unqueried.connect(
+	sigc::mem_fun(this, &Game::on_stack_unqueried));
 
     // init the smallmap
     smallmap.reset(new SmallMap);
@@ -269,9 +297,9 @@ GameScenario *Game::getScenario()
   return current_game->d_gameScenario;
 }
 
-
 void Game::end_turn()
 {
+  //only human players hit this.
     unselect_active_stack();
     clear_stack_info();
     update_control_panel();
@@ -315,7 +343,7 @@ void Game::update_sidebar_stats()
     s.cities = Citylist::getInstance()->countCities(player);
 
     s.units = 0;
-    s.upkeep = player->getUpkeep();
+    s.upkeep = player->getUpkeep() + player->getCostOfUnitsProducedThisTurn();
     Stacklist *sl = player->getStacklist();
     for (Stacklist::iterator i = sl->begin(), iend = sl->end(); i != iend; ++i)
       s.units += (*i)->size();
@@ -329,13 +357,13 @@ void Game::redraw()
 {
     if (bigmap.get())
       {
-	bigmap->draw();
+	bigmap->draw(Playerlist::getViewingplayer());
       }
     if (smallmap.get())
       {
 	//if (Playerlist::getActiveplayer()->getType() == Player::HUMAN ||
 	    //GameScenario::s_hidden_map == false)
-	  smallmap->draw(Playerlist::getActiveplayer());
+	smallmap->draw(Playerlist::getActiveplayer());
       }
 }
 
@@ -368,7 +396,7 @@ void Game::move_selected_stack_dir(int diffx, int diffy)
     move_selected_stack_along_path();
   else
     {
-      Playerlist::getActiveplayer()->getStacklist()->setActivestack(0);
+      Playerlist::getActiveplayer()->setActivestack(0);
       unselect_active_stack();
     }
 }
@@ -440,8 +468,6 @@ void Game::move_selected_stack_southeast()
 void Game::move_selected_stack_along_path()
 {
   Stack *stack = Playerlist::getActiveplayer()->getActivestack();
-  if (stack->isGrouped() == false)
-    Playerlist::getActiveplayer()->stackSplit(stack);
 
   Playerlist::getActiveplayer()->stackMove(stack);
 
@@ -449,7 +475,7 @@ void Game::move_selected_stack_along_path()
   stack = Playerlist::getActiveplayer()->getActivestack();
   if (stack && stack->canMove() == false)
     {
-      Playerlist::getActiveplayer()->getStacklist()->setActivestack(0);
+      Playerlist::getActiveplayer()->setActivestack(0);
       unselect_active_stack();
     }
 }
@@ -466,8 +492,6 @@ void Game::move_all_stacks()
 	{
 	  sl->setActivestack(&s);
 	  select_active_stack();
-	  if (player->getActivestack()->isGrouped() == false)
-	    player->stackSplit(player->getActivestack());
 	  bool moved = player->stackMove(player->getActivestack());
 	  if (!moved)
 	    break;
@@ -477,7 +501,7 @@ void Game::move_all_stacks()
 
   if (sl->getActivestack()->canMove() == false)
     {
-      Playerlist::getActiveplayer()->getStacklist()->setActivestack(0);
+      Playerlist::getActiveplayer()->setActivestack(0);
       unselect_active_stack();
     }
 }
@@ -491,7 +515,7 @@ void Game::defend_selected_stack()
   stack->setDefending(true);
 
   stack = player->getStacklist()->getNextMovable();
-  player->getStacklist()->setActivestack(stack);
+  player->setActivestack(stack);
 
   if (stack)
     select_active_stack();
@@ -507,7 +531,7 @@ void Game::park_selected_stack()
   stack->setParked(true);
 
   stack = player->getStacklist()->getNextMovable();
-  player->getStacklist()->setActivestack(stack);
+  player->setActivestack(stack);
 
   if (stack)
     select_active_stack();
@@ -518,7 +542,7 @@ void Game::park_selected_stack()
 void Game::deselect_selected_stack()
 {
   Player *player = Playerlist::getActiveplayer();
-  player->getStacklist()->setActivestack(0);
+  player->setActivestack(0);
   unselect_active_stack();
 }
 
@@ -529,13 +553,11 @@ void Game::center_selected_stack()
     select_active_stack();
 }
 
-void Game::search_selected_stack()
+void Game::search_stack(Stack *stack)
 {
   Player *player = Playerlist::getActiveplayer();
-  Stack* stack = player->getActivestack();
-
-  Ruin* ruin = GameMap::getRuin(stack->getPos());
-  Temple* temple = GameMap::getTemple(stack->getPos());
+  Ruin* ruin = GameMap::getRuin(stack);
+  Temple* temple = GameMap::getTemple(stack);
 
   if (ruin && !ruin->isSearched() && stack->hasHero() &&
       stack->getFirstHero()->getMoves() > 0 &&
@@ -547,19 +569,22 @@ void Game::search_selected_stack()
       reward = player->stackSearchRuin(stack, ruin);
       if (ruin->hasSage() == true)
 	{
-	  sage_visited.emit(ruin, stack);
-	  reward = ruin->getReward();
+          Sage *sage = ruin->generateSage();
+          if (player->isComputer() == false)
+            reward = sage_visited.emit(ruin, sage, stack);
+          else
+            reward = player->chooseReward(ruin, sage, stack);
 	}
 	  
       if (reward)
 	{
-	  player->giveReward(Playerlist::getActiveplayer()->getActivestack(),
-			     reward);
+	  player->giveReward(stack, reward);
 	  //FIXME: delete this reward, but don't delete the item, or map
 	  redraw();
 	  update_stack_info();
 	  update_control_panel();
-	  ruin_searched.emit(ruin, stack, reward);
+          if (player->isComputer() == false)
+            ruin_searched.emit(ruin, stack, reward);
 	}
       else
 	{
@@ -568,34 +593,46 @@ void Game::search_selected_stack()
 	  update_control_panel();
 	}
 
-
-
       update_sidebar_stats();
     }
-  else if (temple && temple->searchable() && stack->getGroupMoves() > 0)
+  else if (temple && temple->searchable() && stack->getMoves() > 0)
     {
       int blessCount;
       blessCount = player->stackVisitTemple(stack, temple);
-      bool wants_quest = temple_searched.emit(stack->hasHero(), temple, blessCount);
+      bool wants_quest;
+      Hero *hero = stack->getFirstHeroWithoutAQuest();
+      if (player->isComputer() == false)
+        wants_quest = temple_searched.emit(hero, temple, blessCount);
+      else
+        wants_quest = player->chooseQuest(hero);
       if (wants_quest && stack->hasHero())
 	{
-	  Quest *q = player->stackGetQuest
-	    (stack, temple, 
-	     GameScenario::s_razing_cities != GameParameters::NEVER);
-	  Hero* hero = dynamic_cast<Hero*>(stack->getFirstHero());
+	  Quest *q = player->heroGetQuest 
+            (hero, temple, 
+             GameScenario::s_razing_cities != GameParameters::NEVER);
 
 	  if (q)
-	    {
-	      for (Stack::iterator it = stack->begin(); it != stack->end(); it++)
-		if ((*it)->getId() == q->getHeroId())
-		  hero = dynamic_cast<Hero*>(*it);
-
-	    }
-	  quest_assigned.emit(hero, q);
+            {
+              if (player->isComputer() == false)
+                {
+                  Hero *hero;
+                  Army *a = stack->getArmyById(q->getHeroId());
+                  if (a)
+                    hero = dynamic_cast<Hero*>(a);
+                  quest_assigned.emit(hero, q);
+                }
+            }
 	}
     }
 }
 
+void Game::search_selected_stack()
+{
+  Player *player = Playerlist::getActiveplayer();
+  Stack* stack = player->getActivestack();
+  return search_stack(stack);
+}
+
 void Game::stackUpdate(Stack* s)
 {
   if (!s)
@@ -608,24 +645,21 @@ void Game::stackUpdate(Stack* s)
   if (s)
     smallmap->center_view_on_tile(s->getPos(), true);
 
-  redraw();
+  //redraw();
 
   update_stack_info();
   update_control_panel();
 
-  // sleep for a specified amount of time
-  //SDL_Delay(Configuration::s_displaySpeedDelay);
-  Glib::usleep(Configuration::s_displaySpeedDelay);
 }
 
-Army::Stat Game::newLevelArmy(Army* a)
+Army::Stat Game::heroGainsLevel(Hero * h)
 {
   // don't show a dialog if computer or enemy's armies advance
-  if ((a->getOwner()->getType() != Player::HUMAN) ||
-      (a->getOwner() != Playerlist::getInstance()->getActiveplayer()))
-    return Army::STRENGTH;
+  if (h->getOwner()->isComputer() == true ||
+      h->getOwner() != Playerlist::getInstance()->getActiveplayer())
+    return Playerlist::getInstance()->getActiveplayer()->chooseStat(h);
 
-  return army_gains_level.emit(a);
+  return hero_gains_level.emit(h);
 }
 
 void Game::newMedalArmy(Army* a, int medaltype)
@@ -653,48 +687,25 @@ void Game::on_stack_selected(Stack* s)
   update_control_panel();
 }
 
-void Game::on_city_queried (City* c, bool brief)
+void Game::on_city_queried (Vector<int> tile, City *c)
+{
+  MapTipPosition mpos = bigmap->map_tip_position(tile);
+  city_tip_changed.emit(c, mpos);
+}
+
+void Game::on_city_unqueried ()
+{
+  city_tip_changed.emit(NULL, MapTipPosition());
+}
+
+void Game::on_city_visited(City* c)
 {
   if (c)
     {
-      Player *player = c->getOwner();
-
-      if (brief)
-	{
-	  Glib::ustring str;
-
-	  if (c->isCapital())
-	    str = String::ucompose(_("%1 (capital city)"), c->getName());
-	  else
-	    str = c->getName();
-	  str += "\n";
-	  if (player != Playerlist::getInstance()->getNeutral())
-	    {
-	      str += String::ucompose(_("Under rule of %1"), player->getName());
-	      str += "\n";
-	    }
-	  str += String::ucompose(_("Income: %1"), c->getGold());
-	  str += "\n";
-	  str += String::ucompose(_("Defense: %1"), c->getDefenseLevel());
-	  if (c->isBurnt())
-	    {
-	      str += "\n";
-	      str += _("Status: razed!");
-	    }
-
-	  MapTipPosition mpos = bigmap->map_tip_position(c->get_area());
-	  map_tip_changed.emit(str, mpos);
-	}
-      else
-	{
-	  city_visited.emit(c);
-
-	  // some visible city properties (razed) may have changed
-	  redraw();
-	}
+      city_visited.emit(c);
+      // some visible city properties (razed) may have changed
+      redraw();
     }
-  else
-    map_tip_changed.emit("", MapTipPosition());
 }
 
 void Game::on_ruin_queried (Ruin* r, bool brief)
@@ -714,7 +725,7 @@ void Game::on_ruin_queried (Ruin* r, bool brief)
 	    // note to translators: whether a ruin has been searched
 	    str += _("Unexplored");
 
-	  MapTipPosition mpos = bigmap->map_tip_position(r->get_area());
+	  MapTipPosition mpos = bigmap->map_tip_position(r->getArea());
 	  map_tip_changed.emit(str, mpos);
 	}
       else
@@ -734,22 +745,21 @@ void Game::on_signpost_queried (Signpost* s)
 
       str = s->getName();
 
-      MapTipPosition mpos = bigmap->map_tip_position(s->get_area());
+      MapTipPosition mpos = bigmap->map_tip_position(s->getArea());
       map_tip_changed.emit(str, mpos);
     }
   else
     map_tip_changed.emit("", MapTipPosition());
 }
 
-void Game::on_stack_queried (Stack* s)
+void Game::on_stack_unqueried ()
 {
-  if (s)
-    {
-      MapTipPosition mpos = bigmap->map_tip_position(s->getPos());
-      stack_tip_changed.emit(s, mpos);
-    }
-  else
-    stack_tip_changed.emit(NULL, MapTipPosition());
+  stack_tip_changed.emit(NULL, MapTipPosition());
+}
+void Game::on_stack_queried (Vector<int> tile)
+{
+  MapTipPosition mpos = bigmap->map_tip_position(tile);
+  stack_tip_changed.emit(GameMap::getStacks(tile), mpos);
 }
 
 void Game::on_temple_queried (Temple* t, bool brief)
@@ -762,7 +772,7 @@ void Game::on_temple_queried (Temple* t, bool brief)
 
 	  str = t->getName();
 
-	  MapTipPosition mpos = bigmap->map_tip_position(t->get_area());
+	  MapTipPosition mpos = bigmap->map_tip_position(t->getArea());
 	  map_tip_changed.emit(str, mpos);
 	}
       else
@@ -926,17 +936,17 @@ void Game::update_control_panel()
        * temples can be searched by any stack, when the stack has 
        * movement left.
        */
-      if (stack->getGroupMoves() > 0)
+      if (stack->getMoves() > 0)
 	{
 	  Temple *temple;
-	  temple = GameMap::getTemple(stack->getPos());
+	  temple = GameMap::getTemple(stack);
 	  can_search_selected_stack.emit(temple);
 	  can_move_selected_stack.emit(true);
 	}
 
       if (stack->hasHero())
 	{
-	  Ruin *ruin = GameMap::getRuin(stack->getPos());
+	  Ruin *ruin = GameMap::getRuin(stack);
 	  if (stack->getFirstHero()->getMoves() > 0 && ruin)
 	    can_search_selected_stack.emit(!ruin->isSearched());
 
@@ -950,9 +960,9 @@ void Game::update_control_panel()
 		    {
 			  //can't plant on city/ruin/temple/signpost
 			  City *city = GameMap::getCity(stack->getPos());
-			  Temple *temple = GameMap::getTemple(stack->getPos());
-			  Ruin *ruin = GameMap::getRuin(stack->getPos());
-			  Signpost *sign = GameMap::getSignpost(stack->getPos());
+			  Temple *temple = GameMap::getTemple(stack);
+			  Ruin *ruin = GameMap::getRuin(stack);
+			  Signpost *sign = GameMap::getSignpost(stack);
 			  if (!city && !temple && !ruin && !sign)
 			    {
 			      GameMap *gm = GameMap::getInstance();
@@ -974,7 +984,7 @@ void Game::update_control_panel()
 	  can_plant_standard_selected_stack.emit(false);
 	}
 
-      if (GameMap::getSignpost(stack->getPos()))
+      if (GameMap::getSignpost(stack))
 	can_change_signpost.emit(true);
 
       can_disband_stack.emit(true);
@@ -1043,7 +1053,7 @@ void Game::loadGame()
     {
       //human players want access to the controls and an info box
       unlock_inputs();
-      player->getStacklist()->setActivestack(0);
+      player->setActivestack(0);
       center_view_on_city();
       update_sidebar_stats();
       update_control_panel();
@@ -1058,7 +1068,7 @@ void Game::loadGame()
       if (player->hasAlreadyInitializedTurn())
 	{
 	  unlock_inputs();
-	  player->getStacklist()->setActivestack(0);
+	  player->setActivestack(0);
 	  center_view_on_city();
 	  update_sidebar_stats();
 	  update_control_panel();
@@ -1072,6 +1082,8 @@ void Game::loadGame()
 	  lock_inputs();
 	}
     }
+  else
+    lock_inputs();
 
   d_nextTurn->start();
 #if 0
@@ -1097,21 +1109,41 @@ bool Game::saveGame(std::string file)
   return d_gameScenario->saveGame(file);
 }
 
+void Game::blank(bool on)
+{
+  if (GameScenarioOptions::s_hidden_map == true)
+    {
+      bigmap->blank(on);
+      smallmap->blank(on);
+    }
+}
+
 void Game::init_turn_for_player(Player* p)
 {
   Playerlist* pl = Playerlist::getInstance();
 
-  if (GameScenario::s_hidden_map)
-    {
-      smallmap->blank();
-      bigmap->blank();
-    }
+  blank(true);
 
   next_player_turn.emit(p, d_gameScenario->getRound());
 
+  blank(false);
+
   if (p->isObservable() == true)
     center_view_on_city();
 
+  if (p->getType() == Player::HUMAN)
+    {
+      if (Commentator::getInstance()->hasComment() == true)
+        {
+          std::vector<std::string> comments =
+            Commentator::getInstance()->getComments(p);
+          if (comments.size() > 0)
+            commentator_comments.emit(comments[rand() % comments.size()]);
+        }
+    }
+
+  p->maybeRecruitHero();
+
   if (p->getType() == Player::HUMAN)
     {
       unlock_inputs();
@@ -1140,16 +1172,8 @@ void Game::init_turn_for_player(Player* p)
       received_diplomatic_proposal.emit(proposal_received);
       //check to see if we've turned off production due to destitution.
       bool destitute = false;
-      std::list<Action*> actions = p->getReportableActions();
-      std::list<Action*>::iterator it = actions.begin();
-      for (;it != actions.end(); it++)
-	{
-	  if ((*it)->getType() == Action::CITY_DESTITUTE)
-	    {
-	      destitute = true;
-	      break;
-	    }
-	}
+      if (p->countDestituteCitiesThisTurn() > 0)
+        destitute = true;
       city_too_poor_to_produce.emit(destitute);
     }
   else
@@ -1194,12 +1218,16 @@ void Game::on_fight_started(Fight &fight)
       Vector<int> pos = fight.getAttackers().front()->getPos();
       if (GameScenario::s_hidden_map == false)
 	smallmap->center_view_on_tile(pos, true);
-      LocationBox box = Fight::calculateFightBox(fight);
-      bigmap->setFighting(box);
-      bigmap->draw();
-      fight_started.emit(fight);
-      bigmap->setFighting(LocationBox(Vector<int>(-1,-1)));
-      bigmap->draw();
+      fight_started.emit(Fight::calculateFightBox(fight), fight);
+    }
+  else if ((Playerlist::getActiveplayer()->isObservable() == true ||
+      attacking_observable_player) && ai_attacking_neutral &&
+      !ai_attacking_on_hidden_map)
+    {
+      Vector<int> pos = fight.getAttackers().front()->getPos();
+      if (GameScenario::s_hidden_map == false)
+	smallmap->center_view_on_tile(pos, true);
+      abbreviated_fight_started.emit(Fight::calculateFightBox(fight));
     }
 }
 
@@ -1257,7 +1285,20 @@ void Game::unselect_active_stack()
 bool Game::maybeTreachery(Stack *stack, Player *them, Vector<int> pos)
 {
   Player *me = stack->getOwner();
-  bool treachery = me->treachery (stack, them, pos);
+  bool treachery = false;
+  if (me->isComputer())
+    {
+      if (me->getType() == Player::AI_FAST)
+        {
+          AI_Fast *ai = dynamic_cast<AI_Fast*>(me);
+          treachery = ai->chooseTreachery (stack, them, pos);
+        }
+      else if (me->getType() == Player::AI_SMART)
+        {
+          AI_Smart *ai = dynamic_cast<AI_Smart*>(me);
+          treachery = ai->chooseTreachery (stack, them, pos);
+        }
+    }
   if (treachery == false)
     return false;
   me->proposeDiplomacy (Player::NO_PROPOSAL, them);
@@ -1322,7 +1363,7 @@ void Game::on_city_fight_finished(City *city, Fight::Result result)
 	  //great, then let's turn on the production.
 	  //well, we already made a unit, and we want to produce more
 	  //of it.
-	  Stack *o = neu->getStacklist()->getObjectAt(city->getPos());
+	  Stack *o = GameMap::getStacks(city->getPos())->getFriendlyStack(neu);
 	  if (o)
 	    {
 	      int army_type = o->getStrongestArmy()->getTypeId();
@@ -1343,9 +1384,15 @@ void Game::on_city_fight_finished(City *city, Fight::Result result)
     
 bool Game::recruitHero(HeroProto *hero, City *city, int gold)
 {
-  bool retval = hero_offers_service.emit (city->getOwner(), hero, city, gold);
-  if (d_gameScenario->getRound() == 1)
-    city_visited.emit(city);
+  bool retval; 
+  if (city->getOwner()->isComputer())
+    retval = city->getOwner()->chooseHero (hero, city, gold);
+  else
+    {
+      retval = hero_offers_service.emit (city->getOwner(), hero, city, gold);
+      if (d_gameScenario->getRound() == 1)
+        city_visited.emit(city);
+    }
   return retval;
 }
     
@@ -1383,3 +1430,34 @@ void Game::endOfGameRoaming(Player *winner)
   update_control_panel();
   redraw();
 }
+
+void Game::stack_arrives_on_tile(Stack *stack, Vector<int> tile)
+{
+  StackTile *stile = GameMap::getInstance()->getTile(tile)->getStacks();
+  stile->arriving(stack);
+}
+
+void Game::stack_leaves_tile(Stack *stack, Vector<int> tile)
+{
+  StackTile *stile = GameMap::getInstance()->getTile(tile)->getStacks();
+  bool left = stile->leaving(stack);
+  if (left == false)
+    {
+      if (stack == NULL)
+	{
+	  printf("stack is %p\n", stack);
+	  printf("WTFFF!!!!!!!!!!!!!!!!!!!!\n");
+	  return;
+	}
+    }
+}
+    
+void Game::stack_searches_ruin(Ruin *ruin, Stack *stack)
+{
+  search_stack(stack);
+}
+    
+void Game::stack_searches_temple(Temple *temple, Stack *stack)
+{
+  search_stack(stack);
+}
