@@ -26,7 +26,6 @@
 
 #include "army.h"
 #include "path.h"
-#include "stacklist.h"
 #include "stack.h"
 #include "city.h"
 #include "ruin.h"
@@ -40,13 +39,13 @@
 #include "File.h"
 #include "GameMap.h"
 #include "GraphicsCache.h"
-#include "GraphicsLoader.h"
 #include "game.h"
 #include "FogMap.h"
 #include "LocationBox.h"
 #include "Configuration.h"
 #include "gui/image-helpers.h"
 #include "PathCalculator.h"
+#include "stacktile.h"
 
 #include "timing.h"
 
@@ -58,7 +57,8 @@ using namespace std;
 
 namespace 
 {
-    int selection_timeout = 150;	// controls speed of selector rotation
+    // controls speed of selector rotation
+    int selection_timeout = TIMER_BIGMAP_SELECTOR;
 }
 
 GameBigMap::GameBigMap(bool intense_combat, bool see_opponents_production,
@@ -80,9 +80,9 @@ GameBigMap::GameBigMap(bool intense_combat, bool see_opponents_production,
 
   // setup timeout
   selection_timeout_handler = 
-    Timing::instance().register_timer(
-				      sigc::mem_fun(*this, &GameBigMap::on_selection_timeout),
-				      selection_timeout);
+    Timing::instance().register_timer
+    (sigc::mem_fun(*this, &GameBigMap::on_selection_timeout),
+     selection_timeout);
   shift_key_is_down = false;
   control_key_is_down = false;
 }
@@ -102,9 +102,10 @@ void GameBigMap::select_active_stack()
 
   if (stack->getPath()->checkPath(stack) == false)
     {
+      assert (Playerlist::getActiveplayer()->getType() == Player::HUMAN);
       //original path was blocked, so let's find a new way there.
       //this shouldn't happen because nextTurn of stack recalculates.
-      cerr << "original path of stack was blocked\n";
+      //cerr << "original path of stack was blocked\n";
       stack->getPath()->recalculate(stack);
     }
 
@@ -113,7 +114,7 @@ void GameBigMap::select_active_stack()
 
 void GameBigMap::unselect_active_stack()
 {
-  draw();
+  draw(Playerlist::getViewingplayer());
   stack_selected.emit(0);
   if (path_calculator)
     {
@@ -127,7 +128,7 @@ bool GameBigMap::on_selection_timeout()
 {
   // redraw to update the selection
   if (Playerlist::getActiveplayer()->getActivestack())
-    draw();
+    draw(Playerlist::getViewingplayer());
 
   return Timing::CONTINUE;
 }
@@ -138,6 +139,7 @@ void GameBigMap::mouse_button_event(MouseButtonEvent e)
     return;
 
   Player *active = Playerlist::getActiveplayer();
+  Player *viewing = Playerlist::getViewingplayer();
   Vector<int> tile = mouse_pos_to_tile(e.pos);
   current_tile = tile;
 
@@ -158,7 +160,7 @@ void GameBigMap::mouse_button_event(MouseButtonEvent e)
 	    double_clicked = true;
 	  last_clicked = clicked_now;
 	}
-      if (active->getFogMap()->isCompletelyObscuredFogTile(tile) == true)
+      if (viewing->getFogMap()->isCompletelyObscuredFogTile(tile) == true)
 	return;
 
       Stack* stack = Playerlist::getActiveplayer()->getActivestack();
@@ -182,7 +184,7 @@ void GameBigMap::mouse_button_event(MouseButtonEvent e)
 	      if (Ruin *r = GameMap::getRuin(tile))
 		{
 		  if ((r->isHidden() == true && 
-		       r->getOwner() == Playerlist::getActiveplayer()) ||
+		       r->getOwner() == viewing) ||
 		      r->isHidden() == false)
 		    {
 		      set_shift_key_down (false);
@@ -206,7 +208,7 @@ void GameBigMap::mouse_button_event(MouseButtonEvent e)
 		      set_control_key_down (false);
 		      if (d_see_opponents_production == true)
 			{
-			  city_queried (c, false);
+			  city_visited.emit (c);
 			  set_shift_key_down (false);
 			  return;
 			}
@@ -214,7 +216,7 @@ void GameBigMap::mouse_button_event(MouseButtonEvent e)
 			{
 			  if (c->getOwner() == Playerlist::getActiveplayer())
 			    {
-			      city_queried (c, false);
+			      city_visited.emit(c);
 			      set_shift_key_down (false);
 			      return;
 			    }
@@ -227,7 +229,7 @@ void GameBigMap::mouse_button_event(MouseButtonEvent e)
 	      //set in a course, mr crusher.
 	      stack->getPath()->calculate(stack, tile);
 	      path_set.emit();
-	      draw();
+	      draw(Playerlist::getViewingplayer());
 	      return;
 	    }
 	  Vector<int> p;
@@ -238,14 +240,20 @@ void GameBigMap::mouse_button_event(MouseButtonEvent e)
 	    {
 	      if (double_clicked == true)
 		{
-		  if (stack->isGrouped() == true)
-		    stack->ungroup();
+		  StackTile *stile = GameMap::getStacks(stack->getPos());
+		  std::list<Stack *> stks= stile->getFriendlyStacks(active);
+		  if (stks.size() == 1)
+		    stile->ungroup(active);
 		  else
-		    stack->group();
+		    {
+		      stile->group(active);
+		      active->setActivestack(GameMap::getStack(tile));
+		      stack_selected.emit(GameMap::getStack(tile));
+		    }
 		  if (path_calculator)
 		    delete path_calculator;
 		  path_calculator = new PathCalculator(stack);
-		  draw();
+		  draw(Playerlist::getViewingplayer());
 		  stack_grouped_or_ungrouped.emit(stack);
 		  return;
 		}
@@ -254,7 +262,7 @@ void GameBigMap::mouse_button_event(MouseButtonEvent e)
 		  // clear the path
 		  stack->getPath()->clear();
 		  path_set.emit();
-		  draw();
+		  draw(Playerlist::getViewingplayer());
 		  return;
 		}
 	    }
@@ -275,9 +283,6 @@ void GameBigMap::mouse_button_event(MouseButtonEvent e)
 		}
 	    }
 
-	  // split if ungrouped
-	  Playerlist::getActiveplayer()->stackSplit(stack); 
-
 	  int dist = stack->getPath()->calculate(stack, p);
 	  if (dist == -2)
 	    cerr << "error calculating path!";
@@ -337,7 +342,7 @@ void GameBigMap::mouse_button_event(MouseButtonEvent e)
 		  if (deselect)
 		    {
 		      Player *player = Playerlist::getActiveplayer();
-		      player->getStacklist()->setActivestack(0);
+		      player->setActivestack(0);
 		      unselect_active_stack();
 		    }
 		}
@@ -345,7 +350,7 @@ void GameBigMap::mouse_button_event(MouseButtonEvent e)
 
 	  path_set.emit();
 
-	  draw();
+	  draw(Playerlist::getViewingplayer());
 	}
       // Stack hasn't been active yet
       else
@@ -354,7 +359,7 @@ void GameBigMap::mouse_button_event(MouseButtonEvent e)
 	  if (stack && stack->isFriend(Playerlist::getActiveplayer()) && 
 	      d_cursor == GraphicsCache::TARGET)
 	    {
-	      Playerlist::getActiveplayer()->getStacklist()->setActivestack(stack);
+	      Playerlist::getActiveplayer()->setActivestack(stack);
 	      select_active_stack();
 	    }
 	  else
@@ -367,14 +372,14 @@ void GameBigMap::mouse_button_event(MouseButtonEvent e)
 		      set_control_key_down (false);
 		      if (d_see_opponents_production == true)
 			{
-			  city_queried (c, false);
+			  city_visited.emit (c);
 			  set_shift_key_down (false);
 			}
 		      else
 			{
 			  if (c->getOwner() == Playerlist::getActiveplayer())
 			    {
-			      city_queried (c, false);
+			      city_visited.emit (c);
 			      set_shift_key_down (false);
 			    }
 			}
@@ -382,8 +387,7 @@ void GameBigMap::mouse_button_event(MouseButtonEvent e)
 		}
 	      else if (Ruin *r = GameMap::getRuin(tile))
 		{
-		  if ((r->isHidden() == true && 
-		       r->getOwner() == Playerlist::getActiveplayer()) ||
+		  if ((r->isHidden() == true && r->getOwner() == viewing) ||
 		      r->isHidden() == false)
 		    ruin_queried (r, false);
 		}
@@ -439,17 +443,17 @@ void GameBigMap::mouse_button_event(MouseButtonEvent e)
     {
       if (e.state == MouseButtonEvent::PRESSED)
 	{
-	  if (active->getFogMap()->isCompletelyObscuredFogTile(tile) == true)
+	  if (viewing->getFogMap()->isCompletelyObscuredFogTile(tile) == true)
 	    return;
 	  if (City* c = GameMap::getCity(tile))
 	    {
-	      city_queried (c, true);
+	      city_queried.emit (tile, c);
 	      mouse_state = SHOWING_CITY;
 	    }
 	  else if (Ruin* r = GameMap::getRuin(tile))
 	    {
 	      if ((r->isHidden() == true && 
-		   r->getOwner() == Playerlist::getActiveplayer()) ||
+		   r->getOwner() == Playerlist::getViewingplayer()) ||
 		  r->isHidden() == false)
 		{
 		  ruin_queried (r, true);
@@ -470,13 +474,13 @@ void GameBigMap::mouse_button_event(MouseButtonEvent e)
 	    {
 	      if (d_see_opponents_stacks == true)
 		{
-		  stack_queried.emit(st);
+		  stack_queried.emit(tile);
 		  mouse_state = SHOWING_STACK;
 		}
 	      else if (st->getOwner() == Playerlist::getActiveplayer() && 
 		       d_see_opponents_stacks == false)
 		{
-		  stack_queried.emit(st);
+		  stack_queried.emit(tile);
 		  mouse_state = SHOWING_STACK;
 		}
 	    }
@@ -487,7 +491,7 @@ void GameBigMap::mouse_button_event(MouseButtonEvent e)
 	    {
 
 	    case SHOWING_CITY:
-	      city_queried.emit(0, true);
+	      city_unqueried.emit();
 	      break;
 
 	    case SHOWING_RUIN:
@@ -503,7 +507,7 @@ void GameBigMap::mouse_button_event(MouseButtonEvent e)
 	      break;
 
 	    case SHOWING_STACK:
-	      stack_queried.emit(0);
+	      stack_unqueried.emit();
 	      break;
 
 	    case DRAGGING_ENDPOINT:
@@ -513,7 +517,7 @@ void GameBigMap::mouse_button_event(MouseButtonEvent e)
 	      Stack* stack = Playerlist::getActiveplayer()->getActivestack();
 	      if (stack)
 		{
-		  Playerlist::getActiveplayer()->getStacklist()->setActivestack(0);
+		  Playerlist::getActiveplayer()->setActivestack(0);
 		  unselect_active_stack();
 		  mouse_state = NONE;
 		  determine_mouse_cursor(NULL, current_tile);
@@ -574,7 +578,8 @@ void GameBigMap::zoom_out()
 void GameBigMap::determine_mouse_cursor(Stack *stack, Vector<int> tile)
 {
   Player *active = Playerlist::getActiveplayer();
-  if (active->getFogMap()->isCompletelyObscuredFogTile(tile))
+  Player *viewing = Playerlist::getViewingplayer();
+  if (viewing->getFogMap()->isCompletelyObscuredFogTile(tile))
     {
       d_cursor = GraphicsCache::HAND;
     }
@@ -694,7 +699,7 @@ void GameBigMap::determine_mouse_cursor(Stack *stack, Vector<int> tile)
       d_cursor = GraphicsCache::HAND;
       Stack *st;
 
-      st = active->getStacklist()->getObjectAt(tile);
+      st = GameMap::getStack(tile);
       if (st)
 	{
 	  if (st->getOwner() == active)
@@ -739,6 +744,7 @@ void GameBigMap::mouse_motion_event(MouseMotionEvent e)
     return;
 
   Player *active = Playerlist::getActiveplayer();
+  Player *viewing = Playerlist::getViewingplayer();
   Stack* stack = active->getActivestack();
   Vector<int> tile = mouse_pos_to_tile(e.pos);
   current_tile = tile;
@@ -754,7 +760,7 @@ void GameBigMap::mouse_motion_event(MouseMotionEvent e)
 
   if (e.pressed[MouseMotionEvent::LEFT_BUTTON]
       && (mouse_state == NONE || mouse_state == SHOWING_STACK) && 
-      stack && stack->getPos() == tile && active->getFogMap()->isCompletelyObscuredFogTile(tile) == false && d_cursor != GraphicsCache::HAND)
+      stack && stack->getPos() == tile && viewing->getFogMap()->isCompletelyObscuredFogTile(tile) == false && d_cursor != GraphicsCache::HAND)
     {
       //initial dragging of stack from it's tile
       mouse_state = DRAGGING_STACK;
@@ -792,10 +798,10 @@ void GameBigMap::mouse_motion_event(MouseMotionEvent e)
 	  view.x = new_view.x;
 	  view.y = new_view.y;
 	  view_changed.emit(view);
-	  draw(true);
+	  draw(Playerlist::getViewingplayer(),true);
 	}
       else
-	draw(false);
+	draw(Playerlist::getViewingplayer(), false);
       mouse_state = DRAGGING_MAP;
     }
 
@@ -809,7 +815,7 @@ void GameBigMap::mouse_motion_event(MouseMotionEvent e)
   // drag stack with left mouse button
   if (e.pressed[MouseMotionEvent::LEFT_BUTTON]
       && (mouse_state == DRAGGING_STACK || mouse_state == DRAGGING_ENDPOINT) && 
-      active->getFogMap()->isCompletelyObscuredFogTile(tile) == false)
+      viewing->getFogMap()->isCompletelyObscuredFogTile(tile) == false)
     {
       //subsequent dragging
       //alright.  calculate the path, and show it but don't move
@@ -847,7 +853,7 @@ void GameBigMap::mouse_motion_event(MouseMotionEvent e)
 	  delete new_path;
 	  //stack->getPath()->calculate(stack, tile);
 	  path_set.emit();
-	  draw();
+	  draw(Playerlist::getViewingplayer());
 	}
     }
 
@@ -865,7 +871,7 @@ void GameBigMap::reset_zoom()
 {
   magnification_factor = 1.0;
   screen_size_changed(image);
-  draw(true);
+  draw(Playerlist::getViewingplayer(), true);
   view_changed.emit(view);
 }
 
@@ -878,12 +884,14 @@ void GameBigMap::zoom_view(double percent)
   //new_view.pos += Vector<int>(tiles*-1/2, tiles*-1/2);
   //set_view (new_view);
   screen_size_changed(image);
-  draw(true);
+  draw(Playerlist::getViewingplayer(), true);
   view_changed.emit(view);
 }
 
 void GameBigMap::after_draw()
 {
+  if (blank_screen == true)
+    return;
   GraphicsCache *gc = GraphicsCache::getInstance();
   int tilesize = GameMap::getInstance()->getTileset()->getTileSize();
 
@@ -932,10 +940,11 @@ void GameBigMap::after_draw()
 
   if (stack && d_fighting.getPos() == Vector<int>(-1,-1))
     {
+      Player *viewer = Playerlist::getViewingplayer();
       // draw the selection
       Vector<int> p = stack->getPos();
       if (is_inside(buffer_view, Vector<int>(p.x, p.y)) &&
-	  FogMap::isFogged(p, Playerlist::getActiveplayer()) == false)
+	  Playerlist::getViewingplayer()->getFogMap()->isFogged(p) == false)
 	{
 	  static int bigframe = -1;
 	  static int smallframe = -1;
@@ -950,23 +959,22 @@ void GameBigMap::after_draw()
 	    smallframe = 0;
 
 	  p = tile_to_buffer_pos(p);
-	  int num_selected = 0;
-	  for (Stack::iterator it = stack->begin(); it != stack->end(); it++)
-	    {
-	      if ((*it)->isGrouped())
-		num_selected++;
-	    }
 
 	  draw_stack (stack, buffer, buffer_gc);
 
-	  if (input_locked == false)
+	  PixMask *tmp = NULL;
+	  if (stack->size() > 1)
+	    tmp = gc->getSelectorPic(0, bigframe, stack->getOwner());
+	  else
+	    tmp = gc->getSelectorPic(1, smallframe, stack->getOwner());
+	  tmp->blit(buffer, p);
+	  //now re-fog it up because we just drew over the fog.
+	  if (viewer->getFogMap()->isFogged(stack->getPos()))
 	    {
-	      PixMask *tmp = NULL;
-	      if (num_selected > 1)
-		tmp = gc->getSelectorPic(0, bigframe, stack->getOwner());
-	      else
-		tmp = gc->getSelectorPic(1, smallframe, stack->getOwner());
-	      tmp->blit(buffer, p);
+	      int fog_type_id = 
+		viewer->getFogMap()->getShadeTile(stack->getPos());
+	      PixMask *fog = gc->getFogPic(fog_type_id);
+	      fog->blit(buffer, p);
 	    }
 	}
     }
@@ -989,7 +997,8 @@ void GameBigMap::set_control_key_down (bool down)
 {
   control_key_is_down = down;
   Player *active = Playerlist::getActiveplayer();
-  if (active->getFogMap()->isCompletelyObscuredFogTile(current_tile) == true)
+  Player *viewing = Playerlist::getViewingplayer();
+  if (viewing->getFogMap()->isCompletelyObscuredFogTile(current_tile) == true)
     return;
   Stack* active_stack = active->getActivestack();
   //if the key has been released, just show what we'd normally show.
@@ -1005,7 +1014,7 @@ void GameBigMap::set_control_key_down (bool down)
 	return;
 
       Stack* stack;
-      stack = Playerlist::getActiveplayer()->getStacklist()->getObjectAt(current_tile);
+      stack = GameMap::getFriendlyStack(current_tile);
       if (!stack)
 	return;
 
@@ -1043,7 +1052,8 @@ void GameBigMap::set_shift_key_down (bool down)
 {
   shift_key_is_down = down;
   Player *active = Playerlist::getActiveplayer();
-  if (active->getFogMap()->isCompletelyObscuredFogTile(current_tile) == true)
+  Player *viewing = Playerlist::getViewingplayer();
+  if (viewing->getFogMap()->isCompletelyObscuredFogTile(current_tile) == true)
       return;
 
   Stack* active_stack = active->getActivestack();
