@@ -1,9 +1,9 @@
-//  Copyright (C) 2007, Ole Laursen
-//  Copyright (C) 2007, 2008 Ben Asselstine
+//  Copyright (C) 2007 Ole Laursen
+//  Copyright (C) 2007, 2008, 2009 Ben Asselstine
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
-//  the Free Software Foundation; either version 2 of the License, or
+//  the Free Software Foundation; either version 3 of the License, or
 //  (at your option) any later version.
 //
 //  This program is distributed in the hope that it will be useful,
@@ -18,7 +18,7 @@
 
 #include <config.h>
 
-#include <libglademm/xml.h>
+#include <gtkmm.h>
 #include <sigc++/functors/mem_fun.h>
 
 #include "army-bonus-dialog.h"
@@ -31,19 +31,19 @@
 #include "armysetlist.h"
 #include "player.h"
 #include "GraphicsCache.h"
+#include "File.h"
 
 ArmyBonusDialog::ArmyBonusDialog(Player *p)
 {
     d_player = p;
-    Glib::RefPtr<Gnome::Glade::Xml> xml
-	= Gnome::Glade::Xml::create(get_glade_path()
-				    + "/army-bonus-dialog.glade");
+    Glib::RefPtr<Gtk::Builder> xml
+	= Gtk::Builder::create_from_file(get_glade_path()
+				    + "/army-bonus-dialog.ui");
 
-    Gtk::Dialog *d = 0;
-    xml->get_widget("dialog", d);
-    dialog.reset(d);
-    decorate(dialog.get());
-    window_closed.connect(sigc::mem_fun(dialog.get(), &Gtk::Dialog::hide));
+    xml->get_widget("dialog", dialog);
+    decorate(dialog);
+    window_closed.connect(sigc::mem_fun(dialog, &Gtk::Dialog::hide));
+    dialog->set_icon_from_file(File::getMiscFile("various/castle_icon.png"));
 
     armies_list = Gtk::ListStore::create(armies_columns);
     xml->get_widget("treeview", armies_treeview);
@@ -56,11 +56,15 @@ ArmyBonusDialog::ArmyBonusDialog(Player *p)
     armies_treeview->append_column("Bonus", armies_columns.bonus);
     armies_treeview->set_headers_visible(true);
 
-    Uint32 max = Armysetlist::getInstance()->getSize(d_player->getArmyset());
+    guint32 max = Armysetlist::getInstance()->getSize(d_player->getArmyset());
     for (unsigned int i = 0; i < max; i++)
       addArmyType(i);
 }
 
+ArmyBonusDialog::~ArmyBonusDialog()
+{
+  delete dialog;
+}
 void ArmyBonusDialog::set_parent_window(Gtk::Window &parent)
 {
     dialog->set_transient_for(parent);
@@ -87,7 +91,7 @@ void ArmyBonusDialog::run()
 
 }
 
-void ArmyBonusDialog::addArmyType(Uint32 army_type)
+void ArmyBonusDialog::addArmyType(guint32 army_type)
 {
     GraphicsCache *gc = GraphicsCache::getInstance();
     Player *p = d_player;
@@ -97,13 +101,12 @@ void ArmyBonusDialog::addArmyType(Uint32 army_type)
       return; //we don't want to show heroes in this list
     Gtk::TreeIter i = armies_list->append();
     (*i)[armies_columns.name] = a->getName();
-    (*i)[armies_columns.image] = to_pixbuf(gc->getArmyPic(p->getArmyset(),
-                                           army_type,
-                                           p, NULL));
+    (*i)[armies_columns.image] = gc->getArmyPic(p->getArmyset(), army_type,
+						p, NULL)->to_pixbuf();
     (*i)[armies_columns.str] = a->getStrength();
     (*i)[armies_columns.move] = a->getMaxMoves();
-    Uint32 b = a->getMoveBonus();
-    (*i)[armies_columns.move_image] = to_pixbuf(gc->getMoveBonusPic(b, false));
+    guint32 b = a->getMoveBonus();
+    (*i)[armies_columns.move_image] = gc->getMoveBonusPic(b, false)->to_pixbuf();
     (*i)[armies_columns.bonus] = "-";
 
     std::string s = a->getArmyBonusDescription();
