@@ -1,8 +1,8 @@
-//  Copyright (C) 2008 Ben Asselstine
+//  Copyright (C) 2008, 2009 Ben Asselstine
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
-//  the Free Software Foundation; either version 2 of the License, or
+//  the Free Software Foundation; either version 3 of the License, or
 //  (at your option) any later version.
 //
 //  This program is distributed in the hope that it will be useful,
@@ -21,9 +21,7 @@
 #include <memory>
 #include <vector>
 #include <sigc++/trackable.h>
-#include <gtkmm/dialog.h>
-#include <gtkmm/table.h>
-#include <gtkmm/togglebutton.h>
+#include <gtkmm.h>
 
 #include "player.h"
 
@@ -34,6 +32,7 @@ class DiplomacyDialog: public Decorated
 {
  public:
     DiplomacyDialog(Player *player);
+    ~DiplomacyDialog();
 
     void set_parent_window(Gtk::Window &parent);
 
@@ -41,9 +40,8 @@ class DiplomacyDialog: public Decorated
     void hide();
     
  private:
-    std::auto_ptr<Gtk::Dialog> dialog;
+    Gtk::Dialog* dialog;
 
-				     
     void on_report_clicked();
     void on_proposal_toggled (Gtk::ToggleButton *toggle, Player *p, Player::DiplomaticProposal proposal);
 
