@@ -1,8 +1,8 @@
-//  Copyright (C) 2007, 2008 Ben Asselstine
+//  Copyright (C) 2007, 2008, 2009 Ben Asselstine
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
-//  the Free Software Foundation; either version 2 of the License, or
+//  the Free Software Foundation; either version 3 of the License, or
 //  (at your option) any later version.
 //
 //  This program is distributed in the hope that it will be useful,
@@ -17,11 +17,11 @@
 
 #include <config.h>
 
-#include <libglademm/xml.h>
 #include <sigc++/functors/mem_fun.h>
 
 #include "fight-order-dialog.h"
 
+#include <gtkmm.h>
 #include "glade-helpers.h"
 #include "image-helpers.h"
 #include "ucompose.hpp"
@@ -36,15 +36,13 @@ FightOrderDialog::FightOrderDialog(Player *theplayer)
 {
     player = theplayer;
     
-    Glib::RefPtr<Gnome::Glade::Xml> xml
-	= Gnome::Glade::Xml::create(get_glade_path()
-				    + "/fight-order-dialog.glade");
+    Glib::RefPtr<Gtk::Builder> xml
+	= Gtk::Builder::create_from_file(get_glade_path()
+				    + "/fight-order-dialog.ui");
 
-    Gtk::Dialog *d = 0;
-    xml->get_widget("dialog", d);
-    dialog.reset(d);
-    decorate(dialog.get());
-    window_closed.connect(sigc::mem_fun(dialog.get(), &Gtk::Dialog::hide));
+    xml->get_widget("dialog", dialog);
+    decorate(dialog);
+    window_closed.connect(sigc::mem_fun(dialog, &Gtk::Dialog::hide));
 
     armies_list = Gtk::ListStore::create(armies_columns);
     xml->get_widget("treeview", armies_treeview);
@@ -52,13 +50,20 @@ FightOrderDialog::FightOrderDialog(Player *theplayer)
     armies_treeview->append_column("", armies_columns.image);
     armies_treeview->append_column("", armies_columns.name);
 
-    std::list<Uint32> fight_order = theplayer->getFightOrder();
-    std::list<Uint32>::iterator it = fight_order.begin();
+    std::list<guint32> fight_order = theplayer->getFightOrder();
+    std::list<guint32>::iterator it = fight_order.begin();
     for (; it != fight_order.end(); it++)
       {
         addArmyType(*it);
       }
     armies_treeview->set_reorderable(true);
+    xml->get_widget("reverse_button", reverse_button);
+    reverse_button->signal_clicked().connect
+      (sigc::mem_fun (*this, &FightOrderDialog::on_reverse_button_clicked));
+}
+FightOrderDialog::~FightOrderDialog()
+{
+  delete dialog;
 }
 
 void FightOrderDialog::set_parent_window(Gtk::Window &parent)
@@ -85,9 +90,9 @@ void FightOrderDialog::run()
 
     dialog->get_size(width, height);
 
-    if (response == 0)
+    if (response == Gtk::RESPONSE_ACCEPT)
       {
-        std::list<Uint32> fight_order;
+        std::list<guint32> fight_order;
         for (Gtk::TreeIter i = armies_list->children().begin(),
 	     end = armies_list->children().end(); i != end; ++i) 
           fight_order.push_back((*i)[armies_columns.army_type]);
@@ -96,14 +101,23 @@ void FightOrderDialog::run()
       }
 }
 
-void FightOrderDialog::addArmyType(Uint32 army_type)
+void FightOrderDialog::addArmyType(guint32 army_type)
 {
     GraphicsCache *gc = GraphicsCache::getInstance();
     Gtk::TreeIter i = armies_list->append();
     Armysetlist *alist = Armysetlist::getInstance();
     const ArmyProto *a = alist->getArmy(player->getArmyset(), army_type);
     (*i)[armies_columns.name] = a->getName();
-    (*i)[armies_columns.image] = to_pixbuf(gc->getArmyPic(player->getArmyset(),
-                                           army_type, player, NULL));
+    (*i)[armies_columns.image] = gc->getArmyPic(player->getArmyset(),
+                                           army_type, player, NULL)->to_pixbuf();
     (*i)[armies_columns.army_type] = a->getTypeId();
 }
+
+void FightOrderDialog::on_reverse_button_clicked()
+{
+  std::list<int> new_order;
+  Gtk::TreeModel::Children kids = armies_list->children();
+  for (unsigned int i = 0; i < kids.size(); i++)
+    new_order.push_back(kids.size() - i - 1);
+  armies_list->reorder(new_order);
+}
