@@ -1,9 +1,9 @@
-//  Copyright (C) 2007, 2008, Ole Laursen
-//  Copyright (C) 2007, 2008 Ben Asselstine
+//  Copyright (C) 2007, 2008 Ole Laursen
+//  Copyright (C) 2007, 2008, 2009 Ben Asselstine
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
-//  the Free Software Foundation; either version 2 of the License, or
+//  the Free Software Foundation; either version 3 of the License, or
 //  (at your option) any later version.
 //
 //  This program is distributed in the hope that it will be useful,
@@ -23,10 +23,7 @@
 #include <algorithm>
 #include <numeric>
 #include <vector>
-#include <libglademm/xml.h>
-#include <gtkmm/box.h>
-#include <gtkmm/image.h>
-#include <gtkmm/alignment.h>
+#include <gtkmm.h>
 
 #include "fight-window.h"
 
@@ -42,15 +39,14 @@
 #include "army.h"
 #include "GraphicsCache.h"
 #include "Configuration.h"
+#include "sound.h"
 
 FightWindow::FightWindow(Fight &fight)
 {
-    Glib::RefPtr<Gnome::Glade::Xml> xml
-	= Gnome::Glade::Xml::create(get_glade_path() + "/fight-window.glade");
+    Glib::RefPtr<Gtk::Builder> xml
+	= Gtk::Builder::create_from_file(get_glade_path() + "/fight-window.ui");
 
-    Gtk::Window *d = 0;
-    xml->get_widget("window", d);
-    window.reset(d);
+    xml->get_widget("window", window);
     
     window->signal_key_release_event().connect_notify(sigc::mem_fun(*this, &FightWindow::on_key_release_event));
 
@@ -70,7 +66,7 @@ FightWindow::FightWindow(Fight &fight)
     // add the armies
     std::vector<Gtk::HBox *> close_hboxes;
     int close;
-    std::map<Uint32, Uint32> initial_hps = fight.getInitialHPs();
+    std::map<guint32, guint32> initial_hps = fight.getInitialHPs();
 
     // ... attackers
     close = 0;
@@ -99,12 +95,12 @@ FightWindow::FightWindow(Fight &fight)
     Gtk::Image *defender_shield_image;
     p = defenders.front()->getOwner();
     xml->get_widget("defender_shield_image", defender_shield_image);
-    defender_shield_image->property_pixbuf()=to_pixbuf(gc->getShieldPic(2, p));
+    defender_shield_image->property_pixbuf()=gc->getShieldPic(2, p)->to_pixbuf();
 
     Gtk::Image *attacker_shield_image;
     p = attackers.front()->getOwner();
     xml->get_widget("attacker_shield_image", attacker_shield_image);
-    attacker_shield_image->property_pixbuf()=to_pixbuf(gc->getShieldPic(2, p));
+    attacker_shield_image->property_pixbuf()=gc->getShieldPic(2, p)->to_pixbuf();
   
     actions = fight.getCourseOfEvents();
     d_quick = false;
@@ -113,10 +109,15 @@ FightWindow::FightWindow(Fight &fight)
       Configuration::s_displayFightRoundDelayFast; //milliseconds
     normal_round_speed = 
       Configuration::s_displayFightRoundDelaySlow; //milliseconds
+    Sound::getInstance()->disableBackground(true);
+    Sound::getInstance()->playMusic("battle", -1, true);
 }
 
 FightWindow::~FightWindow()
 {
+  Sound::getInstance()->haltMusic(true);
+  Sound::getInstance()->enableBackground();
+  delete window;
 }
 
 void FightWindow::set_parent_window(Gtk::Window &parent)
@@ -238,14 +239,15 @@ void FightWindow::add_army(Army *army, int initial_hp,
     Gtk::VBox *army_box = manage(new Gtk::VBox);
 	
     // image
-    SDL_Surface *pic = gc->getArmyPic(army);
-    Gtk::Image *image = new Gtk::Image(to_pixbuf(pic));
+    Glib::RefPtr<Gdk::Pixbuf> pic = gc->getArmyPic(army)->to_pixbuf();
+    Gtk::Image *image = new Gtk::Image();
+    image->property_pixbuf() = pic;
     army_box->add(*manage(image));
     
     // hit points graph
     Gtk::ProgressBar *progress = manage(new Gtk::ProgressBar);
     progress->set_fraction(double(initial_hp) / army->getStat(Army::HP));
-    progress->property_width_request() = pic->w;
+    progress->property_width_request() = pic->get_width();
     progress->property_height_request() = 12;
     army_box->pack_start(*progress, Gtk::PACK_SHRINK, 4);
 
@@ -280,7 +282,7 @@ void FightWindow::add_army(Army *army, int initial_hp,
 bool FightWindow::do_round()
 {
   GraphicsCache *gc = GraphicsCache::getInstance();
-  SDL_Surface *expl = gc->getExplosionPic();
+  Glib::RefPtr<Gdk::Pixbuf> expl = gc->getExplosionPic()->to_pixbuf();
 
   // first we clear out any explosions
   for (army_items_type::iterator i = army_items.begin(),
@@ -290,7 +292,7 @@ bool FightWindow::do_round()
       continue;
     
     Glib::RefPtr<Gdk::Pixbuf> empty_pic
-      = Gdk::Pixbuf::create(Gdk::COLORSPACE_RGB, true, 8, expl->w, expl->h);
+      = Gdk::Pixbuf::create(Gdk::COLORSPACE_RGB, true, 8, expl->get_width(), expl->get_height());
     empty_pic->fill(0x00000000);
     i->image->property_pixbuf() = empty_pic;
     i->exploding = false;
@@ -315,7 +317,7 @@ bool FightWindow::do_round()
         if (fraction == 0.0)
         {
           i->bar->hide();
-          i->image->property_pixbuf() = to_pixbuf(expl);
+          i->image->property_pixbuf() = expl;
           i->exploding = true;
         }
 		
