@@ -1,9 +1,9 @@
 //  Copyright (C) 2007, 2008, Ole Laursen
-//  Copyright (C) 2007, 2008 Ben Asselstine
+//  Copyright (C) 2007, 2008, 2009 Ben Asselstine
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
-//  the Free Software Foundation; either version 2 of the License, or
+//  the Free Software Foundation; either version 3 of the License, or
 //  (at your option) any later version.
 //
 //  This program is distributed in the hope that it will be useful,
@@ -24,9 +24,7 @@
 #include <list>
 #include <sigc++/trackable.h>
 #include <glibmm/main.h>
-#include <gtkmm/window.h>
-#include <gtkmm/progressbar.h>
-#include <gtkmm/alignment.h>
+#include <gtkmm.h>
 
 #include "game-parameters.h"
 #include "fight.h"
@@ -47,7 +45,7 @@ class FightWindow: public sigc::trackable
     void run(bool *quick);
     
  private:
-    std::auto_ptr<Gtk::Window> window;
+    Gtk::Window* window;
     static const int max_cols = 8;
 
     struct ArmyItem
