@@ -1,8 +1,8 @@
-//  Copyright (C) 2008 Ben Asselstine
+//  Copyright (C) 2008, 2009 Ben Asselstine
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
-//  the Free Software Foundation; either version 2 of the License, or
+//  the Free Software Foundation; either version 3 of the License, or
 //  (at your option) any later version.
 //
 //  This program is distributed in the hope that it will be useful,
@@ -21,27 +21,12 @@
 #include <string>
 #include <memory>
 #include <vector>
-#include <gtkmm/dialog.h>
-#include <gtkmm/image.h>
-#include <gtkmm/box.h>
-#include <gtkmm/button.h>
-#include <gtkmm/entry.h>
-#include <gtkmm/label.h>
-#include <sigc++/signal.h>
-#include <gtkmm/combobox.h>
-#include <gtkmm/liststore.h>
-#include <gtkmm/treemodelcolumn.h>
-#include <gtkmm/scrolledwindow.h>
-#include <gtkmm/textview.h>
-#include <gtkmm/cellrenderercombo.h>
-#include <gtkmm/cellrenderertoggle.h>
-#include <gtkmm/treeview.h>
+#include <gtkmm.h>
 #include "citymap.h"
 #include "GameScenario.h"
 #include "game-station.h"
 
 #include "decorated.h"
-struct SDL_Surface;
 class NextTurnNetworked;
 
 class Player;
@@ -73,14 +58,14 @@ class GameLobbyDialog: public Decorated
   sigc::signal<void, GameScenario *, NextTurnNetworked*> start_network_game;
 
  private:
-    std::auto_ptr<Gtk::Dialog> dialog;
+    Gtk::Dialog* dialog;
     //! The mini map that shows the scenario map
-    std::auto_ptr<CityMap> citymap;
+    CityMap* citymap;
     Gtk::Image *map_image;
 
     void initDialog(GameScenario *gamescenario, NextTurnNetworked *next_turn,
 		    GameStation *game_station);
-    void on_map_changed(SDL_Surface *map);
+    void on_map_changed(Glib::RefPtr< Gdk::Pixmap> map);
     GameScenario *d_game_scenario;
     GameStation *d_game_station;
     NextTurnNetworked *d_next_turn;
@@ -89,6 +74,7 @@ class GameLobbyDialog: public Decorated
     Gtk::Label *cities_label;
     Gtk::Button *play_button;
     Gtk::Button *cancel_button;
+    Gtk::Button *show_options_button;
     Gtk::ScrolledWindow *chat_scrolledwindow;
     Gtk::TextView *chat_textview;
     Gtk::Entry *chat_entry;
@@ -109,7 +95,7 @@ class GameLobbyDialog: public Decorated
 	Gtk::TreeModelColumn<bool> sitting;
 	Gtk::TreeModelColumn<Glib::ustring> person, name, type;
 	Gtk::TreeModelColumn<Glib::RefPtr<Gdk::Pixbuf> > turn;
-	Gtk::TreeModelColumn<Uint32> player_id;
+	Gtk::TreeModelColumn<guint32> player_id;
     };
     const PlayerColumns player_columns;
     Glib::RefPtr<Gtk::ListStore> player_list;
