@@ -1,9 +1,9 @@
-//  Copyright (C) 2007, Ole Laursen
-//  Copyright (C) 2007, 2008 Ben Asselstine
+//  Copyright (C) 2007 Ole Laursen
+//  Copyright (C) 2007, 2008, 2009 Ben Asselstine
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
-//  the Free Software Foundation; either version 2 of the License, or
+//  the Free Software Foundation; either version 3 of the License, or
 //  (at your option) any later version.
 //
 //  This program is distributed in the hope that it will be useful,
@@ -22,10 +22,7 @@
 #include <memory>
 #include <vector>
 #include <sigc++/trackable.h>
-#include <gtkmm/dialog.h>
-#include <gtkmm/image.h>
-#include <gtkmm/radiobutton.h>
-#include <gtkmm/entry.h>
+#include <gtkmm.h>
 
 #include "heromap.h"
 #include "player.h"
@@ -33,13 +30,13 @@
 #include "heroproto.h"
 
 #include "decorated.h"
-struct SDL_Surface;
 
 //! dialog for accepting/rejecting a hero
 class HeroOfferDialog: public Decorated
 {
  public:
     HeroOfferDialog(Player *player, HeroProto *hero, City *city, int gold);
+    ~HeroOfferDialog();
 
     void set_parent_window(Gtk::Window &parent);
 
@@ -47,9 +44,9 @@ class HeroOfferDialog: public Decorated
     void hide();
     
  private:
-    std::auto_ptr<Gtk::Dialog> dialog;
+    Gtk::Dialog* dialog;
     //! The smallmap that shows where the Hero is emerging.
-    std::auto_ptr<HeroMap> heromap;
+    HeroMap* heromap;
 
     Gtk::Image *map_image;
     Gtk::Image *hero_image;
@@ -60,7 +57,7 @@ class HeroOfferDialog: public Decorated
     City *city;
 
     void on_male_toggled();
-    void on_map_changed(SDL_Surface *map);
+    void on_map_changed(Glib::RefPtr<Gdk::Pixmap> map);
 };
 
 #endif
