@@ -1,8 +1,9 @@
-//  Copyright (C) 2007, Ole Laursen
+//  Copyright (C) 2007 Ole Laursen
+//  Copyright (C) 2009 Ben Asselstine
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
-//  the Free Software Foundation; either version 2 of the License, or
+//  the Free Software Foundation; either version 3 of the License, or
 //  (at your option) any later version.
 //
 //  This program is distributed in the hope that it will be useful,
@@ -20,15 +21,19 @@
 
 #include <vector>
 #include <gdkmm/pixbuf.h>
-#include <SDL_video.h>
-
-// convert surface to pixbuf - note that this will make a full copy of the
-// surface pixels
-Glib::RefPtr<Gdk::Pixbuf> to_pixbuf(SDL_Surface *surface);
+#include "PixMask.h"
 
 // convert a file containing one large image with subimages, each of the same
 // width, to an array of pixbufs corresponding to the subimages
-std::vector<Glib::RefPtr<Gdk::Pixbuf> >
+std::vector<PixMask*>
 disassemble_row(const std::string &file, int no);
+std::vector<PixMask*>
+disassemble_row(const std::string &file, int no, bool first_half_height);
 
+Glib::RefPtr<Gdk::Pixmap> to_pixmap(Glib::RefPtr<Gdk::Pixbuf> pixbuf);
+
+int get_pwidth(Glib::RefPtr<Gdk::Pixmap> pixmap);
+int get_pheight(Glib::RefPtr<Gdk::Pixmap> pixmap);
+Glib::RefPtr<Gdk::Pixbuf> to_pixbuf(Glib::RefPtr<Gdk::Pixmap> pixmap);
+Glib::RefPtr<Gdk::Pixmap> scale (Glib::RefPtr<Gdk::Pixmap> pixmap, int w, int h);
 #endif
