@@ -1,8 +1,8 @@
-//  Copyright (C) 2008 Ben Asselstine
+//  Copyright (C) 2008, 2009 Ben Asselstine
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
-//  the Free Software Foundation; either version 2 of the License, or
+//  the Free Software Foundation; either version 3 of the License, or
 //  (at your option) any later version.
 //
 //  This program is distributed in the hope that it will be useful,
@@ -17,10 +17,8 @@
 
 #include <config.h>
 
-#include <libglademm/xml.h>
-#include <gtkmm/eventbox.h>
+#include <gtkmm.h>
 #include <sigc++/functors/mem_fun.h>
-#include <gtkmm/label.h>
 
 #include "network-game-selector-dialog.h"
 
@@ -33,15 +31,13 @@
 
 NetworkGameSelectorDialog::NetworkGameSelectorDialog()
 {
-    Glib::RefPtr<Gnome::Glade::Xml> xml
-	= Gnome::Glade::Xml::create
-	(get_glade_path() + "/pick-network-game-to-join-dialog.glade");
+    Glib::RefPtr<Gtk::Builder> xml
+	= Gtk::Builder::create_from_file
+	(get_glade_path() + "/pick-network-game-to-join-dialog.ui");
 
-    Gtk::Dialog *d = 0;
-    xml->get_widget("dialog", d);
-    dialog.reset(d);
-    decorate(dialog.get());
-    window_closed.connect(sigc::mem_fun(dialog.get(), &Gtk::Dialog::hide));
+    xml->get_widget("dialog", dialog);
+    decorate(dialog);
+    window_closed.connect(sigc::mem_fun(dialog, &Gtk::Dialog::hide));
 
     xml->get_widget("hostname_entry", hostname_entry);
     xml->get_widget("port_spinbutton", port_spinbutton);
@@ -82,6 +78,11 @@ NetworkGameSelectorDialog::NetworkGameSelectorDialog()
     port_spinbutton->set_value(LORDSAWAR_PORT);
 }
 	    
+NetworkGameSelectorDialog::~NetworkGameSelectorDialog()
+{
+  delete dialog;
+}
+
 void NetworkGameSelectorDialog::addRecentlyJoinedGame(RecentlyPlayedNetworkedGame*recent)
 {
     Gtk::TreeIter i = recently_joined_games_list->append();
@@ -117,7 +118,7 @@ void NetworkGameSelectorDialog::hide()
 bool NetworkGameSelectorDialog::run()
 {
   int response = dialog->run();
-  if (response == 0)
+  if (response == Gtk::RESPONSE_ACCEPT)
     {
       hide();
       game_selected.emit(hostname_entry->get_text(), LORDSAWAR_PORT);
