@@ -1,8 +1,8 @@
-//  Copyright (C) 2007, 2008 Ben Asselstine
+//  Copyright (C) 2007, 2008, 2009 Ben Asselstine
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
-//  the Free Software Foundation; either version 2 of the License, or
+//  the Free Software Foundation; either version 3 of the License, or
 //  (at your option) any later version.
 //
 //  This program is distributed in the hope that it will be useful,
@@ -17,9 +17,8 @@
 
 #include <config.h>
 
-#include <libglademm/xml.h>
+#include <gtkmm.h>
 #include <sigc++/functors/mem_fun.h>
-#include <gtkmm/label.h>
 
 #include "report-dialog.h"
 
@@ -33,7 +32,7 @@
 #include "GameMap.h"
 #include "playerlist.h"
 #include "citylist.h"
-#include "stacklist.h"
+#include "city.h"
 #include "action.h"
 #include "GraphicsCache.h"
 #include "armyprodbase.h"
@@ -42,24 +41,22 @@
 ReportDialog::ReportDialog(Player *player, ReportType type)
 {
   d_player = player;
-  Glib::RefPtr<Gnome::Glade::Xml> xml
-    = Gnome::Glade::Xml::create(get_glade_path() + "/report-dialog.glade");
+  Glib::RefPtr<Gtk::Builder> xml
+    = Gtk::Builder::create_from_file(get_glade_path() + "/report-dialog.ui");
 
-  Gtk::Dialog *d = 0;
-  xml->get_widget("dialog", d);
-  dialog.reset(d);
-  decorate(dialog.get());
-  window_closed.connect(sigc::mem_fun(dialog.get(), &Gtk::Dialog::hide));
+  xml->get_widget("dialog", dialog);
+  decorate(dialog);
+  window_closed.connect(sigc::mem_fun(dialog, &Gtk::Dialog::hide));
 
   xml->get_widget("map_image", map_image);
-  citymap.reset(new CityMap());
+  citymap = new CityMap();
   citymap->map_changed.connect
     (sigc::mem_fun(this, &ReportDialog::on_city_map_changed));
-  armymap.reset(new ArmyMap());
+  armymap = new ArmyMap();
   armymap->map_changed.connect
     (sigc::mem_fun(this, &ReportDialog::on_army_map_changed));
   City *c = Citylist::getInstance()->getFirstCity(d_player);
-  vectormap.reset(new VectorMap(c, VectorMap::SHOW_ALL_VECTORING, false));
+  vectormap = new VectorMap(c, VectorMap::SHOW_ALL_VECTORING, false);
   vectormap->map_changed.connect
     (sigc::mem_fun(this, &ReportDialog::on_vector_map_changed));
 
@@ -83,7 +80,7 @@ ReportDialog::ReportDialog(Player *player, ReportType type)
 
   //loop through the action list looking for production actions
   std::list<Action*> actions = player->getReportableActions();
-  Uint32 total = 0;
+  guint32 total = 0;
   std::list<Action*>::const_iterator it;
   for (it = actions.begin(); it != actions.end(); it++)
     {
@@ -92,6 +89,8 @@ ReportDialog::ReportDialog(Player *player, ReportType type)
 	total++;
     addProduction(*it);
     }
+    armies_treeview->get_selection()->signal_changed().connect
+      (sigc::mem_fun(*this, &ReportDialog::on_army_selected));
 
   Glib::ustring s;
   s = String::ucompose(ngettext("You produced %1 army this turn!",
@@ -115,6 +114,13 @@ ReportDialog::ReportDialog(Player *player, ReportType type)
   closing = false;
 }
 
+ReportDialog::~ReportDialog()
+{
+    delete dialog;
+    delete vectormap;
+    delete armymap;
+    delete citymap;
+}
 void ReportDialog::on_close_button()
 {
   closing = true;
@@ -146,24 +152,24 @@ void ReportDialog::run()
   dialog->run();
 }
 
-void ReportDialog::on_army_map_changed(SDL_Surface *map)
+void ReportDialog::on_army_map_changed(Glib::RefPtr<Gdk::Pixmap> map)
 {
   if (report_notebook->get_current_page() == ARMY)
-    map_image->property_pixbuf() = to_pixbuf(map);
+    map_image->property_pixmap() = map;
 }
 
-void ReportDialog::on_city_map_changed(SDL_Surface *map)
+void ReportDialog::on_city_map_changed(Glib::RefPtr<Gdk::Pixmap> map)
 {
   if (report_notebook->get_current_page() == CITY ||
       report_notebook->get_current_page() == GOLD ||
       report_notebook->get_current_page() == WINNING)
-    map_image->property_pixbuf() = to_pixbuf(map);
+    map_image->property_pixmap() = map;
 }
 
-void ReportDialog::on_vector_map_changed(SDL_Surface *map)
+void ReportDialog::on_vector_map_changed(Glib::RefPtr<Gdk::Pixmap> map)
 {
   if (report_notebook->get_current_page() == PRODUCTION)
-    map_image->property_pixbuf() = to_pixbuf(map);
+    map_image->property_pixmap() = map;
 }
 
 void ReportDialog::on_switch_page(GtkNotebookPage *page, guint number)
@@ -173,15 +179,15 @@ void ReportDialog::on_switch_page(GtkNotebookPage *page, guint number)
   switch (number)
     {
     case ARMY:
-      map_image->property_pixbuf() = to_pixbuf(armymap->get_surface());
+      map_image->property_pixmap() = armymap->get_surface();
       break;
     case CITY: 
     case GOLD: 
     case WINNING:
-      map_image->property_pixbuf() = to_pixbuf(citymap->get_surface());
+      map_image->property_pixmap() = citymap->get_surface();
       break;
     case PRODUCTION:
-      map_image->property_pixbuf() = to_pixbuf(vectormap->get_surface());
+      map_image->property_pixmap() = vectormap->get_surface();
       break;
     }
   fill_in_info();
@@ -211,11 +217,11 @@ void ReportDialog::fill_in_info()
 
 void ReportDialog::updateArmyChart()
 {
-  std::list<Uint32> bars;
+  std::list<guint32> bars;
   std::list<Gdk::Color> colours;
   Gdk::Color colour;
   Glib::ustring s;
-  Uint32 total;
+  guint32 total;
   for (unsigned int i = 0; i < MAX_PLAYERS; i++)
     {
       Player *p = Playerlist::getInstance()->getPlayer(i);
@@ -224,10 +230,9 @@ void ReportDialog::updateArmyChart()
       if (p == Playerlist::getInstance()->getNeutral())
 	continue;
       total = 0;
-      total = p->getStacklist()->countArmies();
+      total = p->countArmies();
       bars.push_back(total);
-      SDL_Color sdl = p->getColor();
-      colour.set_red(sdl.r * 255); colour.set_green(sdl.g * 255); colour.set_blue(sdl.b * 255);
+      colour = p->getColor();
       colours.push_back(colour);
       if (p == d_player)
 	{
@@ -244,11 +249,11 @@ void ReportDialog::updateArmyChart()
 
 void ReportDialog::updateCityChart()
 {
-  std::list<Uint32> bars;
+  std::list<guint32> bars;
   std::list<Gdk::Color> colours;
   Gdk::Color colour;
   Glib::ustring s;
-  Uint32 total;
+  guint32 total;
   for (unsigned int i = 0; i < MAX_PLAYERS; i++)
     {
       Player *p = Playerlist::getInstance()->getPlayer(i);
@@ -259,8 +264,7 @@ void ReportDialog::updateCityChart()
       total = Citylist::getInstance()->countCities(p);
 
       bars.push_back(total);
-      SDL_Color sdl = p->getColor();
-      colour.set_red(sdl.r * 255); colour.set_green(sdl.g * 255); colour.set_blue(sdl.b * 255);
+      colour = p->getColor();
       colours.push_back(colour);
       if (p == d_player)
 	{
@@ -277,11 +281,11 @@ void ReportDialog::updateCityChart()
 
 void ReportDialog::updateGoldChart()
 {
-  std::list<Uint32> bars;
+  std::list<guint32> bars;
   std::list<Gdk::Color> colours;
   Gdk::Color colour;
   Glib::ustring s;
-  Uint32 total;
+  guint32 total;
   bars.clear();
   for (unsigned int i = 0; i < MAX_PLAYERS; i++)
     {
@@ -292,8 +296,7 @@ void ReportDialog::updateGoldChart()
 	continue;
       total = p->getGold();
       bars.push_back(total);
-      SDL_Color sdl = p->getColor();
-      colour.set_red(sdl.r * 255); colour.set_green(sdl.g * 255); colour.set_blue(sdl.b * 255);
+      colour = p->getColor();
       colours.push_back(colour);
       if (p == d_player)
 	{
@@ -307,15 +310,15 @@ void ReportDialog::updateGoldChart()
   gold_alignment->add(*manage(gold_chart));
 }
 
-std::string ReportDialog::calculateRank(std::list<Uint32> scores, Uint32 score)
+std::string ReportDialog::calculateRank(std::list<guint32> scores, guint32 score)
 {
   char* rank_strings[MAX_PLAYERS] =
     {
       _("first"), _("second"), _("third"), _("fourth"), _("fifth"),
       _("sixth"), _("seventh"), _("eighth"),
     };
-  Uint32 rank = 0;
-  std::list<Uint32>::iterator it = scores.begin();
+  guint32 rank = 0;
+  std::list<guint32>::iterator it = scores.begin();
   for (; it != scores.end(); it++)
     {
       if (score < *it)
@@ -327,11 +330,11 @@ std::string ReportDialog::calculateRank(std::list<Uint32> scores, Uint32 score)
 
 void ReportDialog::updateWinningChart()
 {
-  std::list<Uint32> bars;
+  std::list<guint32> bars;
   std::list<Gdk::Color> colours;
   Gdk::Color colour;
   Glib::ustring s;
-  Uint32 score;
+  guint32 score;
   for (unsigned int i = 0; i < MAX_PLAYERS; i++)
     {
       Player *p = Playerlist::getInstance()->getPlayer(i);
@@ -341,8 +344,7 @@ void ReportDialog::updateWinningChart()
 	continue;
       score = p->getScore();
       bars.push_back(score);
-      SDL_Color sdl = p->getColor();
-      colour.set_red(sdl.r * 255); colour.set_green(sdl.g * 255); colour.set_blue(sdl.b * 255);
+      colour = p->getColor();
       colours.push_back(colour);
     }
   s = String::ucompose(_("You are coming %1"), calculateRank(bars, d_player->getScore()));
@@ -357,6 +359,7 @@ void ReportDialog::addProduction(const Action *action)
   Player *p = d_player;
 
   int army_type = 0;
+  guint32 city_id = 0;
 
   Glib::ustring s = "";
   if (action->getType() == Action::PRODUCE_UNIT)
@@ -373,6 +376,7 @@ void ReportDialog::addProduction(const Action *action)
 	  }
       if (act->getVectored())
 	s += "...";
+      city_id = act->getCityId();
     }
   else if (action->getType() == Action::PRODUCE_VECTORED_UNIT)
     {
@@ -380,12 +384,13 @@ void ReportDialog::addProduction(const Action *action)
       act = dynamic_cast<const Action_ProduceVectored*>(action);
       army_type = act->getArmy()->getTypeId();
       Vector<int> pos = act->getDestination();
-      City *c = Citylist::getInstance()->getObjectAt(pos);
+      City *c = GameMap::getCity(pos);
       s+="...";
       if (c)
 	s += c->getName();
       else
 	s += _("Standard");
+      city_id = GameMap::getCity(act->getOrigination())->getId();
     }
   else if (action->getType() == Action::CITY_DESTITUTE)
     {
@@ -395,12 +400,26 @@ void ReportDialog::addProduction(const Action *action)
       City *c = Citylist::getInstance()->getById(act->getCityId());
       s = c->getName();
       s += " stops production!";
+      city_id = act->getCityId();
     }
   const ArmyProto *a;
   a = Armysetlist::getInstance()->getArmy(p->getArmyset(), army_type);
   Gtk::TreeIter i = armies_list->append();
-  (*i)[armies_columns.image] = to_pixbuf(gc->getArmyPic(p->getArmyset(),
-							army_type,
-							p, NULL));
+  (*i)[armies_columns.city_id] = city_id;
+  (*i)[armies_columns.image] = gc->getArmyPic(p->getArmyset(), army_type, p, 
+					      NULL)->to_pixbuf();
   (*i)[armies_columns.desc] = s;
 }
+
+void ReportDialog::on_army_selected()
+{
+    Gtk::TreeIter i = armies_treeview->get_selection()->get_selected();
+    if (i)
+    {
+	City *c = 
+	  Citylist::getInstance()->getById((*i)[armies_columns.city_id]);
+	if (c)
+	  vectormap->setCity(c);
+    }
+
+}
