@@ -1,8 +1,8 @@
-//  Copyright (C) 2007, 2008 Ben Asselstine
+//  Copyright (C) 2007, 2008, 2009 Ben Asselstine
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
-//  the Free Software Foundation; either version 2 of the License, or
+//  the Free Software Foundation; either version 3 of the License, or
 //  (at your option) any later version.
 //
 //  This program is distributed in the hope that it will be useful,
@@ -21,15 +21,7 @@
 #include <memory>
 #include <vector>
 #include <sigc++/trackable.h>
-#include <gtkmm/dialog.h>
-#include <gtkmm/image.h>
-#include <gtkmm/label.h>
-#include <gtkmm/notebook.h>
-#include <gtkmm/button.h>
-#include <gtkmm/liststore.h>
-#include <gtkmm/treemodelcolumn.h>
-#include <gtkmm/treeview.h>
-#include <gtkmm/alignment.h>
+#include <gtkmm.h>
 
 #include "LocationList.h"
 #include "vectormap.h"
@@ -39,7 +31,6 @@
 
 class Action;
 class BarChart;
-struct SDL_Surface;
 
 #include "decorated.h"
 
@@ -51,18 +42,19 @@ class ReportDialog: public Decorated
     enum ReportType {ARMY = 0, CITY, GOLD, PRODUCTION, WINNING};
 
     ReportDialog(Player *player, ReportType type);
+    ~ReportDialog();
 
     void set_parent_window(Gtk::Window &parent);
 
-    static std::string calculateRank(std::list<Uint32> scores, Uint32 score);
+    static std::string calculateRank(std::list<guint32> scores, guint32 score);
     void run();
     void hide();
     
  private:
-    std::auto_ptr<Gtk::Dialog> dialog;
-    std::auto_ptr<VectorMap> vectormap;
-    std::auto_ptr<ArmyMap> armymap;
-    std::auto_ptr<CityMap> citymap;
+    Gtk::Dialog* dialog;
+    VectorMap* vectormap;
+    ArmyMap* armymap;
+    CityMap* citymap;
 
     Gtk::Image *map_image;
     
@@ -82,9 +74,9 @@ class ReportDialog: public Decorated
     BarChart *winning_chart;
 
     Player *d_player;
-    void on_army_map_changed(SDL_Surface *map);
-    void on_city_map_changed(SDL_Surface *map);
-    void on_vector_map_changed(SDL_Surface *map);
+    void on_army_map_changed(Glib::RefPtr<Gdk::Pixmap> map);
+    void on_city_map_changed(Glib::RefPtr<Gdk::Pixmap> map);
+    void on_vector_map_changed(Glib::RefPtr<Gdk::Pixmap> map);
     void on_switch_page(GtkNotebookPage *page, guint number);
 
     Gtk::TreeView *armies_treeview;
@@ -92,8 +84,9 @@ class ReportDialog: public Decorated
     class ArmiesColumns: public Gtk::TreeModelColumnRecord {
     public:
 	ArmiesColumns() 
-        { add(image); add(desc);}
+        { add(city_id) ;add(image); add(desc);}
 	
+	Gtk::TreeModelColumn<guint32> city_id;
 	Gtk::TreeModelColumn<Glib::RefPtr<Gdk::Pixbuf> > image;
 	Gtk::TreeModelColumn<Glib::ustring> desc;
     };
@@ -107,6 +100,7 @@ class ReportDialog: public Decorated
     void updateCityChart();
     void updateGoldChart();
     void updateWinningChart();
+    void on_army_selected();
 };
 
 #endif
