@@ -44,11 +44,10 @@ SurrenderDialog::SurrenderDialog(int numEnemies)
     xml->get_widget("image", image);
     
     Glib::ustring s;
-    s = String::ucompose
-      (ngettext("Your final opponent comes on bended knee and offers surrender!",
-		"%1 opponents come on bended knee and offer surrender!",
-		numEnemies), numEnemies);
-    s += _("\nDo you accept?");
+    s = ngettext("Your enemy grudgingly surrenders!\n",
+		 "Your enemies respectfully surrender!\n",
+		 numEnemies);
+    s += _("Do you accept?");
     label->set_text(s);
     image->property_file()
       = File::getMiscFile("various/parley_offered.png");
