@@ -1,8 +1,8 @@
-//  Copyright (C) 2007, 2008 Ben Asselstine
+//  Copyright (C) 2007, 2008, 2009 Ben Asselstine
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
-//  the Free Software Foundation; either version 2 of the License, or
+//  the Free Software Foundation; either version 3 of the License, or
 //  (at your option) any later version.
 //
 //  This program is distributed in the hope that it will be useful,
@@ -17,12 +17,8 @@
 
 #include <config.h>
 
-#include <libglademm/xml.h>
 #include <sigc++/functors/mem_fun.h>
-#include <gtkmm/label.h>
-#include <gtkmm/scrolledwindow.h>
-#include <gtkmm/notebook.h>
-#include <gtkmm/image.h>
+#include <gtkmm.h>
 
 #include "triumphs-dialog.h"
 
@@ -41,14 +37,12 @@
 TriumphsDialog::TriumphsDialog(Player *player)
 {
   d_player = player;
-  Glib::RefPtr<Gnome::Glade::Xml> xml
-    = Gnome::Glade::Xml::create(get_glade_path() + "/triumphs-dialog.glade");
+  Glib::RefPtr<Gtk::Builder> xml
+    = Gtk::Builder::create_from_file(get_glade_path() + "/triumphs-dialog.ui");
 
-  Gtk::Dialog *d = 0;
-  xml->get_widget("dialog", d);
-  dialog.reset(d);
-  decorate(dialog.get());
-  window_closed.connect(sigc::mem_fun(dialog.get(), &Gtk::Dialog::hide));
+  xml->get_widget("dialog", dialog);
+  decorate(dialog);
+  window_closed.connect(sigc::mem_fun(dialog, &Gtk::Dialog::hide));
 
   Gtk::HBox *contents;
   xml->get_widget("outer_hbox", contents);
@@ -59,6 +53,10 @@ TriumphsDialog::TriumphsDialog(Player *player)
   notebook->set_current_page(d_player->getId());
 }
 
+TriumphsDialog::~TriumphsDialog()
+{
+  delete dialog;
+}
 void TriumphsDialog::set_parent_window(Gtk::Window &parent)
 {
   dialog->set_transient_for(parent);
@@ -75,10 +73,10 @@ void TriumphsDialog::run()
   dialog->run();
 }
 
-Uint32 TriumphsDialog::tally(Player *p, Triumphs::TriumphType type)
+guint32 TriumphsDialog::tally(Player *p, Triumphs::TriumphType type)
 {
   Playerlist *pl = Playerlist::getInstance();
-  Uint32 count = 0;
+  guint32 count = 0;
   if (p == d_player)
     {
       // add up what the other players did to us
@@ -103,7 +101,7 @@ void TriumphsDialog::fill_in_page(Player *p)
   //here we tally up the stats, make a vbox and append it as a new page
   //tally it up differently when p == d_player
 	
-  Uint32 count;
+  guint32 count;
   Glib::ustring s;
   count = tally(p, Triumphs::TALLY_HERO);
   if (p == d_player)
@@ -129,8 +127,8 @@ void TriumphsDialog::fill_in_page(Player *p)
 	  break;
 	}
     }
-  Gtk::Image *hero_image = new Gtk::Image
-    (to_pixbuf(gc->getArmyPic(p->getArmyset(), hero->getTypeId(), p, NULL)));
+  Gtk::Image *hero_image = new Gtk::Image();
+  hero_image->property_pixbuf() = gc->getArmyPic(p->getArmyset(), hero->getTypeId(), p, NULL)->to_pixbuf();
   Gtk::HBox *hero_hbox = new Gtk::HBox();
   hero_hbox->pack_start(*manage(hero_image), Gtk::PACK_SHRINK, 10);
   hero_hbox->pack_start(*manage(hero_label), Gtk::PACK_SHRINK, 10);
@@ -145,7 +143,8 @@ void TriumphsDialog::fill_in_page(Player *p)
 				   "%1 navies rest with the fishes!",
 				   count), count);
   Gtk::Label *ship_label = new Gtk::Label(s);
-  Gtk::Image *ship_image = new Gtk::Image (to_pixbuf(gc->getShipPic(p)));
+  Gtk::Image *ship_image = new Gtk::Image ();
+  ship_image->property_pixbuf() = gc->getShipPic(p)->to_pixbuf();
   Gtk::HBox *ship_hbox = new Gtk::HBox();
   ship_hbox->pack_start(*manage(ship_image), Gtk::PACK_SHRINK, 10);
   ship_hbox->pack_start(*manage(ship_label), Gtk::PACK_SHRINK, 10);
@@ -160,8 +159,8 @@ void TriumphsDialog::fill_in_page(Player *p)
 				   "%1 armies smote like sheep!",
 				   count), count);
   Gtk::Label *normal_label = new Gtk::Label(s);
-  Gtk::Image *normal_image = new Gtk::Image
-    (to_pixbuf(gc->getArmyPic(p->getArmyset(), 0, p, NULL)));
+  Gtk::Image *normal_image = new Gtk::Image();
+  normal_image->property_pixbuf() = gc->getArmyPic(p->getArmyset(), 0, p, NULL)->to_pixbuf();
   Gtk::HBox *normal_hbox = new Gtk::HBox();
   normal_hbox->pack_start(*manage(normal_image), Gtk::PACK_SHRINK, 10);
   normal_hbox->pack_start(*manage(normal_label), Gtk::PACK_SHRINK, 10);
@@ -188,9 +187,9 @@ void TriumphsDialog::fill_in_page(Player *p)
 	  break;
 	}
     }
-  Gtk::Image *special_image = new Gtk::Image
-    (to_pixbuf(gc->getArmyPic(p->getArmyset(), 
-			      special->getTypeId(), p, NULL)));
+  Gtk::Image *special_image = new Gtk::Image();
+  special_image->property_pixbuf() = 
+    gc->getArmyPic(p->getArmyset(), special->getTypeId(), p, NULL)->to_pixbuf();
   Gtk::HBox *special_hbox = new Gtk::HBox();
   special_hbox->pack_start(*manage(special_image), Gtk::PACK_SHRINK, 10);
   special_hbox->pack_start(*manage(special_label), Gtk::PACK_SHRINK, 10);
@@ -206,8 +205,8 @@ void TriumphsDialog::fill_in_page(Player *p)
 			   "%1 standards wrested from a vanquished foe!",
 			   count), count);
   Gtk::Label *flag_label = new Gtk::Label(s);
-  Gtk::Image *flag_image = new Gtk::Image 
-    (to_pixbuf(gc->getPlantedStandardPic(p)));
+  Gtk::Image *flag_image = new Gtk::Image ();
+  flag_image->property_pixbuf() = gc->getPlantedStandardPic(p)->to_pixbuf();
   Gtk::HBox *flag_hbox = new Gtk::HBox();
   flag_hbox->pack_start(*manage(flag_image), Gtk::PACK_SHRINK, 10);
   flag_hbox->pack_start(*manage(flag_label), Gtk::PACK_SHRINK, 10);
@@ -218,9 +217,9 @@ void TriumphsDialog::fill_in_page(Player *p)
   contents->add(*manage(hero_hbox));
   contents->add(*manage(ship_hbox));
   contents->add(*manage(flag_hbox));
-  notebook->append_page 
-    (*manage(contents), 
-     *manage(new Gtk::Image(to_pixbuf(gc->getShieldPic(2, p)))));
+  Gtk::Image *shield_image = new Gtk::Image();
+  shield_image->property_pixbuf() = gc->getShieldPic(2, p)->to_pixbuf();
+  notebook->append_page (*manage(contents), *manage(shield_image));
 }
 
 void TriumphsDialog::fill_in_info()
