@@ -1,9 +1,9 @@
-//  Copyright (C) 2007, 2008 Ben Asselstine
+//  Copyright (C) 2007, 2008, 2009 Ben Asselstine
 //  Copyright (C) 2008 Ole Laursen
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
-//  the Free Software Foundation; either version 2 of the License, or
+//  the Free Software Foundation; either version 3 of the License, or
 //  (at your option) any later version.
 //
 //  This program is distributed in the hope that it will be useful,
@@ -21,8 +21,11 @@
 
 #include <vector>
 #include "defs.h"
+#include "hero.h"
 
+class ArmyProto;
 class HeroProto;
+class XML_Helper;
 
 //! A list of Item objects.
 /** 
@@ -41,6 +44,8 @@ class HeroTemplates
         static void deleteInstance();
 
         HeroProto *getRandomHero(int player_id);
+
+	HeroProto *getRandomHero(Hero::Gender gender, int player_id);
         
     protected:
 	//! Default constructor. The function reads in the heronames file and produces a set of hero templates to be randomly selected from.
@@ -48,10 +53,17 @@ class HeroTemplates
 	//! Destructor.
         ~HeroTemplates();
 
+	bool load(std::string tag, XML_Helper *helper);
+
     private:
         /* the contents of the heronames data file */
         std::vector<HeroProto*> d_herotemplates[MAX_PLAYERS];
 
+	//a list of male hero prototypes contained in the the army set.
+	std::vector<const ArmyProto*> d_male_heroes;
+	//a list of female hero prototypes contained in the the army set.
+	std::vector<const ArmyProto*> d_female_heroes;
+
         static HeroTemplates* d_instance;
 
         int loadHeroTemplates();
