@@ -32,6 +32,10 @@
 #include "File.h"
 #include "GraphicsCache.h"
 #include "recently-played-game-list.h"
+#include "cityset.h"
+#include "tileset.h"
+#include "shieldset.h"
+#include "armyset.h"
 
 #include "gui/main.h"
 
@@ -107,11 +111,6 @@ int main(int argc, char* argv[])
 	    {
 	      kit.start_robots = -1;
 	    }
-	  else if (parameter == "--record" || parameter == "-R")
-	    {
-	      i++;
-	      kit.record = argv[i-1];
-	    }
 	  else if (parameter == "--help" || parameter == "-h")
 	    {
 	      cout << argv[0] << " [OPTION]... [FILE]" << endl << endl;
@@ -123,7 +122,6 @@ int main(int argc, char* argv[])
 	      cout << "  -S, --seed <number>        " << _("Seed the random number generator with NUMBER") << endl;
 	      cout << "  -s, --stress-test          " << _("Non-interactive stress test") << endl;
 	      cout << "  -r, --robots               " << _("Non-interactive network stress test") << endl;
-	      cout << "  -R, --record FILE          " << _("Record gameplay to FILE") << endl;
 	      cout << endl;
 	      cout << _("FILE can be a saved game file (.sav), or a map (.map) file.") << endl;
 	      cout << endl;
@@ -161,10 +159,10 @@ int main(int argc, char* argv[])
 
 
   // Check if armysets are in the path (otherwise exit)
-  File::scanArmysets();
-  File::scanTilesets();
-  File::scanShieldsets();
-  File::scanCitysets();
+  Armyset::scanSystemCollection();
+  Tileset::scanSystemCollection();
+  Shieldset::scanSystemCollection();
+  Cityset::scanSystemCollection();
 
 
   kit.start_main_loop();
