@@ -21,6 +21,9 @@
 #include <cstring>
 #include <cstdlib>
 #include <gnet.h>
+#include <stdio.h>
+#include <sys/types.h>
+#include <sys/stat.h>
 
 #include "network-common.h"
 
@@ -62,6 +65,33 @@ void NetworkConnection::connectToHost(std::string host, int port)
   gnet_conn_timeout(conn, 30 * 1000);
 }
 
+void NetworkConnection::sendFile(MessageType type, const std::string filename)
+{
+  if (gnet_conn_is_connected(conn) == false)
+    return;
+  FILE *fileptr = fopen (filename.c_str(), "r");
+  if (fileptr == NULL)
+    return;
+
+  struct stat statbuf;
+  stat (filename.c_str(), &statbuf);
+  // write the preamble
+  gchar buf[MESSAGE_SIZE_BYTES + MESSAGE_PREAMBLE_EXTRA_BYTES];
+  guint32 l = g_htonl(MESSAGE_PREAMBLE_EXTRA_BYTES + statbuf.st_size);
+  memcpy(buf, &l, MESSAGE_SIZE_BYTES);
+  buf[MESSAGE_SIZE_BYTES] = MESSAGE_PROTOCOL_VERSION;
+  buf[MESSAGE_SIZE_BYTES + 1] = type;
+  
+  gnet_conn_write(conn, buf, MESSAGE_SIZE_BYTES + MESSAGE_PREAMBLE_EXTRA_BYTES);
+
+  std::cerr << "sending file " << type <<" of length " << MESSAGE_PREAMBLE_EXTRA_BYTES + statbuf.st_size << " to " << gnet_inetaddr_get_name(conn->inetaddr) << std::endl;
+  char *buffer = (char*) malloc (statbuf.st_size);
+  size_t bytesread = fread (buffer, 1, statbuf.st_size, fileptr);
+  fclose (fileptr);
+  gnet_conn_write(conn, const_cast<gchar *>(buffer), bytesread);
+  free (buffer);
+}
+
 void NetworkConnection::send(MessageType type, const std::string &payload)
 {
   if (gnet_conn_is_connected(conn) == false)
