@@ -114,10 +114,39 @@ void NetworkPlayer::invadeCity(City* c)
 {
   assert(false);
 }
-
-void NetworkPlayer::levelArmy(Army* a)
+bool NetworkPlayer::chooseHero(HeroProto *hero, City *city, int gold)
 {
   assert(false);
+  return true;
+}
+
+Reward *NetworkPlayer::chooseReward(Ruin *ruin, Sage *sage, Stack *stack)
+{
+  assert(false);
+  return NULL;
+}
+
+void NetworkPlayer::heroGainsLevel(Hero * a)
+{
+  assert(false);
+}
+
+bool NetworkPlayer::chooseTreachery (Stack *stack, Player *player, Vector <int> pos)
+{
+  assert(false);
+  return true;
+}
+
+Army::Stat NetworkPlayer::chooseStat(Hero *hero)
+{
+  assert(false);
+  return Army::STRENGTH;
+}
+
+bool NetworkPlayer::chooseQuest(Hero *hero)
+{
+  assert(false);
+  return true;
 }
 
 void NetworkPlayer::decodeActions(std::list<Action *> actions)
@@ -243,15 +272,6 @@ Stack *findStackById(guint32 id)
   return 0;
 }
 
-Item *findItemById(const std::list<Item *> &l, guint32 id) 
-{
-  for (std::list<Item *>::const_iterator i = l.begin(), end = l.end(); i != end; ++i)
-    if ((*i)->getId() == id)
-      return *i;
-  
-  return 0;
-}
-
 
 // decoders
 
@@ -266,7 +286,7 @@ void NetworkPlayer::decodeActionMove(const Action_Move *action)
 	for (int y = -1; y <= 1; y++)
 	  {
 	    Vector<int> dest = action->getEndingPosition() + Vector<int>(x,y);
-	    Stack *s = d_stacklist->getOwnObjectAt(dest);
+	    Stack *s = GameMap::getFriendlyStack(dest);
 	    printf ("stack at position %d,%d is %p\n", dest.x, dest.y, s);
 	    if (s)
 	      printf ("stack id is %d\n", s->getId());
@@ -284,28 +304,12 @@ void NetworkPlayer::decodeActionSplit(const Action_Split *action)
   Stack *stack = d_stacklist->getStackById(stack_id);
   assert (stack != NULL);
   
-  for (Stack::iterator it = stack->begin(); it != stack->end(); it++)
-    (*it)->setGrouped(true);
-  for (unsigned int i = 0; i < MAX_STACK_SIZE; ++i) {
-    guint32 army_id = action->getGroupedArmyId(i);
-    if (army_id == 0)
-      continue;
+  std::list<guint32> armies;
+  for (unsigned int i = 0; i < MAX_STACK_SIZE; ++i) 
+    armies.push_back(action->getGroupedArmyId(i));
 
-    Army *army = stack->getArmyById(army_id);
-    if (army == NULL)
-      {
-	printf ("couldn't find army with id %d in stack %d\n", army_id, stack_id);
-	printf ("available IDs are...\n");
-	for (Stack::iterator it = stack->begin(); it != stack->end(); it++)
-	  printf ("%d ", (*it)->getId());
-	printf ("\n");
-	exit(0);
-      }
-    assert (army != NULL);
-    army->setGrouped(false);
-  }
-
-  Stack *new_stack = doStackSplit(stack);
+  Stack *new_stack = NULL;
+  doStackSplitArmies(stack, armies, new_stack);
   assert (new_stack != NULL);
   if (new_stack->getId() != action->getNewStackId())
     {
@@ -350,7 +354,7 @@ void NetworkPlayer::decodeActionJoin(const Action_Join *action)
 
   assert (receiver != NULL);
   assert (joining != NULL);
-  doStackJoin(receiver, joining, false);
+  doStackJoin(receiver, joining);
   receiver->sortForViewing(true);
   supdatingStack.emit(0);
 }
@@ -389,6 +393,7 @@ void NetworkPlayer::decodeActionRuin(const Action_Ruin *action)
 
   r->setSearched(searched);
 
+  //the reward is given to the player via the decodeActionReward method.
   supdatingStack.emit(0);
 }
 
@@ -512,11 +517,11 @@ void NetworkPlayer::decodeActionEquip(const Action_Equip *action)
 void NetworkPlayer::decodeActionLevel(const Action_Level *action)
 {
   Stack *stack = d_stacklist->getArmyStackById(action->getArmyId());
-  Army *army = stack->getArmyById(action->getArmyId());
+  Hero*hero= dynamic_cast<Hero*>(stack->getArmyById(action->getArmyId()));
 
-  doLevelArmy(army, Army::Stat(action->getStatToIncrease()));
-  printf ("army is hero? %d\n", army->isHero());
-  printf ("new level is %d\n", army->getLevel());
+  doHeroGainsLevel(hero, Army::Stat(action->getStatToIncrease()));
+  printf ("army is hero? %d\n", hero->isHero());
+  printf ("new level is %d\n", hero->getLevel());
 }
 
 void NetworkPlayer::decodeActionDisband(const Action_Disband *action)
@@ -581,6 +586,8 @@ void NetworkPlayer::decodeActionProduce(const Action_Produce *action)
   Army *army = new Army (*a, this);
   Stack *s = c->addArmy(army);
   printf ("created army id %d, in stack %d of size %d\n", army->getId(), s->getId(), s->size());
+  assert (s->getPos() == action->getDestination());
+  assert (army->getId() == action->getArmyId());
   s->sortForViewing(true);
 }
 
