@@ -111,7 +111,7 @@ bool Path::save(XML_Helper* helper) const
     return retval;
 }
 
-bool Path::checkPath(Stack* s)
+bool Path::checkPath(Stack* s, int enemy_city_avoidance, int enemy_stack_avoidance)
 {
     if (empty())
         return true;
@@ -122,7 +122,8 @@ bool Path::checkPath(Stack* s)
 	secondlast--;
 	for (iterator it = begin(); it != secondlast; it++)
 	  {
-	    if (PathCalculator::isBlocked(s, *it) == false)
+	    if (PathCalculator::isBlocked(s, *it, enemy_city_avoidance, 
+					  enemy_stack_avoidance) == false)
 	      {
 		valid = false;
 		break;
@@ -176,7 +177,7 @@ guint32 Path::calculateToCity (Stack *s, City *c, bool zigzag)
 	if (checkJoin == true)
 	  {
 	    Stack *other_stack = GameMap::getStack(c->getPos() + Vector<int>(i,j));
-	    if (other_stack && s->canJoin(other_stack) == false)
+	    if (other_stack && GameMap::canJoin(s,other_stack) == false)
 	      continue;
 	  }
 	int distance = dist (s->getPos(), c->getPos() + Vector<int>(i, j));
@@ -200,7 +201,7 @@ guint32 Path::calculateToCity (Stack *s, City *c, bool zigzag)
 	    if (checkJoin == true)
 	      {
 		Stack *other_stack = GameMap::getStack(c->getPos() + Vector<int>(i,j));
-		if (other_stack && s->canJoin(other_stack) == false)
+		if (other_stack && GameMap::canJoin(s,other_stack) == false)
 		  continue;
 	      }
 	    int dist = calculate(s, c->getPos() + Vector<int>(i, j), zigzag);
@@ -245,7 +246,19 @@ void Path::calculate (Stack* s, Vector<int> dest, guint32 &moves, guint32 &turns
   // Finally, all that is left is finding the minimum distance way from start
   // point to destination.
 
-  PathCalculator pc = PathCalculator(s, zigzag);
+    
+  int enemy_city_avoidance = -1;
+  int enemy_stack_avoidance = -1;
+  if (s->getOwner() && s->getOwner()->isComputer())
+    {
+      //If we're a computer player we don't let enemy stacks and cities
+      //prevent us from reaching our destination.  When we encounter them
+      //we'll fight, but we try not to encounter them.
+      enemy_city_avoidance = 10;
+      enemy_stack_avoidance = 10;
+    }
+  PathCalculator pc = PathCalculator(s, zigzag, enemy_city_avoidance, 
+				     enemy_stack_avoidance);
 
   Path *calculated_path = pc.calculate(dest, moves, turns, zigzag);
   if (calculated_path->size())
@@ -256,7 +269,7 @@ void Path::calculate (Stack* s, Vector<int> dest, guint32 &moves, guint32 &turns
 
   //calculate when the waypoints show no more movement possible
   guint32 pathcount = 0;
-  guint32 moves_left = s->getGroupMoves();
+  guint32 moves_left = s->getMoves();
   for (iterator it = begin(); it != end(); it++)
     {
       guint32 moves = s->calculateTileMovementCost(*it);
