@@ -52,6 +52,7 @@ using namespace std;
 std::string Playerlist::d_tag = "playerlist";
 Playerlist* Playerlist::s_instance = 0;
 Player* Playerlist::d_activeplayer = 0;
+Player* Playerlist::viewingplayer = 0;
 
 Playerlist* Playerlist::getInstance()
 {
@@ -84,12 +85,14 @@ Playerlist::Playerlist()
     :d_neutral(0)
 {
     d_activeplayer = 0;
+    viewingplayer = 0;
 }
 
 Playerlist::Playerlist(XML_Helper* helper)
     :d_neutral(0)
 {
     d_activeplayer=0;
+    viewingplayer=0;
     //load data. This consists currently of two values: size (for checking
     //the size; we don't use it yet) and active(which player is active)
     //we do it by calling load with playerlist as string
@@ -181,6 +184,7 @@ void Playerlist::nextPlayer()
     }
 
     d_activeplayer = (*it);
+    viewingplayer = (*it);
     debug("got player: " <<d_activeplayer->getName())
 }
 
@@ -196,14 +200,11 @@ Player* Playerlist::getPlayer(string name) const
 
 Player* Playerlist::getPlayer(guint32 id) const
 {
-    debug("getPlayer()");
-    for (const_iterator it = begin(); it != end(); it++)
-    {
-        if (id == (*it)->getId())
-            return (*it);
-    }
+  IdMap::const_iterator it = d_id.find(id);
 
-    return 0;
+  if (it == d_id.end())
+    return NULL;
+  return  (*it).second;
 }
 
 guint32 Playerlist::getNoOfPlayers() const
@@ -246,6 +247,11 @@ bool Playerlist::save(XML_Helper* helper) const
     return retval;
 }
 
+void Playerlist::add(Player *player)
+{
+  push_back(player);
+  d_id[player->getId()] = player;
+}
 bool Playerlist::load(string tag, XML_Helper* helper)
 {
     static guint32 active = 0;
@@ -266,13 +272,15 @@ bool Playerlist::load(string tag, XML_Helper* helper)
         return false;
 
     //insert player...
-    push_back(p);
+    add(p);
 
     //set neutral and active
     if (p->getId() == neutral)
         d_neutral = p;
     if (p->getId() == active)
-        d_activeplayer = p;
+      d_activeplayer = p;
+
+    viewingplayer = d_activeplayer;
 
     return true;
 }
@@ -433,7 +441,7 @@ void Playerlist::calculateWinners()
     return;
 }
 
-guint32 Playerlist::countHumanPlayersAlive()
+guint32 Playerlist::countHumanPlayersAlive() const
 {
   guint32 retval = 0;
   for (const_iterator it = begin(); it != end(); it++)
@@ -442,7 +450,7 @@ guint32 Playerlist::countHumanPlayersAlive()
   return retval;
 }
 
-guint32 Playerlist::countPlayersAlive ()
+guint32 Playerlist::countPlayersAlive () const
 {
   guint32 numAlive = 0; 
 
@@ -531,11 +539,15 @@ void Playerlist::swap(Player *old_player, Player *new_player)
   rlist->changeOwnership (old_player, new_player);
   VectoredUnitlist *vlist = VectoredUnitlist::getInstance();
   vlist->changeOwnership (old_player, new_player);
+  AI_Analysis::changeOwnership(old_player, new_player);
   if (old_player == d_activeplayer)
     {
       d_activeplayer = new_player;
-      d_activeplayer->getStacklist()->setActivestack(0);
+      d_activeplayer->setActivestack(0);
     }
+  if (old_player == viewingplayer)
+    viewingplayer = new_player;
+  d_id[new_player->getId()] = new_player;
   /* note, we don't have to change the player associated with flag graphics
      because it's stored as an id. */
 }
@@ -564,6 +576,7 @@ void Playerlist::randomizeOrder()
 {
   sort(randomly);
   d_activeplayer = NULL;
+  viewingplayer = NULL;
 }
 
 void Playerlist::nextRound(bool diplomacy, bool *surrender_already_offered)
@@ -638,10 +651,11 @@ void Playerlist::syncPlayer(GameParameters::Player player)
       if (p)
 	{
 	  p->setGold(gold);
-	  push_back(p);
+	  add(p);
 
 	  sort(inOrderOfId);
 	  d_activeplayer = getFirstLiving();
+	  viewingplayer = d_activeplayer;
 	}
       return;
     }
@@ -691,6 +705,7 @@ void Playerlist::syncPlayer(GameParameters::Player player)
 
   sort(inOrderOfId);
   d_activeplayer = getFirstLiving();
+  viewingplayer = d_activeplayer;
   return;
 }
 
@@ -810,12 +825,13 @@ void Playerlist::reorder(std::list<guint32> order)
   sort(inGivenOrder);
   given_turn_order.clear();
   d_activeplayer = getFirstLiving();
+  viewingplayer = d_activeplayer;
 }
 
-std::list<History *>Playerlist::getHistoryForHeroId(guint32 id)
+std::list<History *>Playerlist::getHistoryForHeroId(guint32 id) const
 {
   std::list<History*> no_events;
-  for (iterator it = begin(); it != end(); ++it)
+  for (const_iterator it = begin(); it != end(); ++it)
     {
       std::list<History *>events = (*it)->getHistoryForHeroId(id);
       if (events.size() > 0)
@@ -836,14 +852,14 @@ void Playerlist::surrender()
     }
 }
 
-bool Playerlist::isEndOfRound()
+bool Playerlist::isEndOfRound() const
 {
   //check to see if all players have moved this round.
   //do all players have the same number of history:end_turn events?
   if (d_activeplayer == NULL)
     return false;
   guint32 count = d_activeplayer->countEndTurnHistoryEntries();
-  for (iterator it = begin(); it != end(); it++)
+  for (const_iterator it = begin(); it != end(); it++)
     {
       if (*it == d_activeplayer)
 	continue;
@@ -852,8 +868,45 @@ bool Playerlist::isEndOfRound()
     }
   return true;
 }
+
 void Playerlist::setWinningPlayer(Player *winner)
 {
   //only for humans
   d_activeplayer = winner;
 }
+
+int Playerlist::countFightsThisTurn() const
+{
+  int count = 0;
+  for (const_iterator it = begin(); it != end(); it++)
+    count += (*it)->countFightsThisTurn();
+  return count;
+}
+
+int Playerlist::countMovesThisTurn() const
+{
+  int count = 0;
+  for (const_iterator it = begin(); it != end(); it++)
+    count += (*it)->countMovesThisTurn();
+  return count;
+}
+
+Player *Playerlist::getWinningPlayer() const
+{
+  guint32 best_score = 0;
+  Player *winning_player = NULL;
+  for (const_iterator it = begin(); it != end(); it++)
+    {
+      Player *p = (*it);
+      if (p->isDead() == false)
+        continue;
+      if (p == getNeutral())
+        continue;
+      if (p->getScore() >= best_score)
+        {
+          best_score = p->getScore();
+          winning_player = p;
+        }
+    }
+  return winning_player;
+}
