@@ -33,6 +33,10 @@
 class MoveResult;
 class XML_Helper;
 class City;
+class HeroProto;
+class Sage;
+class Ruin;
+class Stack;
 
 //! A local human Player.
 /** 
@@ -46,25 +50,46 @@ class City;
 class RealPlayer : public Player
 {
     public:
-        // CREATORS
+
+	//! Default constructor.
         RealPlayer(std::string name, guint32 armyset, Gdk::Color color, 
 		   int width, int height, Player::Type type = Player::HUMAN, 
 		   int player_no = -1);
+
+	//! Copy constructor.
         RealPlayer(const Player&);
+
+	//! Loading constructor.
         RealPlayer(XML_Helper* helper);
-        ~RealPlayer();
+
+	//! Destructor.
+        virtual ~RealPlayer();
+
+	virtual bool isComputer() const {return false;};
 
         virtual bool save(XML_Helper* helper) const;
 
 	virtual void abortTurn();
+
         virtual bool startTurn();
+
         virtual void endTurn();
+
         virtual void invadeCity(City* c);
-        virtual void levelArmy(Army* a);
+
+        virtual bool chooseHero(HeroProto *hero, City* c, int gold);
+
+        virtual Reward *chooseReward(Ruin *ruin, Sage *sage, Stack *stack);
+
+        virtual void heroGainsLevel(Hero * a);
+
+	virtual bool chooseTreachery (Stack *stack, Player *player, Vector <int> pos);
+        virtual Army::Stat chooseStat(Hero *hero);
+        
+        virtual bool chooseQuest(Hero *hero);
 
 	bool d_abort_requested;
-    protected:
-        bool maybeRecruitHero();
+
 };
 
 #endif // REAL_PLAYER_H
