@@ -100,7 +100,7 @@ Reward_Gold::Reward_Gold (const Reward_Gold & orig)
 {
 }
 
-bool Reward_Gold::save(XML_Helper* helper)
+bool Reward_Gold::save(XML_Helper* helper) const
 {
   bool retval = true;
   retval &= helper->openTag(Reward::d_tag);
@@ -154,7 +154,7 @@ Reward_Allies::Reward_Allies (const Reward_Allies& orig)
 {
 }
 
-bool Reward_Allies::save(XML_Helper* helper)
+bool Reward_Allies::save(XML_Helper* helper) const
 {
   bool retval = true;
   retval &= helper->openTag(Reward::d_tag);
@@ -170,29 +170,34 @@ bool Reward_Allies::save(XML_Helper* helper)
 	
 const guint32 Reward_Allies::getRandomAmountOfAllies()
 {
-  return (rand() % MAX_STACK_SIZE) + 1;
+  int percent = rand() % 100;
+  if (percent < 30)
+    return 1;
+  else if (percent < 50)
+    return 2;
+  else if (percent < 70)
+    return 3;
+  else if (percent < 80)
+    return 4;
+  else if (percent < 85)
+    return 5;
+  else if (percent < 90)
+    return 6;
+  else if (percent < 95)
+    return 7;
+  else if (percent < 100)
+    return 8;
+  else
+    return 1;
 }
 
 const ArmyProto* Reward_Allies::randomArmyAlly()
 {
-  guint32 allytype;
-  // list all the army types that can be allies.
-  std::vector<const ArmyProto*> allytypes;
   Armysetlist *al = Armysetlist::getInstance();
   Player *p = Playerlist::getInstance()->getActiveplayer();
   if (!p)
     p = Playerlist::getInstance()->getNeutral();
-  for (unsigned int j = 0; j < al->getSize(p->getArmyset()); j++)
-    {
-      const ArmyProto *a = al->getArmy (p->getArmyset(), j);
-      if (a->getAwardable())
-        allytypes.push_back(a);
-    }
-  if (allytypes.empty())
-    return NULL;
-
-  allytype = rand() % allytypes.size();
-  return allytypes[allytype];
+  return al->getArmyset(p->getArmyset())->getRandomAwardableAlly();
 }
 
 bool Reward_Allies::addAllies(Player *p, Vector<int> pos, const ArmyProto *army, guint32 alliesCount)
@@ -201,7 +206,7 @@ bool Reward_Allies::addAllies(Player *p, Vector<int> pos, const ArmyProto *army,
     {
       Army* ally = new Army(*army, p);
       ally->setUpkeep(0);
-      if (GameMap::getInstance()->addArmy(pos, ally) == NULL)
+      if (GameMap::getInstance()->addArmyAtPos(pos, ally) == NULL)
         return false;
     }
   return true;
@@ -251,7 +256,7 @@ Reward_Item::Reward_Item (const Reward_Item& orig)
 {
 }
 
-bool Reward_Item::save(XML_Helper* helper)
+bool Reward_Item::save(XML_Helper* helper) const
 {
   bool retval = true;
   retval &= helper->openTag(Reward::d_tag);
@@ -297,7 +302,7 @@ Reward_Ruin::Reward_Ruin (const Reward_Ruin& orig)
 	:Reward(orig), d_ruin_pos(orig.d_ruin_pos)
 {
 }
-bool Reward_Ruin::save(XML_Helper* helper)
+bool Reward_Ruin::save(XML_Helper* helper) const
 {
   bool retval = true;
   retval &= helper->openTag(Reward::d_tag);
@@ -382,7 +387,7 @@ Reward_Map::Reward_Map (const Reward_Map& orig)
   d_sightmap = new SightMap(*orig.d_sightmap);
 }
 
-bool Reward_Map::save(XML_Helper* helper)
+bool Reward_Map::save(XML_Helper* helper) const
 {
   bool retval = true;
   retval &= helper->openTag(Reward::d_tag);
@@ -410,21 +415,21 @@ Reward_Map::~Reward_Map()
     delete d_sightmap;
 }
 
-std::string Reward::getDescription()
+std::string Reward::getDescription() const
 {
   Glib::ustring s = "";
   switch (getType())
     {
     case Reward::GOLD:
 	{
-	  Reward_Gold *g = dynamic_cast<Reward_Gold*>(this);
+	  const Reward_Gold *g = dynamic_cast<const Reward_Gold*>(this);
 	  s += String::ucompose(ngettext("%1 Gold Piece", "%1 Gold Pieces", 
 					 g->getGold()), g->getGold());
 	  return s;
 	}
     case Reward::ALLIES:
 	{
-	  Reward_Allies *a = dynamic_cast<Reward_Allies *>(this);
+	  const Reward_Allies *a = dynamic_cast<const Reward_Allies *>(this);
 	  if (a->getArmy())
 	    s += String::ucompose(_("Allies: %1 x %2"), a->getArmy()->getName(),
 				  a->getNoOfAllies());
@@ -432,21 +437,21 @@ std::string Reward::getDescription()
 	}
     case Reward::ITEM:
 	{
-	  Reward_Item *i = dynamic_cast<Reward_Item *>(this);
+	  const Reward_Item *i = dynamic_cast<const Reward_Item *>(this);
 	  if (i->getItem())
 	    s += String::ucompose(_("Item: %1"), i->getItem()->getName());
 	  return s;
 	}
     case Reward::RUIN:
 	{
-	  Reward_Ruin *r = dynamic_cast<Reward_Ruin *>(this);
+	  const Reward_Ruin *r = dynamic_cast<const Reward_Ruin *>(this);
 	  if (r->getRuin())
 	    s += String::ucompose(_("Site: %1"), r->getRuin()->getName());
 	  return s;
 	}
     case Reward::MAP:
 	{
-	  Reward_Map *m = dynamic_cast<Reward_Map *>(this);
+	  const Reward_Map *m = dynamic_cast<const Reward_Map *>(this);
 	  s += String::ucompose(_("Map: %1,%2 %3x%4"), 
 				  m->getLocation().x,
 				  m->getLocation().y,
