@@ -22,11 +22,12 @@
 #include "GameMap.h"
 #include "rewardlist.h"
 #include <stdlib.h>
+#include "Sage.h"
 
 std::string Ruin::d_tag = "ruin";
 
-Ruin::Ruin(Vector<int> pos, std::string name, int type, Stack* occupant, bool searched, bool hidden, Player *owner, bool sage)
-:NamedLocation(pos, Ruin::getWidth(), name,
+Ruin::Ruin(Vector<int> pos, guint32 width, std::string name, int type, Stack* occupant, bool searched, bool hidden, Player *owner, bool sage)
+:NamedLocation(pos, width, name,
 	        name + _(" is inhabited by monsters and full of treasure!")), 
     d_searched(searched), 
     d_type(type), d_occupant(occupant), d_hidden(hidden), d_owner(owner), 
@@ -50,8 +51,15 @@ Ruin::Ruin(const Ruin& ruin)
 {
 }
 
-Ruin::Ruin(XML_Helper* helper)
-    :NamedLocation(helper, Ruin::getWidth()), d_type(0), d_occupant(0), 
+Ruin::Ruin(const Ruin& ruin, Vector<int> pos)
+    :NamedLocation(ruin, pos), d_searched(ruin.d_searched), 
+    d_type(ruin.d_type), d_occupant(ruin.d_occupant), d_hidden(ruin.d_hidden), 
+    d_owner(ruin.d_owner), d_sage(ruin.d_sage), d_reward(ruin.d_reward)
+{
+}
+
+Ruin::Ruin(XML_Helper* helper, guint32 width)
+    :NamedLocation(helper, width), d_type(0), d_occupant(0), 
     d_hidden(0), d_owner(0), d_sage(0), d_reward(0)
 {
     guint32 ui;
@@ -61,7 +69,7 @@ Ruin::Ruin(XML_Helper* helper)
     helper->getData(d_searched, "searched");
     helper->getData(d_sage, "sage");
     helper->getData(d_hidden, "hidden");
-    if (d_hidden)
+    if (d_hidden || d_searched)
       {
         helper->getData(ui, "owner");
 	if (ui != MAX_PLAYERS)
@@ -199,4 +207,9 @@ Ruin::Type Ruin::ruinTypeFromString(const std::string str)
     return Ruin::STRONGHOLD;
   return Ruin::RUIN;
 }
+
+Sage* Ruin::generateSage() const
+{
+  return new Sage();
+}
 // End of file
