@@ -23,7 +23,7 @@
 #include "GraphicsCache.h"
 #include "xmlhelper.h"
 #include "ucompose.hpp"
-#include "rgb_shift.h"
+#include "shieldset.h"
 
 std::string Shield::d_tag = "shield";
 
@@ -33,14 +33,13 @@ std::string Shield::d_tag = "shield";
 Shield::Shield(XML_Helper* helper)
 {
   helper->getData(d_owner, "owner");
-  std::string s;
-  helper->getData(s, "color");
-  guint32 r,g,b;
-  std::istringstream scolor(s);
-  scolor >> r;
-  scolor >> g;
-  scolor >> b;
-  d_color.set_rgb_p(r/255.0,g/255.0,b/255.0);
+  helper->getData(d_color, "color");
+}
+	
+Shield::Shield(Shield::Colour owner, Gdk::Color color)
+{
+  d_owner = guint32(owner);
+  d_color = color;
 }
 
 Shield::~Shield()
@@ -76,62 +75,6 @@ Gdk::Color Shield::get_default_color_for_neutral()
   return color;
 }
 
-struct rgb_shift Shield::getMaskColorShifts()
-{
-    // This is a bit tricky. The color values we return here encode additional
-    // shifts that are performed when getting the color. I.e. a color value for
-    // red of 8 means that the red color is completely ignored.
-
-    // For each color component, find the n where 2^n best describes the color.
-    // The mask value then is (8-n).
-    struct rgb_shift shifts;
-
-    guint32 r,g,b;
-    guint8 target_red = guint8(d_color.get_red_p() * 255.0);
-    for (int i = 8, diff = 257; i > 0; i--)
-    {
-        int color = 1<<i;
-        int tmp_diff = abs(target_red - color);
-
-	r = 8 - (i + 1);
-
-        if (diff < tmp_diff)
-            break;
-        else
-            diff = tmp_diff;
-    }
-        
-    guint8 target_green = guint8(d_color.get_green_p() * 255.0);
-    for (int i = 8, diff = 257; i > 0; i--)
-    {
-        int color = 1<<i;
-        int tmp_diff = abs(target_green - color);
-	g = 8 - (i + 1);
-
-        if (diff < tmp_diff)
-            break;
-        else
-            diff = tmp_diff;
-    }
-        
-    guint8 target_blue = guint8(d_color.get_blue_p() * 255.0);
-    for (int i = 8, diff = 257; i > 0; i--)
-    {
-        int color = 1<<i;
-        int tmp_diff = abs(target_blue - color);
-	b = 8 - (i + 1);
-
-        if (diff < tmp_diff)
-            break;
-        else
-            diff = tmp_diff;
-    }
-    //c.set_rgb_p(r,g,b);
-    shifts.r = r;
-    shifts.g = g;
-    shifts.b = b;
-    return shifts;
-}
 std::string Shield::colourToString(const Shield::Colour c)
 {
   switch (c)
@@ -157,3 +100,40 @@ std::string Shield::colourToString(const Shield::Colour c)
     }
   return "Shield::NEUTRAL";
 }
+
+bool Shield::save(XML_Helper *helper) const
+{
+  bool retval = true;
+
+  retval &= helper->openTag(d_tag);
+  retval &= helper->saveData("owner", d_owner);
+  retval &= helper->saveData("color", d_color);
+  for (const_iterator it = begin(); it != end(); it++)
+    (*it)->save(helper);
+  retval &= helper->closeTag();
+  return retval;
+}
+	
+void Shield::instantiateImages(Shieldset *s)
+{
+  for (iterator it = begin(); it != end(); it++)
+    {
+      if ((*it)->getImageName().empty() == false)
+	(*it)->instantiateImages(s->getFile((*it)->getImageName()), s);
+    }
+}
+void Shield::uninstantiateImages()
+{
+  for (iterator it = begin(); it != end(); it++)
+    (*it)->uninstantiateImages();
+}
+
+ShieldStyle *Shield::getFirstShieldstyle(ShieldStyle::Type type)
+{
+  for (iterator i = begin(); i != end(); i++)
+    {
+      if ((*i)->getType() == type)
+	return *i;
+    }
+  return NULL;
+}
