@@ -26,10 +26,10 @@
 
 #include "xmlhelper.h"
 #include "shield.h"
+#include "set.h"
 
 #include "defs.h"
 
-struct rgbshift;
 //! A list of Shield graphic objects in a shield theme.
 /**
  * Every scenario has a shield set; it is the theme of the shield graphics 
@@ -48,15 +48,25 @@ struct rgbshift;
  *
  * The shieldset configuration file is a same named XML file inside the 
  * Shieldset's directory.  
- * E.g. shield/${Shieldset::d_dir}/${Shieldset::d_dir}.xml.
+ * E.g. shield/${Shieldset::d_subdir}/${Shieldset::d_subdir}.xml.
  */
-class Shieldset: public std::list<Shield *>, public sigc::trackable
+class Shieldset: public std::list<Shield *>, public sigc::trackable, public Set
 {
     public:
 
 	//! The xml tag of this object in a shieldset configuration file.
 	static std::string d_tag; 
 
+	//! The file extension for shieldset files.  It includes the dot.
+	static std::string file_extension;
+
+
+	//! Default constructor.
+	/**
+	 * Make a new shieldset given a unique id and a subdir name.
+	 */
+	Shieldset(guint32 id, std::string name);
+
 	//! Load a Shieldset from a shieldset configuration file.
 	/**
 	 * Make a new Shieldset object by reading it in from the shieldset
@@ -65,12 +75,26 @@ class Shieldset: public std::list<Shield *>, public sigc::trackable
 	 * @param helper  The opened shieldset configuration file to load the
 	 *                Shieldset from.
 	 */
-        Shieldset(XML_Helper* helper, bool private_collection = false);
+        Shieldset(XML_Helper* helper, std::string directory);
 
-	static Shieldset *create(std::string filename, bool private_collection = false);
 	//! Destructor.
         ~Shieldset();
 
+	// Get Methods
+
+	//! Get the unique identifier for this shieldset.
+	/**
+	 * Analagous to the shieldset.d_id XML entity in the shieldset
+	 * configuration file.
+	 */
+        guint32 getId() const {return d_id;}
+
+	//! Get the directory in which the shieldset configuration file resides.
+        std::string getSubDir() const {return d_subdir;}
+
+	//! Return the mask colour for the given player.
+	Gdk::Color getColor(guint32 owner) const;
+
 	//! Return the number of pixels high the small shields are.
 	guint32 getSmallHeight() const {return d_small_height;}
 
@@ -95,12 +119,17 @@ class Shieldset: public std::list<Shield *>, public sigc::trackable
 	//! Return the name of the Shieldset.
         std::string getName() const {return _(d_name.c_str());}
 
-	//! Get the unique identifier for this shieldset.
-	/**
-	 * Analagous to the shieldset.d_id XML entity in the shieldset
-	 * configuration file.
-	 */
-        guint32 getId() const {return d_id;}
+	//! Return the copyright holders of the shieldset.
+	std::string getCopyright() const {return d_copyright;};
+
+	//! Return the license of the shieldset.
+	std::string getLicense() const {return d_license;};
+
+        //! Returns the description of the shieldset.
+        std::string getInfo() const {return _(d_info.c_str());}
+
+
+	// Set Methods
 
 	//! Set the unique identifier for this shieldset.
         void setId(guint32 id) {d_id = id;}
@@ -108,11 +137,23 @@ class Shieldset: public std::list<Shield *>, public sigc::trackable
 	//! Set the name of the Shieldset.
         void setName(std::string name) {d_name = name;}
 
-	//! Get the directory in which the shieldset configuration file resides.
-        std::string getSubDir() const {return d_dir;}
+	//! Sets the description of the shieldset.
+	void setInfo(std::string description) {d_info = description;};
+
+	//! Set the copyright holders of the shieldset.
+	void setCopyright(std::string copy) {d_copyright = copy;};
+
+	//! Set the license of this shieldset.
+	void setLicense(std::string license) {d_license = license;};
 
 	//! Set the direction where the shieldset configuration file resides.
-        void setSubDir(std::string dir) {d_dir = dir;}
+        void setSubDir(std::string dir) {d_subdir = dir;}
+
+
+	// Methods that operate on the class data but do not modify the class.
+
+	bool save(XML_Helper *helper) const;
+
 
 	//! Find the shield of a given size and colour in this Shieldset.
 	/**
@@ -126,21 +167,53 @@ class Shieldset: public std::list<Shield *>, public sigc::trackable
 	 *         If no Shield object could be found that matches the given
 	 *         parameters, NULL is returned.
 	 */
-	ShieldStyle * lookupShieldByTypeAndColour(guint32 type, guint32 colour);
+	ShieldStyle * lookupShieldByTypeAndColour(guint32 type, guint32 colour) const;
 
-	Gdk::Color getColor(guint32 owner);
-	struct rgb_shift getMaskColorShifts(guint32 owner);
+	//! Get filenames in this shieldset, excepting the configuration file.
+	void getFilenames(std::list<std::string> &filenames) const;
 
-	//! Return whether this is a shieldset in the user's personal dir.
-	bool fromPrivateCollection() {return private_collection;};
+	//! Return the name of this shieldset's configuration file.
+	std::string getConfigurationFile() const;
+	
+	//! Check to see if this shieldset can be used in the game.
+	bool validate() const;
+
+	//! Check to see if the number of shields is sufficient.
+	bool validateNumberOfShields() const;
+
+	//! Check to see if the images for the shieldset are supplied.
+	bool validateShieldImages(Shield::Colour c) const;
+
+
+	// Methods that operate on the class data and also modify the class.
+
+	//! Load images associated with this shieldset.
+	void instantiateImages();
+
+	//! Destroy images associated with this shieldset.
+	void uninstantiateImages();
+
+
+	// Static Methods
+
+	//! Create a shieldset from the given shieldset configuration file.
+	static Shieldset *create(std::string filename);
+
+	//! Return a list of shieldset subdirs in the system collection.
+	static std::list<std::string> scanSystemCollection();
+
+	//! Return a list of shieldset subdirs in the users personal collection.
+	static std::list<std::string> scanUserCollection();
 
-	//! get filenames in this shieldset, excepting the configuration file.
-	void getFilenames(std::list<std::string> &filenames);
     private:
 
-        //! Callback function to load Shield objects into the Shieldset.
-        bool loadShield(std::string tag, XML_Helper* helper);
+	//! Callback function to load Shield objects into the Shieldset.
+	bool loadShield(std::string tag, XML_Helper* helper);
 
+	// DATA
+
+	//! A unique numeric identifier among all shieldset.
+	guint32 d_id;
 
 	//! The name of the Shieldset.
 	/**
@@ -151,8 +224,18 @@ class Shieldset: public std::list<Shield *>, public sigc::trackable
 	 */
         std::string d_name;
 
-	//! A unique numeric identifier among all shieldset.
-	guint32 d_id;
+	//! The copyright holders of the shieldset.
+	std::string d_copyright;
+
+	//! The license of the shieldset.
+	std::string d_license;
+
+	//! The description of the shieldset.
+	/**
+	 * Equates to the shieldset.d_info XML entity in the shieldset 
+	 * configuration file.
+	 */
+        std::string d_info;
 
 	//! The subdirectory of the Shieldset.
 	/**
@@ -160,7 +243,7 @@ class Shieldset: public std::list<Shield *>, public sigc::trackable
 	 * residing in.  It does not contain a path (e.g. no slashes).
 	 * Shieldset directories sit in the shield/ directory.
 	 */
-        std::string d_dir;
+        std::string d_subdir;
 
 	//! The number of pixels high the small shield occupies onscreen.
 	/**
@@ -203,9 +286,6 @@ class Shieldset: public std::list<Shield *>, public sigc::trackable
 	 * configuration file.
 	 */
 	guint32 d_large_width;
-
-	//! Whether this is a system shieldset, or one that the user made.
-	bool private_collection;
 };
 
 #endif // SHIELDSET_H
