@@ -2,7 +2,7 @@
 // Copyright (C) 2002, 2003, 2004, 2005, 2006 Ulf Lorenz
 // Copyright (C) 2004, 2005, 2006 Andrea Paternesi
 // Copyright (C) 2004 Thomas Plonka
-// Copyright (C) 2006, 2007, 2008, 2009 Ben Asselstine
+// Copyright (C) 2006, 2007, 2008, 2009, 2010 Ben Asselstine
 // Copyright (C) 2007 Ole Laursen
 //
 //  This program is free software; you can redistribute it and/or modify
@@ -38,6 +38,7 @@ SmallMap::SmallMap()
     input_locked = false;
     view.pos = Vector<int>(0, 0);
     view.dim = Vector<int>(3, 3);
+    sleep_interval = TIMER_SMALLMAP_REFRESH;
 }
 
 void SmallMap::set_view(Rectangle new_view)
@@ -45,7 +46,7 @@ void SmallMap::set_view(Rectangle new_view)
     if (view != new_view)
     {
 	view = new_view;
-	draw(Playerlist::getActiveplayer());
+	draw(Playerlist::getViewingplayer());
     }
 }
 
@@ -78,8 +79,8 @@ void SmallMap::draw_selection()
 
 void SmallMap::center_view_on_tile(Vector<int> pos, bool slide)
 {
-  pos = clip(Vector<int>(0,0), pos - view.dim / calculateResizeFactor(), 
-	     GameMap::get_dim() - view.dim);
+  pos = clip(Vector<int>(0,0), pos - view.dim / 2, 
+             GameMap::get_dim() - view.dim);
 
   if (slide)
     slide_view(Rectangle(pos.x, pos.y, view.w, view.h));
@@ -94,7 +95,7 @@ void SmallMap::center_view_on_pixel(Vector<int> pos, bool slide)
   pos.x = int(round(pos.x / pixels_per_tile));
   pos.y = int(round(pos.y / pixels_per_tile));
 
-  pos -= view.dim / calculateResizeFactor();
+  pos -= view.dim / pixels_per_tile;
 
   pos = clip(Vector<int>(0, 0), pos, GameMap::get_dim() - view.dim);
 
@@ -108,7 +109,14 @@ void SmallMap::center_view_on_pixel(Vector<int> pos, bool slide)
 
 void SmallMap::after_draw()
 {
-  Player *p = getViewingPlayer();
+  int width = 0, height = 0;
+  surface->get_size(width, height);
+  if (blank_screen == true)
+    {
+      map_changed.emit(surface, Gdk::Rectangle(0, 0, width, height));
+      return;
+    }
+  Player *p = Playerlist::getViewingplayer();
   OverviewMap::after_draw();
   if (p->getType() == Player::HUMAN ||
       GameScenarioOptions::s_hidden_map == false)
@@ -122,8 +130,6 @@ void SmallMap::after_draw()
       draw_cities(false);
       draw_selection();
     }
-    int width = 0, height = 0;
-    surface->get_size(width, height);
     map_changed.emit(surface, Gdk::Rectangle(0, 0, width, height));
 }
 
@@ -179,8 +185,9 @@ void SmallMap::slide_view(Rectangle new_view)
 	  tmp_view.y = slide(tmp_view.y, new_view.y);
 
 	  view = tmp_view;
-	  draw(Playerlist::getActiveplayer());
-	  view_slid.emit(view);
+	  draw(Playerlist::getViewingplayer());
+          view_slid.emit(view);
+          Glib::usleep(sleep_interval);
 
 	  if (tmp_view.x == new_view.x && tmp_view.y == new_view.y)
 	    break;
@@ -188,18 +195,6 @@ void SmallMap::slide_view(Rectangle new_view)
     }
 }
 
-void SmallMap::blank()
-{
-  Gdk::Color fog_color = Gdk::Color();
-  fog_color.set_rgb_p(0,0,0);
-  int width = 0;
-  int height = 0;
-  surface->get_size(width, height);
-  surface_gc->set_rgb_fg_color(fog_color);
-  surface->draw_rectangle(surface_gc, true, 0,0,width, height);
-  map_changed.emit(surface, Gdk::Rectangle(0, 0, width, height));
-}
-
 void SmallMap::move_map_in_dir(Vector<int> dir)
 {
   Rectangle new_view = view;
