@@ -24,6 +24,7 @@
 #include "templelist.h"
 #include "playerlist.h"
 #include "stack.h"
+#include "GameMap.h"
 
 //#define debug(x) {cerr<<__FILE__<<": "<<__LINE__<<": "<<x<<endl<<flush;}
 #define debug(x)
@@ -62,7 +63,7 @@ Templelist::Templelist()
 
 Templelist::Templelist(XML_Helper* helper)
 {
-    helper->registerTag(Temple::d_tag, sigc::mem_fun(this, &Templelist::load));
+  helper->registerTag(Temple::d_tag, sigc::mem_fun(this, &Templelist::load));
 }
 
 bool Templelist::save(XML_Helper* helper) const
@@ -85,17 +86,18 @@ bool Templelist::load(std::string tag, XML_Helper* helper)
     //what has happened?
         return false;
     
-    add(new Temple(helper));
+    guint32 width = GameMap::getInstance()->getCityset()->getTempleTileWidth();
+    add(new Temple(helper, width));
 
     return true;
 }
 
 static bool isFogged(void *t)
 {
-  return ((Temple*)t)->isVisible(Playerlist::getActiveplayer()) == false;
+  return ((Temple*)t)->isVisible(Playerlist::getViewingplayer()) == false;
 }
 
-Temple * Templelist::getNearestVisibleTemple(const Vector<int>& pos)
+Temple * Templelist::getNearestVisibleTemple(const Vector<int>& pos) const
 {
   std::list<bool (*)(void *)> filters;
   filters.push_back(isFogged);
@@ -103,13 +105,13 @@ Temple * Templelist::getNearestVisibleTemple(const Vector<int>& pos)
 }
 
 Temple* Templelist::getNearestVisibleAndUsefulTemple(Stack *s, 
-						     double percent_can_be_blessed)
+						     double percent_can_be_blessed) const
 {
   Vector<int> pos = s->getPos();
   int diff = -1;
-  iterator diffit;
+  const_iterator diffit;
 
-  for (iterator it = begin(); it != end(); ++it)
+  for (const_iterator it = begin(); it != end(); ++it)
     {
       Temple *temple = *it;
       if (isFogged(temple))
@@ -138,7 +140,7 @@ Temple* Templelist::getNearestVisibleAndUsefulTemple(Stack *s,
 
 Temple* Templelist::getNearestVisibleAndUsefulTemple(Stack *stack, 
 						     double percent_can_be_blessed, 
-						     int dist)
+						     int dist) const
 {
   Vector<int> pos = stack->getPos();
   Temple *t = getNearestVisibleAndUsefulTemple
@@ -151,9 +153,11 @@ Temple* Templelist::getNearestVisibleAndUsefulTemple(Stack *stack,
   return NULL;
 }
 
-Temple* Templelist::getNearestVisibleTemple(const Vector<int>& pos, int dist)
+Temple* Templelist::getNearestVisibleTemple(const Vector<int>& pos, int dist) const
 {
   Temple *t = getNearestVisibleTemple(pos);
+  if (!t)
+    return NULL;
   if (t->getPos().x <= pos.x + dist && t->getPos().x >= pos.x - dist &&
       t->getPos().y <= pos.y + dist && t->getPos().y >= pos.y - dist)
     return t;
