@@ -22,6 +22,7 @@
 
 #include "GraphicsCache.h"
 #include "xmlhelper.h"
+#include "gui/image-helpers.h"
 
 std::string TileStyleSet::d_tag = "tilestyleset";
 using namespace std;
@@ -44,29 +45,54 @@ TileStyleSet::~TileStyleSet()
     delete (*this)[i];
 }
 
-bool TileStyleSet::save(XML_Helper *helper)
+bool TileStyleSet::save(XML_Helper *helper) const
 {
   bool retval = true;
 
   retval &= helper->openTag(TileStyleSet::d_tag);
   retval &= helper->saveData("name", d_name);
-  for (TileStyleSet::iterator i = begin(); i != end(); ++i)
+  for (TileStyleSet::const_iterator i = begin(); i != end(); ++i)
     retval &= (*i)->save(helper);
   retval &= helper->closeTag();
 
   return retval;
 }
 
-void TileStyleSet::getUniqueTileStyleTypes(std::list<TileStyle::Type> &types)
+void TileStyleSet::getUniqueTileStyleTypes(std::list<TileStyle::Type> &types) const
 {
-  for (TileStyleSet::iterator i = begin(); i != end(); ++i)
+  for (TileStyleSet::const_iterator i = begin(); i != end(); ++i)
     if (find (types.begin(), types.end(), (*i)->getType()) == types.end())
       types.push_back((*i)->getType());
 }
 
-bool TileStyleSet::validate()
+bool TileStyleSet::validate() const
 {
+  if (getName().empty() == true)
+    return false;
   return true;
 }
 
+void TileStyleSet::uninstantiateImages()
+{
+  for (unsigned int i = 0; i < size(); i++)
+    {
+      if ((*this)[i]->getImage() != NULL)
+	{
+	  delete (*this)[i]->getImage();
+	  (*this)[i]->setImage(NULL);
+	}
+    }
+}
+void TileStyleSet::instantiateImages(int tilesize, std::string filename)
+{
+  if (filename.empty() == false)
+    {
+      std::vector<PixMask *> styles = disassemble_row(filename, size());
+      for (unsigned int i = 0; i < size(); i++)
+	{
+	  PixMask::scale(styles[i], tilesize, tilesize);
+	  (*this)[i]->setImage(styles[i]);
+	}
+    }
+}
 // End of file
